{
    "$graph": [
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function(ext) { var root = inputs.input_file.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.input_file.basename+ext:root+ext; };"
                    ]
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": "${\n  return  [\n            {\n              \"entry\": inputs.annotation_filename,\n              \"entryname\": inputs.annotation_filename.basename,\n              \"writable\": true\n            }\n          ]\n}\n"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/atdp:v0.0.1"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "File"
                    ],
                    "inputBinding": {
                        "position": 3,
                        "prefix": "--a=",
                        "separate": false
                    },
                    "doc": "Annotation file, tsv\n",
                    "id": "#atdp.cwl/annotation_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "--avd_heat_window=",
                        "separate": false
                    },
                    "doc": "Average tag density window for heatmap\n",
                    "id": "#atdp.cwl/avd_heat_window_bp"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--avd_smooth=",
                        "separate": false
                    },
                    "doc": "Average smooth window (odd), int [0]\n",
                    "id": "#atdp.cwl/avd_smooth_bp"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--avd_window=",
                        "separate": false
                    },
                    "doc": "Average tag density window, int [5000]\n",
                    "id": "#atdp.cwl/avd_window_bp"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--sam_twicechr=",
                        "separate": false
                    },
                    "doc": "The chromosomes to be doubled, string\n",
                    "id": "#atdp.cwl/double_chr"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--f=",
                        "separate": false
                    },
                    "doc": "Fragmentsize, int [150]\n",
                    "id": "#atdp.cwl/fragmentsize_bp"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "--sam_ignorechr=",
                        "separate": false
                    },
                    "doc": "The chromosomes to be ignored, string\n",
                    "id": "#atdp.cwl/ignore_chr"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "--index=",
                        "separate": false
                    },
                    "doc": "Index file\n",
                    "id": "#atdp.cwl/index_file"
                },
                {
                    "type": [
                        "File"
                    ],
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--in=",
                        "separate": false
                    },
                    "doc": "Input indexed BAM file (optionally +BAI index file in secondaryFiles)\n",
                    "id": "#atdp.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--log=",
                        "separate": false,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename('_atdp.log');\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Log filename\n",
                    "id": "#atdp.cwl/log_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "--m=",
                        "separate": false
                    },
                    "doc": "Mapped reads number\n",
                    "id": "#atdp.cwl/mapped_reads"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 4,
                        "prefix": "--out=",
                        "separate": false,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename('_atdp.tsv');\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Base output file name, tsv\n",
                    "id": "#atdp.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nset -- \"$0\" \"$@\"\necho \"Original arguments:\"\nfor i in \"$@\";\n    do echo $i;\ndone;\nset -- \"$1\" --a=$(basename \"${2:4}\") \"${@:3}\"\necho Updated arguments:\nfor i in \"$@\";\n    do echo $i;\ndone;\nrefgene-sort -i \"${2:4}\" -o \"${2:4}\" -s \"ORDER BY chrom, strand, CASE strand WHEN '+' THEN txStart WHEN '-' THEN txEnd END\"\natdp \"$@\"\n",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Bash function to run refgene-sort and atdp\n",
                    "id": "#atdp.cwl/script"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "Tool calculates average tag density profile around all annotated TSS.\n\n`default_output_filename` function returns output filename with sufix set as `ext` argument. Function is called when\neither `output_filename` or `log_filename` inputs are not provided.\n\nBefore running `baseCommand`, annotaion file `annotation_filename` is staged into output directory\n(Docker's `--workdir`) with `\"writable\": true` (to allow to overwrite it by `refgene-sort`).\n\nTo run `atdp` index file should be provided (either in `secondaryFiles` of `input_file` or as separate input\n`index_file`)\n\n`baseCommand` runs bash script from `script` input. Script runs `refgene-sort` to sort annotaion file and then runs\n`atdp`. `refgene-sort` - utility to sort refgene annotation files, using MySQL syntax.\n\nOptionally (with cwltool==1.0.20171107133715), script can be simplified to\n    #!/bin/bash\n    set -- \"$0\" \"$@\"\n    refgene-sort -i \"${2:4}\" -o \"${2:4}\" -s \"ORDER BY chrom, strand, CASE strand WHEN '+' THEN txStart WHEN '-' THEN txEnd END\"\n    atdp \"$@\"\nbecause `set -- \"$1\" --a=$(basename \"${2:4}\") \"${@:3}\"` is not needed anymore.\n",
            "id": "#atdp.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#atdp-metadata.yaml",
                "http://schema.org/name": "atdp",
                "http://schema.org/about": "Based on BioWardrobe atdp (https://github.com/cincinnati-childrens-hospital/biowardrobe/tree/master/src/atdp) The software is used to calculate average tag density profile around all annotated TSS. Such data can be used to estimate the success of ChIP-Seq type experiments for some histone modifications (e.g. H3K4me)\n",
                "http://schema.org/url": "https://github.com/Barski-lab/atdp/blob/master/README.md",
                "http://schema.org/codeRepository": "https://github.com/Barski-lab/atdp",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "0.0.1",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/discussionUrl": [
                    "https://github.com/Barski-lab/atdp/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Barski Research Lab",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Andrey Kartashov",
                                "http://schema.org/description": "Main author"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Michael Kotliar",
                                "http://schema.org/description": "Update to work with files instead of DB"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "atdp",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/atdp.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "id": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:\natdp [options] --in=pathToFile --a=pathtoFile --out=pathToFile\n  --a                        \tTab-separated annotation file\n  --avd_bsmooth              \tAverage smooth window for gene body\n  --avd_guid                 \tGenelist uid\n  --avd_heat_window          \tAverage tag density window for heatmap\n  --avd_smooth               \tAverage smooth window (odd)\n  --avd_window               \tAverage tag density window\n  --f                        \tFragmentsize, bp\n  --in                       \tInput BAM file\n  --index                    \tInput index bai file\n  --log                      \tLog file name (default is ./logfile_def.log)\n  --out                      \tBase output filename\n  --sam_ignorechr            \tWhich chromosome to ignore\n  --sam_twicechr             \tWhich chromosome to double\n  --m                         Mapped reads number\n",
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.log_filename == \"\"){\n    return default_output_filename('_atdp.log');\n  } else {\n    return inputs.log_filename;\n  }\n}\n"
                    },
                    "id": "#atdp.cwl/log_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_filename == \"\"){\n    return default_output_filename('_atdp.tsv');\n  } else {\n    return inputs.output_filename;\n  }\n}\n"
                    },
                    "id": "#atdp.cwl/result_file"
                }
            ]
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "label": "Input BAM file",
                    "doc": "Input BAM file, sorted by coordinates",
                    "id": "#bam-bedgraph-bigwig.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Output filename for generated bedGraph",
                    "id": "#bam-bedgraph-bigwig.cwl/bedgraph_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Output filename for generated bigWig",
                    "id": "#bam-bedgraph-bigwig.cwl/bigwig_filename"
                },
                {
                    "type": "File",
                    "doc": "Tab delimited chromosome length file: <chromName><TAB><chromSize>",
                    "id": "#bam-bedgraph-bigwig.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Change strand af the mate read, so both reads come from the same strand",
                    "id": "#bam-bedgraph-bigwig.cwl/dutp"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Set fixed fragment size for genome coverage calculation",
                    "id": "#bam-bedgraph-bigwig.cwl/fragment_size"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Parameter to calculate scale as 1000000/mapped_reads_number. Ignored by bedtools-genomecov.cwl in\nbam_to_bedgraph step if scale is provided\n",
                    "id": "#bam-bedgraph-bigwig.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Enable paired-end genome coverage calculation",
                    "id": "#bam-bedgraph-bigwig.cwl/pairchip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "doc": "Coefficient to scale the genome coverage by a constant factor",
                    "id": "#bam-bedgraph-bigwig.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Calculate genome coverage for each part of the splitted by 'N' and 'D' read",
                    "id": "#bam-bedgraph-bigwig.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Calculate genome coverage of intervals from a specific strand",
                    "id": "#bam-bedgraph-bigwig.cwl/strand"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputSource": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file",
                    "label": "bedGraph output file",
                    "doc": "Output bedGraph file",
                    "id": "#bam-bedgraph-bigwig.cwl/bedgraph_file"
                },
                {
                    "type": "File",
                    "outputSource": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bigwig_file",
                    "doc": "Output bigWig file",
                    "id": "#bam-bedgraph-bigwig.cwl/bigwig_file"
                }
            ],
            "steps": [
                {
                    "run": "#bedtools-genomecov.cwl",
                    "in": [
                        {
                            "default": "-bg",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/depth"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/dutp",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/du"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/fragment_size",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/fragment_size"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bam_file",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/input_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/mapped_reads_number",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/mapped_reads_number"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bedgraph_filename",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/output_filename"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/pairchip",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/pairchip"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/scale",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/scale"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/split",
                            "valueFrom": "${\n  if (self == null){\n    return true;\n  } else {\n    return self;\n  }\n}\n",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/split"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/strand",
                            "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/strand"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/genome_coverage_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph"
                },
                {
                    "run": "#linux-sort.cwl",
                    "in": [
                        {
                            "default": [
                                "1,1",
                                "2,2n"
                            ],
                            "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/key"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bam_to_bedgraph/genome_coverage_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/unsorted_file"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/sort_bedgraph"
                },
                {
                    "run": "#ucsc-bedgraphtobigwig.cwl",
                    "in": [
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/sort_bedgraph/sorted_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bedgraph_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/chrom_length_file",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/chrom_length_file"
                        },
                        {
                            "source": "#bam-bedgraph-bigwig.cwl/bigwig_filename",
                            "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/output_filename"
                        }
                    ],
                    "out": [
                        "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig/bigwig_file"
                    ],
                    "id": "#bam-bedgraph-bigwig.cwl/sorted_bedgraph_to_bigwig"
                }
            ],
            "doc": "Workflow converts input BAM file into bigWig and bedGraph files.\n\nInput BAM file should be sorted by coordinates (required by `bam_to_bedgraph` step).\n\nIf `split` input is not provided use true by default. Default logic is implemented in `valueFrom` field of `split`\ninput inside `bam_to_bedgraph` step to avoid possible bug in cwltool with setting default values for workflow inputs.\n\n`scale` has higher priority over the `mapped_reads_number`. The last one is used to calculate `-scale` parameter for\n`bedtools genomecov` (step `bam_to_bedgraph`) only in a case when input `scale` is not provided. All logic is\nimplemented inside `bedtools-genomecov.cwl`.\n\n`bigwig_filename` defines the output name only for generated bigWig file. `bedgraph_filename` defines the output name\nfor generated bedGraph file and can influence on generated bigWig filename in case when `bigwig_filename` is not provided.\n\nAll workflow inputs and outputs don't have `format` field to avoid format incompatibility errors when workflow is used\nas subworkflow.",
            "id": "#bam-bedgraph-bigwig.cwl",
            "http://schema.org/name": "bam-bedgraph-bigwig",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "id": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { var ext = (inputs.depth == \"-bg\" || inputs.depth == \"-bga\")?\".bedGraph\":\".tab\"; return inputs.input_file.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.') + ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bedtools2:v2.26.0"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 17,
                        "prefix": "-g"
                    },
                    "doc": "Input genome file. Needed only when -i flag. The genome file is tab delimited <chromName><TAB><chromSize>\n",
                    "id": "#bedtools-genomecov.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "enum",
                            "name": "#bedtools-genomecov.cwl/depth/depth",
                            "symbols": [
                                "#bedtools-genomecov.cwl/depth/depth/-bg",
                                "#bedtools-genomecov.cwl/depth/depth/-bga",
                                "#bedtools-genomecov.cwl/depth/depth/-d",
                                "#bedtools-genomecov.cwl/depth/depth/-dz"
                            ]
                        }
                    ],
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Report the depth type. By default, bedtools genomecov will compute a histogram of coverage\nfor the genome file provided (intputs.chrom_length_file)\n",
                    "id": "#bedtools-genomecov.cwl/depth"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-du"
                    },
                    "doc": "Change strand af the mate read (so both reads from the same strand) useful for strand specific.\nWorks for BAM files only\n",
                    "id": "#bedtools-genomecov.cwl/du"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-fs"
                    },
                    "doc": "Set fixed fragment size\n",
                    "id": "#bedtools-genomecov.cwl/fragment_size"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 16,
                        "valueFrom": "${\n  var prefix = ((/.*\\.bam$/i).test(inputs.input_file.path))?'-ibam':'-i';\n  return [prefix, inputs.input_file.path];\n}\n"
                    },
                    "doc": "The input file can be in BAM format (Note: BAM must be sorted by position) or <bed/gff/vcf>.\nPrefix is selected on the base of input file extension\n",
                    "id": "#bedtools-genomecov.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "-3"
                    },
                    "doc": "Calculate coverage of 3\" positions (instead of entire interval)\n",
                    "id": "#bedtools-genomecov.cwl/m3"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "-5"
                    },
                    "doc": "Calculate coverage of 5\" positions (instead of entire interval)\n",
                    "id": "#bedtools-genomecov.cwl/m5"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-scale",
                        "valueFrom": "${\n  if (inputs.scale){\n    return null;\n  } else if (inputs.mapped_reads_number) {\n    return 1000000/inputs.mapped_reads_number;\n  } else {\n    return null;\n  }\n}\n"
                    },
                    "doc": "Optional parameter to calculate scale as 1000000/mapped_reads_number if inputs.scale is not provided\n",
                    "id": "#bedtools-genomecov.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "-max"
                    },
                    "doc": "Combine all positions with a depth >= max into\na single bin in the histogram. Irrelevant\nfor -d and -bedGraph\n- (INTEGER)\n",
                    "id": "#bedtools-genomecov.cwl/max"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Name for generated output file\n",
                    "id": "#bedtools-genomecov.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-pc"
                    },
                    "doc": "pair-end chip seq experiment\n",
                    "id": "#bedtools-genomecov.cwl/pairchip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "-scale"
                    },
                    "doc": "Scale the coverage by a constant factor.\nEach coverage value is multiplied by this factor before being reported.\nUseful for normalizing coverage by, e.g., reads per million (RPM).\n- Default is 1.0; i.e., unscaled.\n- (FLOAT)\n",
                    "id": "#bedtools-genomecov.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "-split"
                    },
                    "doc": "treat \"split\" BAM or BED12 entries as distinct BED intervals.\nwhen computing coverage.\nFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\nto infer the blocks for computing coverage.\nFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\nfields (i.e., columns 10,11,12).\n",
                    "id": "#bedtools-genomecov.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "-strand"
                    },
                    "doc": "Calculate coverage of intervals from a specific strand.\nWith BED files, requires at least 6 columns (strand is column 6).\n- (STRING): can be + or -\n",
                    "id": "#bedtools-genomecov.cwl/strand"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "doc": "Generated genome coverage output file\n",
                    "id": "#bedtools-genomecov.cwl/genome_coverage_file",
                    "outputBinding": {
                        "glob": "${\n    return inputs.output_filename ? inputs.output_filename : default_output_filename();\n}\n"
                    }
                }
            ],
            "stdout": "${\n    return inputs.output_filename ? inputs.output_filename : default_output_filename();\n}\n",
            "baseCommand": [
                "bedtools",
                "genomecov"
            ],
            "doc": "Tool calculates genome coverage from input bam/bed/gff/vcf using `bedtools genomecov`\n\nDepending on `input_file` extension additional prefix is used: if `*.bam` use `-ibam`, else use `-i`.\n\n`scale` and `mapped_reads_number` inputs result in the same parameter `-scale`. If `scale` is not provided, check if\n`mapped_reads_number` is not null and calculate `-scale` as `1000000/mapped_reads_number`. If both inputs are\nnull, `bedtools genomecov` will use its default scaling value.\n\n`default_output_filename` function returns default output filename and is used when `output_filename` is not provided.\nDefault output file extention is `.tab`. If bedGraph should be generated (check flags `inputs.depth`), extension is\nupdated to `.bedGraph`. Default basename of the output file is generated on the base of `input_file` basename.\n",
            "id": "#bedtools-genomecov.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#bedtools-metadata.yaml",
                "http://schema.org/name": "bedtools",
                "http://schema.org/about": "A software suite for the comparison, manipulation and annotation of genomic features in browser extensible data (BED) and general feature format (GFF) format. BEDTools also supports the comparison of sequence alignments in BAM format to both BED and GFF features. The tools are extremely efficient and allow the user to compare large datasets (e.g. next-generation sequencing data) with both public and custom genome annotation tracks. BEDTools can be combined with one another as well as with standard UNIX commands, thus facilitating routine genomics tasks as well as pipelines that can quickly answer intricate questions of large genomic datasets.\n",
                "http://schema.org/url": "http://bedtools.readthedocs.org",
                "http://schema.org/codeRepository": "https://github.com/arq5x/bedtools2",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-2.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "2.26.0",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#bedtools-metadata.yaml/10.1093/bioinformatics/btq033",
                        "datePublished": "28 January 2010",
                        "http://schema.org/name": "BEDTools: a flexible suite of utilities for comparing genomic features",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Aaron R. Quinlan"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ira M. Hall"
                            }
                        ],
                        "http://schema.org/url": "https://academic.oup.com/bioinformatics/article-lookup/doi/10.1093/bioinformatics/btq033"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/arq5x/bedtools2/issues",
                    "https://groups.google.com/forum/#!forum/bedtools-discuss"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "University of Utah",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Aaron Quinlan",
                                "http://schema.org/description": "Main author"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Neil Kindlon",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "bedtools-genomecov",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bedtools-genomecov.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: bedtools genomecov [OPTIONS] -i <bed/gff/vcf> -g <genome>\n\nOptions:\n\t-ibam\t\tThe input file is in BAM format.\n\t\t\tNote: BAM _must_ be sorted by position\n\n\t-d\t\tReport the depth at each genome position (with one-based coordinates).\n\t\t\tDefault behavior is to report a histogram.\n\n\t-dz\t\tReport the depth at each genome position (with zero-based coordinates).\n\t\t\tReports only non-zero positions.\n\t\t\tDefault behavior is to report a histogram.\n\n\t-bg\t\tReport depth in BedGraph format. For details, see:\n\t\t\tgenome.ucsc.edu/goldenPath/help/bedgraph.html\n\n\t-bga\t\tReport depth in BedGraph format, as above (-bg).\n\t\t\tHowever with this option, regions with zero\n\t\t\tcoverage are also reported. This allows one to\n\t\t\tquickly extract all regions of a genome with 0\n\t\t\tcoverage by applying: \"grep -w 0$\" to the output.\n\n\t-split\t\tTreat \"split\" BAM or BED12 entries as distinct BED intervals.\n\t\t\twhen computing coverage.\n\t\t\tFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\n\t\t\tto infer the blocks for computing coverage.\n\t\t\tFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\n\t\t\tfields (i.e., columns 10,11,12).\n\n\t-strand\t\tCalculate coverage of intervals from a specific strand.\n\t\t\tWith BED files, requires at least 6 columns (strand is column 6).\n\t\t\t- (STRING): can be + or -\n\n\t-pc\t\tCalculate coverage of pair-end fragments.\n\t\t\tWorks for BAM files only\n\t-fs\t\tForce to use provided fragment size instead of read length\n\t\t\tWorks for BAM files only\n\t-du\t\tChange strand af the mate read (so both reads from the same strand) useful for strand specific\n\t\t\tWorks for BAM files only\n\t-5\t\tCalculate coverage of 5\" positions (instead of entire interval).\n\n\t-3\t\tCalculate coverage of 3\" positions (instead of entire interval).\n\n\t-max\t\tCombine all positions with a depth >= max into\n\t\t\ta single bin in the histogram. Irrelevant\n\t\t\tfor -d and -bedGraph\n\t\t\t- (INTEGER)\n\n\t-scale\t\tScale the coverage by a constant factor.\n\t\t\tEach coverage value is multiplied by this factor before being reported.\n\t\t\tUseful for normalizing coverage by, e.g., reads per million (RPM).\n\t\t\t- Default is 1.0; i.e., unscaled.\n\t\t\t- (FLOAT)\n\n\t-trackline\tAdds a UCSC/Genome-Browser track line definition in the first line of the output.\n\t\t\t- See here for more details about track line definition:\n\t\t\t      http://genome.ucsc.edu/goldenPath/help/bedgraph.html\n\t\t\t- NOTE: When adding a trackline definition, the output BedGraph can be easily\n\t\t\t      uploaded to the Genome Browser as a custom track,\n\t\t\t      BUT CAN NOT be converted into a BigWig file (w/o removing the first line).\n\n\t-trackopts\tWrites additional track line definition parameters in the first line.\n\t\t\t- Example:\n\t\t\t   -trackopts 'name=\"My Track\" visibility=2 color=255,30,30'\n\t\t\t   Note the use of single-quotes if you have spaces in your parameters.\n\t\t\t- (TEXT)\n\nNotes:\n\t(1) The genome file should tab delimited and structured as follows:\n\t <chromName><TAB><chromSize>\n\n\tFor example, Human (hg19):\n\tchr1\t249250621\n\tchr2\t243199373\n\t...\n\tchr18_gl000207_random\t4262\n\n\t(2) The input BED (-i) file must be grouped by chromosome.\n\t A simple \"sort -k 1,1 <BED> > <BED>.sorted\" will suffice.\n\n\t(3) The input BAM (-ibam) file must be sorted by position.\n\t A \"samtools sort <BAM>\" should suffice.\n\nTips:\n\tOne can use the UCSC Genome Browser's MySQL database to extract\n\tchromosome sizes. For example, H. sapiens:\n\n\tmysql --user=genome --host=genome-mysql.cse.ucsc.edu -A -e \\\n\t\"select chrom, size from hg19.chromInfo\" > hg19.genome\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function(ext) { if (inputs.output_filename != \"\" && !ext){ return inputs.output_filename; } ext = ext || \".sam\"; var root = \"\"; if (inputs.output_filename != \"\"){ root = inputs.output_filename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.output_filename+ext:root+ext; } else if (Array.isArray(inputs.upstream_filelist) && inputs.upstream_filelist.length > 0){ root = inputs.upstream_filelist[0].basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.upstream_filelist[0].basename+ext:root+ext; } else if (inputs.upstream_filelist != null){ root = inputs.upstream_filelist.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.upstream_filelist.basename+ext:root+ext; } else if (Array.isArray(inputs.downstream_filelist) && inputs.downstream_filelist.length > 0){ root = inputs.downstream_filelist[0].basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.downstream_filelist[0].basename+ext:root+ext; } else if (inputs.downstream_filelist != null){ root = inputs.downstream_filelist.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.downstream_filelist.basename+ext:root+ext; } else if (Array.isArray(inputs.crossbow_filelist) && inputs.crossbow_filelist.length > 0){ root = inputs.crossbow_filelist[0].basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.crossbow_filelist[0].basename+ext:root+ext; } else if (inputs.crossbow_filelist != null){ root = inputs.crossbow_filelist.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.crossbow_filelist.basename+ext:root+ext; } else { return null; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/bowtie:v1.2.0"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-B"
                    },
                    "doc": "--offbase <int> leftmost ref offset = <int> in bowtie output (default: 0)\n",
                    "id": "#bowtie-alignreads.cwl/B"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-C"
                    },
                    "doc": "reads and index are in colorspace\n",
                    "id": "#bowtie-alignreads.cwl/C"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-I"
                    },
                    "doc": "--minins <int>  minimum insert size for paired-end alignment (default: 0)\n",
                    "id": "#bowtie-alignreads.cwl/I"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-M"
                    },
                    "doc": "<int> like -m, but reports 1 random hit (MAPQ=0)\n",
                    "id": "#bowtie-alignreads.cwl/M"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-Q"
                    },
                    "doc": "QV file(s) corresponding to CSFASTA inputs; use with -f -C\n",
                    "id": "#bowtie-alignreads.cwl/Q"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--Q1"
                    },
                    "doc": "--Q1/--Q2 <file>   same as -Q, but for mate files 1 and 2 respectively\n",
                    "id": "#bowtie-alignreads.cwl/Q1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--Q2"
                    },
                    "doc": "--Q1/--Q2 <file>   same as -Q, but for mate files 1 and 2 respectively\n",
                    "id": "#bowtie-alignreads.cwl/Q2"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-X"
                    },
                    "doc": "--maxins <int>  maximum insert size for paired-end alignment (default: 250)\n",
                    "id": "#bowtie-alignreads.cwl/X"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-a"
                    },
                    "doc": "--all           report all alignments per read (much slower than low -k)\n",
                    "id": "#bowtie-alignreads.cwl/a"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--al",
                        "valueFrom": "$(self + \".fastq\")"
                    },
                    "doc": "<fname>       write aligned reads/pairs to file(s) <fname>\n",
                    "id": "#bowtie-alignreads.cwl/aligned_prefix"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--best"
                    },
                    "doc": "hits guaranteed best stratum; ties broken by quality\n",
                    "id": "#bowtie-alignreads.cwl/best"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-c"
                    },
                    "doc": "query sequences given on cmd line (as <mates>, <singles>)\n",
                    "id": "#bowtie-alignreads.cwl/c"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--chunkmbs"
                    },
                    "doc": "<int>   max megabytes of RAM for best-first search frames (def: 64)\n",
                    "id": "#bowtie-alignreads.cwl/chunkmbs"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-3"
                    },
                    "doc": "--trim3 <int>   trim <int> bases from 3' (right) end of reads\n",
                    "id": "#bowtie-alignreads.cwl/clip_3p_end"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-5"
                    },
                    "doc": "--trim5 <int>   trim <int> bases from 5' (left) end of reads\n",
                    "id": "#bowtie-alignreads.cwl/clip_5p_end"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--col-cqual"
                    },
                    "doc": "print original colorspace quals, not decoded quals\n",
                    "id": "#bowtie-alignreads.cwl/col_cqual"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--col-cseq"
                    },
                    "doc": "print aligned colorspace seqs as colors, not decoded bases\n",
                    "id": "#bowtie-alignreads.cwl/col_cseq"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--col-keepends"
                    },
                    "doc": "keep nucleotides at extreme ends of decoded alignment\n",
                    "id": "#bowtie-alignreads.cwl/col_keepends"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 86,
                        "prefix": "-12"
                    },
                    "doc": "Comma-separated list of files containing Crossbow-style reads.\nCan be a mixture of paired and unpaired.  Specify \"-\"for stdin.\n",
                    "id": "#bowtie-alignreads.cwl/crossbow_filelist"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 85
                    },
                    "doc": "Comma-separated list of files containing downstream mates (or the\nsequences themselves if -c is set) paired with mates in <m1>\n",
                    "id": "#bowtie-alignreads.cwl/downstream_filelist"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-e"
                    },
                    "doc": "--maqerr <int>  max sum of mismatch quals across alignment for -n (def: 70)\n",
                    "id": "#bowtie-alignreads.cwl/e"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-f"
                    },
                    "doc": "query input files are (multi-)FASTA .fa/.mfa\n",
                    "id": "#bowtie-alignreads.cwl/f"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--ff"
                    },
                    "doc": "--fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n",
                    "id": "#bowtie-alignreads.cwl/ff"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fr"
                    },
                    "doc": "--fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n",
                    "id": "#bowtie-alignreads.cwl/fr"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--fullref"
                    },
                    "doc": "write entire ref name (default: only up to 1st space)\n",
                    "id": "#bowtie-alignreads.cwl/fullref"
                },
                {
                    "type": "Directory",
                    "inputBinding": {
                        "position": 81,
                        "valueFrom": "${\n    for (var i = 0; i < self.listing.length; i++) {\n        if (self.listing[i].path.split('.').slice(-3).join('.') == 'rev.1.ebwt' ||\n            self.listing[i].path.split('.').slice(-3).join('.') == 'rev.1.ebwtl'){\n          return self.listing[i].path.split('.').slice(0,-3).join('.');\n        }\n    }\n    return null;\n}\n"
                    },
                    "doc": "Folder with Bowtie indices\n",
                    "id": "#bowtie-alignreads.cwl/indices_folder"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--integer-quals"
                    },
                    "doc": "qualities are given as space-separated integers (not ASCII)\n",
                    "id": "#bowtie-alignreads.cwl/integer_quals"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-k"
                    },
                    "doc": "<int>           report up to <int> good alignments per read (default: 1)\n",
                    "id": "#bowtie-alignreads.cwl/k"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-l"
                    },
                    "doc": "--seedlen <int> seed length for -n (default: 28)\n",
                    "id": "#bowtie-alignreads.cwl/l"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--large-index"
                    },
                    "doc": "force usage of a 'large' index, even if a small one is present\n",
                    "id": "#bowtie-alignreads.cwl/large_index"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-m"
                    },
                    "doc": "<int>           suppress all alignments if > <int> exist (def: no limit)\n",
                    "id": "#bowtie-alignreads.cwl/m"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--mapq"
                    },
                    "doc": "<int>       default mapping quality (MAPQ) to print for SAM alignments\n",
                    "id": "#bowtie-alignreads.cwl/mapq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--maxbts"
                    },
                    "doc": "<int>     max # backtracks for -n 2/3 (default: 125, 800 for --best)\n",
                    "id": "#bowtie-alignreads.cwl/maxbts"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--mm"
                    },
                    "doc": "use memory-mapped I/O for index; many 'bowtie's can share\n",
                    "id": "#bowtie-alignreads.cwl/mm"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--max",
                        "valueFrom": "$(self + \".fastq\")"
                    },
                    "doc": "<fname>      write reads/pairs over -m limit to file(s) <fname>\n",
                    "id": "#bowtie-alignreads.cwl/multimapped_prefix"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-n"
                    },
                    "doc": "--seedmms <int> max mismatches in seed (can be 0-3, default: -n 2)\n",
                    "id": "#bowtie-alignreads.cwl/n"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--nofw"
                    },
                    "doc": "--norc      do not align to forward/reverse-complement reference strand\n",
                    "id": "#bowtie-alignreads.cwl/nofw"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--nomaqround"
                    },
                    "doc": "disable Maq-like quality rounding for -n (nearest 10 <= 30)\n",
                    "id": "#bowtie-alignreads.cwl/nomaqround"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-o"
                    },
                    "doc": "--offrate <int> override offrate of index; must be >= index's offrate\n",
                    "id": "#bowtie-alignreads.cwl/o"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 90,
                        "valueFrom": "$(default_output_filename())"
                    },
                    "default": "",
                    "doc": "Generates default output filename on the base of upstream_filelist/downstream_filelist files\n",
                    "id": "#bowtie-alignreads.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--pairtries"
                    },
                    "doc": "<int>  max # attempts to find mate for anchor hit (default: 100)\n",
                    "id": "#bowtie-alignreads.cwl/pairtries"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--phred33-quals"
                    },
                    "doc": "input quals are Phred+33 (default)\n",
                    "id": "#bowtie-alignreads.cwl/phred33_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--phred64-quals"
                    },
                    "doc": "input quals are Phred+64 (same as --solexa1.3-quals)\n",
                    "id": "#bowtie-alignreads.cwl/phred64_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-q"
                    },
                    "doc": "query input files are FASTQ .fq/.fastq (default)\n",
                    "id": "#bowtie-alignreads.cwl/q"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--quiet"
                    },
                    "doc": "print nothing but the alignments\n",
                    "id": "#bowtie-alignreads.cwl/quiet"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-r"
                    },
                    "doc": "query input files are raw one-sequence-per-line\n",
                    "id": "#bowtie-alignreads.cwl/r"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--reads-per-batch"
                    },
                    "doc": "# of reads to read from input file at once (default: 16)\n",
                    "id": "#bowtie-alignreads.cwl/reads_per_batch"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--refidx"
                    },
                    "doc": "refer to ref. seqs by 0-based index rather than name\n",
                    "id": "#bowtie-alignreads.cwl/refidx"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--refout"
                    },
                    "doc": "write alignments to files refXXXXX.map, 1 map per reference\n",
                    "id": "#bowtie-alignreads.cwl/refout"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--rf"
                    },
                    "doc": "--fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n",
                    "id": "#bowtie-alignreads.cwl/rf"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-s"
                    },
                    "doc": "--skip <int>    skip the first <int> reads/pairs in the input\n",
                    "id": "#bowtie-alignreads.cwl/s"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-S"
                    },
                    "doc": "--sam write hits in SAM format\n",
                    "id": "#bowtie-alignreads.cwl/sam"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-RG"
                    },
                    "doc": "<text>    add <text> (usually \"lab=value\") to @RG line of SAM header\n",
                    "id": "#bowtie-alignreads.cwl/sam_RG"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-nohead"
                    },
                    "doc": "supppress header lines (starting with @) for SAM output\n",
                    "id": "#bowtie-alignreads.cwl/sam_nohead"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--sam-nosq"
                    },
                    "doc": "supppress @SQ header lines for SAM output\n",
                    "id": "#bowtie-alignreads.cwl/sam_nosq"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seed"
                    },
                    "doc": "<int>       seed for random number generator\n",
                    "id": "#bowtie-alignreads.cwl/seed"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--shmem"
                    },
                    "doc": "use shared mem for index\n",
                    "id": "#bowtie-alignreads.cwl/shmem"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--snpfrac"
                    },
                    "doc": "<dec>    approx. fraction of SNP bases (e.g. 0.001); sets --snpphred\n",
                    "id": "#bowtie-alignreads.cwl/snpfrac"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--snpphred"
                    },
                    "doc": "<int>   Phred penalty for SNP when decoding colorspace (def: 30)\n",
                    "id": "#bowtie-alignreads.cwl/snpphred"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--solexa1.3-quals"
                    },
                    "doc": "input quals are from GA Pipeline ver. >= 1.3\n",
                    "id": "#bowtie-alignreads.cwl/solexa1.3_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--solexa-quals"
                    },
                    "doc": "input quals are from GA Pipeline ver. < 1.3\n",
                    "id": "#bowtie-alignreads.cwl/solexa_quals"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--strata"
                    },
                    "doc": "hits in sub-optimal strata aren't reported (requires --best)\n",
                    "id": "#bowtie-alignreads.cwl/strata"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--suppress"
                    },
                    "doc": "<cols>  suppresses given columns (comma-delim'ed) in default output\n",
                    "id": "#bowtie-alignreads.cwl/suppress"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-t"
                    },
                    "doc": "--time          print wall-clock time taken by search phases\n",
                    "id": "#bowtie-alignreads.cwl/t"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-p"
                    },
                    "doc": "--threads <int> number of alignment threads to launch (default: 1)\n",
                    "id": "#bowtie-alignreads.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-u"
                    },
                    "doc": "--qupto <int>   stop after first <int> reads/pairs (excl. skipped reads)\n",
                    "id": "#bowtie-alignreads.cwl/u"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--un",
                        "valueFrom": "$(self + \".fastq\")"
                    },
                    "doc": "<fname>       write unaligned reads/pairs to file(s) <fname>\n",
                    "id": "#bowtie-alignreads.cwl/unaligned_prefix"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "itemSeparator": ",",
                        "position": 83
                    },
                    "doc": "Comma-separated list of files containing upstream mates (or the\nsequences themselves, if -c is set) paired with mates in <m2>\n",
                    "id": "#bowtie-alignreads.cwl/upstream_filelist"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-v"
                    },
                    "doc": "report end-to-end hits w/ <=v mismatches; ignore qualities\n",
                    "id": "#bowtie-alignreads.cwl/v"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--verbose"
                    },
                    "doc": "verbose output (for debugging)\n",
                    "id": "#bowtie-alignreads.cwl/verbose"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-y"
                    },
                    "doc": "--tryhard try hard to find valid alignments, at the expense of speed\n",
                    "id": "#bowtie-alignreads.cwl/y"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.aligned_prefix + \"*\")"
                    },
                    "id": "#bowtie-alignreads.cwl/aligned_fastq"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(default_output_filename(\".bw\"))"
                    },
                    "id": "#bowtie-alignreads.cwl/log_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(default_output_filename(\".bw\"))",
                        "outputEval": "${\n  var mappedRegex = /alignment\\:.*/;\n  return parseInt(self[0].contents.match(mappedRegex)[0].split(\" \")[1]);\n}\n"
                    },
                    "id": "#bowtie-alignreads.cwl/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.multimapped_prefix + \"*\")"
                    },
                    "id": "#bowtie-alignreads.cwl/multimapped_fastq"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "*ref*.map*"
                    },
                    "id": "#bowtie-alignreads.cwl/refout_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(default_output_filename())"
                    },
                    "id": "#bowtie-alignreads.cwl/sam_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(default_output_filename(\".bw\"))",
                        "outputEval": "${\n  var totalRegex = /processed\\:.*/;\n  return parseInt(self[0].contents.match(totalRegex)[0].split(\" \")[1]);\n}\n"
                    },
                    "id": "#bowtie-alignreads.cwl/total_reads_number"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.unaligned_prefix + \"*\")"
                    },
                    "id": "#bowtie-alignreads.cwl/unaligned_fastq"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(default_output_filename(\".bw\"))",
                        "outputEval": "${\n  var unmappedRegex = /align\\:.*/;\n  return parseInt(self[0].contents.match(unmappedRegex)[0].split(\" \")[1]);\n}\n"
                    },
                    "id": "#bowtie-alignreads.cwl/unmapped_reads_number"
                }
            ],
            "baseCommand": [
                "bowtie"
            ],
            "arguments": [
                {
                    "valueFrom": "${\n  if (inputs.upstream_filelist && inputs.downstream_filelist){\n    return \"-1\";\n  }\n  return null;\n}\n",
                    "position": 82
                },
                {
                    "valueFrom": "${\n  if (inputs.upstream_filelist && inputs.downstream_filelist){\n    return \"-2\";\n  }\n  return null;\n}\n",
                    "position": 84
                },
                {
                    "valueFrom": "${\n  return ' 2> ' + default_output_filename(\".bw\");\n}\n",
                    "position": 100000,
                    "shellQuote": false
                }
            ],
            "doc": "Tool maps input raw reads files to reference genome using Bowtie.\n\n`default_output_filename` function returns default name for SAM output and log files. In case when `sam` and\n`output_filename` inputs are not set, default filename will have `.sam` extension but format may not correspond SAM\nspecification. To set output filename manually use `output_filename` input. Default output filename is based on\n`output_filename` or basename of `upstream_filelist`, `downstream_filelist` or `crossbow_filelist` file (if array,\nthe first file in array is taken). If function is called without argenments and `output_filename` input is set, it\nwill be returned from the function.\n\nFor single-end input data any of the `upstream_filelist` or `downstream_filelist` inputs can be used.\n\nLog filename (`log_file` output) is generated by `default_output_filename` function with ex='.bw'\n\n`indices_folder` defines folder to contain Bowtie indices. Based on the first found file with `rev.1.ebwt` or\n`rev.1.ebwtl` extension, bowtie index prefix is returned from input's `valueFrom` field.\n",
            "id": "#bowtie-alignreads.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#bowtie-metadata.yaml",
                "http://schema.org/name": "bowtie2",
                "http://schema.org/about": "Bowtie is an ultrafast, memory-efficient short read aligner. It aligns short DNA sequences (reads)\nto the human genome at a rate of over 25 million 35-bp reads per hour. Bowtie indexes the genome\nwith a Burrows-Wheeler index to keep its memory footprint small: typically about 2.2 GB for\nthe human genome (2.9 GB for paired-end).\n",
                "http://schema.org/url": "http://bowtie-bio.sourceforge.net/index.shtml",
                "http://schema.org/codeRepository": "https://github.com/BenLangmead/bowtie",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "1.2.0",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#bowtie-metadata.yaml/10.1186/gb-2009-10-3-r25",
                        "datePublished": "04 March 2009",
                        "http://schema.org/name": "Ultrafast and memory-efficient alignment of short DNA sequences to the human genome",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ben Langmead"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Steven L Salzberg"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Cole Trapnell"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Mihai Pop"
                            }
                        ],
                        "http://schema.org/url": "https://genomebiology.biomedcentral.com/articles/10.1186/gb-2009-10-3-r25"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://github.com/BenLangmead/bowtie/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Johns Hopkins University",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Ben Langmead",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "bowtie",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bowtie.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:\nbowtie [options]* <ebwt> {-1 <m1> -2 <m2> | --12 <r> | <s>} [<hit>]\n\n  <m1>    Comma-separated list of files containing upstream mates (or the\n          sequences themselves, if -c is set) paired with mates in <m2>\n  <m2>    Comma-separated list of files containing downstream mates (or the\n          sequences themselves if -c is set) paired with mates in <m1>\n  <r>     Comma-separated list of files containing Crossbow-style reads.  Can be\n          a mixture of paired and unpaired.  Specify \"-\" for stdin.\n  <s>     Comma-separated list of files containing unpaired reads, or the\n          sequences themselves, if -c is set.  Specify \"-\" for stdin.\n  <hit>   File to write hits to (default: stdout)\nInput:\n  -q                 query input files are FASTQ .fq/.fastq (default)\n  -f                 query input files are (multi-)FASTA .fa/.mfa\n  -r                 query input files are raw one-sequence-per-line\n  -c                 query sequences given on cmd line (as <mates>, <singles>)\n  -C                 reads and index are in colorspace\n  -Q/--quals <file>  QV file(s) corresponding to CSFASTA inputs; use with -f -C\n  --Q1/--Q2 <file>   same as -Q, but for mate files 1 and 2 respectively\n  -s/--skip <int>    skip the first <int> reads/pairs in the input\n  -u/--qupto <int>   stop after first <int> reads/pairs (excl. skipped reads)\n  -5/--trim5 <int>   trim <int> bases from 5' (left) end of reads\n  -3/--trim3 <int>   trim <int> bases from 3' (right) end of reads\n  --phred33-quals    input quals are Phred+33 (default)\n  --phred64-quals    input quals are Phred+64 (same as --solexa1.3-quals)\n  --solexa-quals     input quals are from GA Pipeline ver. < 1.3\n  --solexa1.3-quals  input quals are from GA Pipeline ver. >= 1.3\n  --integer-quals    qualities are given as space-separated integers (not ASCII)\n  --large-index      force usage of a 'large' index, even if a small one is present\nAlignment:\n  -v <int>           report end-to-end hits w/ <=v mismatches; ignore qualities\n    or\n  -n/--seedmms <int> max mismatches in seed (can be 0-3, default: -n 2)\n  -e/--maqerr <int>  max sum of mismatch quals across alignment for -n (def: 70)\n  -l/--seedlen <int> seed length for -n (default: 28)\n  --nomaqround       disable Maq-like quality rounding for -n (nearest 10 <= 30)\n  -I/--minins <int>  minimum insert size for paired-end alignment (default: 0)\n  -X/--maxins <int>  maximum insert size for paired-end alignment (default: 250)\n  --fr/--rf/--ff     -1, -2 mates align fw/rev, rev/fw, fw/fw (default: --fr)\n  --nofw/--norc      do not align to forward/reverse-complement reference strand\n  --maxbts <int>     max # backtracks for -n 2/3 (default: 125, 800 for --best)\n  --pairtries <int>  max # attempts to find mate for anchor hit (default: 100)\n  -y/--tryhard       try hard to find valid alignments, at the expense of speed\n  --chunkmbs <int>   max megabytes of RAM for best-first search frames (def: 64)\n --reads-per-batch   # of reads to read from input file at once (default: 16)\nReporting:\n  -k <int>           report up to <int> good alignments per read (default: 1)\n  -a/--all           report all alignments per read (much slower than low -k)\n  -m <int>           suppress all alignments if > <int> exist (def: no limit)\n  -M <int>           like -m, but reports 1 random hit (MAPQ=0); requires --best\n  --best             hits guaranteed best stratum; ties broken by quality\n  --strata           hits in sub-optimal strata aren't reported (requires --best)\nOutput:\n  -t/--time          print wall-clock time taken by search phases\n  -B/--offbase <int> leftmost ref offset = <int> in bowtie output (default: 0)\n  --quiet            print nothing but the alignments\n  --refout           write alignments to files refXXXXX.map, 1 map per reference\n  --refidx           refer to ref. seqs by 0-based index rather than name\n  --al <fname>       write aligned reads/pairs to file(s) <fname>\n  --un <fname>       write unaligned reads/pairs to file(s) <fname>\n  --max <fname>      write reads/pairs over -m limit to file(s) <fname>\n  --suppress <cols>  suppresses given columns (comma-delim'ed) in default output\n  --fullref          write entire ref name (default: only up to 1st space)\nColorspace:\n  --snpphred <int>   Phred penalty for SNP when decoding colorspace (def: 30)\n     or\n  --snpfrac <dec>    approx. fraction of SNP bases (e.g. 0.001); sets --snpphred\n  --col-cseq         print aligned colorspace seqs as colors, not decoded bases\n  --col-cqual        print original colorspace quals, not decoded quals\n  --col-keepends     keep nucleotides at extreme ends of decoded alignment\nSAM:\n  -S/--sam           write hits in SAM format\n  --mapq <int>       default mapping quality (MAPQ) to print for SAM alignments\n  --sam-nohead       supppress header lines (starting with @) for SAM output\n  --sam-nosq         supppress @SQ header lines for SAM output\n  --sam-RG <text>    add <text> (usually \"lab=value\") to @RG line of SAM header\nPerformance:\n  -o/--offrate <int> override offrate of index; must be >= index's offrate\n  -p/--threads <int> number of alignment threads to launch (default: 1)\n  --mm               use memory-mapped I/O for index; many 'bowtie's can share\n  --shmem            use shared mem for index; many 'bowtie's can share\nOther:\n  --seed <int>       seed for random number generator\n  --verbose          verbose output (for debugging)\n  --version          print version information and quit\n  -h/--help          print this usage message\n"
        },
        {
            "class": "CommandLineTool",
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "scidap/scidap:v0.0.4"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 100000,
                    "inputBinding": {
                        "position": 9
                    },
                    "id": "#bypass-trimgalore-se.cwl/min_reads_count"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 6
                    },
                    "id": "#bypass-trimgalore-se.cwl/original_fastq_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nORIGINAL_FASTQ=$0\nTRIMMED_FASTQ=$1\nTRIMMING_REPORT=$2\nMIN_COUNTS=$3\nTRIMMED_COUNTS=$(( `cat $TRIMMED_FASTQ | wc -l` / 4 )) \n\necho \"ORIGINAL_FASTQ: ${ORIGINAL_FASTQ}\"\necho \"TRIMMED_FASTQ: ${TRIMMED_FASTQ}\"\necho \"TRIMMING_REPORT: ${TRIMMING_REPORT}\"\necho \"TRIMMED_COUNTS: ${TRIMMED_COUNTS}\"\n\nif (( $TRIMMED_COUNTS < $MIN_COUNTS )); then\n  echo \"Bypassing adapter trimming\"\n  cp $ORIGINAL_FASTQ `basename $TRIMMED_FASTQ`\nelse\n  echo \"Using adapter trimming results\"\n  cp $TRIMMED_FASTQ .\n  cp $TRIMMING_REPORT .\nfi\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "id": "#bypass-trimgalore-se.cwl/script"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 7
                    },
                    "id": "#bypass-trimgalore-se.cwl/trimmed_fastq_file"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 8
                    },
                    "id": "#bypass-trimgalore-se.cwl/trimming_report_file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.trimmed_fastq_file.basename)"
                    },
                    "id": "#bypass-trimgalore-se.cwl/selected_fastq_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(inputs.trimming_report_file.basename)"
                    },
                    "id": "#bypass-trimgalore-se.cwl/selected_report_file"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "If the number of reads in the trimmed_fastq_file is less then min_reads_count, tool\nwill return original_fastq_file and null as selected_report_file. Otherwise, the\ntrimmed_fastq_file and trimming_report_file will be returned. Might be usefull in\ncase of trimgalore removed all reads from the original_fastq_file\n",
            "id": "#bypass-trimgalore-se.cwl",
            "http://schema.org/name": "bypass-trimgalore-se",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/bypass-trimgalore-se.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "If the number of reads in the trimmed_fastq_file is less then min_reads_count, tool\nwill return original_fastq_file and null as selected_report_file. Otherwise, the\ntrimmed_fastq_file and trimming_report_file will be returned. Might be usefull in\ncase of trimgalore removed all reads from the original_fastq_file"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_output_prefix = function() { if (inputs.output_prefix) { return inputs.output_prefix; } var root = inputs.bowtie_alignment_report.basename.split('.').slice(0,-1).join('.'); var suffix = \"_collected_statistics_report\"; return (root == \"\")?inputs.bowtie_alignment_report.basename+suffix:root+suffix; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "rackspacedot/python37"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "--bamstatsfilter"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/bam_statistics_after_filtering_report"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 9,
                        "prefix": "--bamstats"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/bam_statistics_report"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--bowtie"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/bowtie_alignment_report"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 11,
                        "prefix": "--macs2"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/macs2_called_peaks"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "--output",
                        "valueFrom": "$(get_output_prefix())"
                    },
                    "default": "",
                    "id": "#collect-statistics-chip-seq.cwl/output_prefix"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "--paired"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/paired_end"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "--preseq"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/preseq_results"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/usr/bin/env python\nimport os\nimport sys\nimport argparse\nimport yaml\nimport math\n\ndef cut_int(s):\n    return int(s.strip().split()[0])\n\n\ndef cut_float(s):\n    return float(s.strip().split()[0])\n\n\nTRIMGALORE = {\n    \"Trimming mode\": {\n        \"alias\": \"trimming mode\",\n        \"function\": str,\n        \"pair_end_specific\": False\n    },\n    \"Quality Phred score cutoff\": {\n        \"alias\": \"quality phred score cutoff\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"Quality encoding type selected\": {\n        \"alias\": \"quality encoding type\",\n        \"function\": str,\n        \"pair_end_specific\": False\n    },\n    \"Adapter sequence\": {\n        \"alias\": \"adapter sequence\",\n        \"function\": str,\n        \"pair_end_specific\": False\n    },\n    \"Minimum required adapter overlap\": {\n        \"alias\": \"minimum required adapter overlap\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"Maximum trimming error rate\": {\n        \"alias\": \"maximum trimming error rate\",\n        \"function\": cut_float,\n        \"pair_end_specific\": False\n    },\n    \"Minimum required sequence length\": {\n        \"alias\": \"minimum required read length\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"Total number of sequences analysed\": {\n        \"alias\": \"number of reads/pairs analysed for length validation\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"Number of sequence pairs removed\": {\n        \"alias\": \"reads/pairs removed because of length cutoff\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"Sequences removed because they became shorter\": {\n        \"alias\": \"reads/pairs removed because of length cutoff\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"order\": [\"fastq\",\n            \"trimming mode\",\n            \"adapter sequence\",\n            \"number of reads/pairs analysed for length validation\",\n            \"reads/pairs removed because of length cutoff\",\n            \"minimum required read length\",\n            \"quality phred score cutoff\",\n            \"quality encoding type\",\n            \"minimum required adapter overlap\",\n            \"maximum trimming error rate\"]\n}\n\n\nBOWTIE = {\n    \"reads processed\": {\n        \"alias\": \"total reads/pairs processed\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"reads with at least one reported alignment\": {\n        \"alias\": \"reads/pairs with at least one reported alignment\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"reads that failed to align\": {\n        \"alias\": \"reads/pairs unmapped\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"reads with alignments suppressed due to -m\": {\n        \"alias\": \"reads/pairs suppressed due to multimapping\",\n        \"function\": cut_int,\n        \"pair_end_specific\": False\n    },\n    \"order\": [\"total reads/pairs processed\",\n            \"reads/pairs with at least one reported alignment\",\n            \"reads/pairs suppressed due to multimapping\",\n            \"reads/pairs unmapped\"]\n}\n\n\nBAMSTATS = {\n    \"raw total sequences\": {\n        \"alias\": \"total reads/pairs\",\n        \"function\": int,\n        \"pair_end_specific\": True\n    },\n    \"reads mapped\": {\n        \"alias\": \"reads/pairs mapped\",\n        \"function\": int,\n        \"pair_end_specific\": True\n    },\n    \"reads unmapped\": {\n        \"alias\": \"reads/pairs unmapped\",\n        \"function\": int,\n        \"pair_end_specific\": True\n    },\n    \"average length\": {\n        \"alias\": \"reads average length\",\n        \"function\": float,\n        \"pair_end_specific\": False\n    },\n    \"maximum length\": {\n        \"alias\": \"reads maximum length\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"average quality\": {\n        \"alias\": \"reads average quality\",\n        \"function\": float,\n        \"pair_end_specific\": False\n    },\n    \"insert size average\": {\n        \"alias\": \"insert size average\",\n        \"function\": float,\n        \"pair_end_specific\": False\n    },\n    \"insert size standard deviation\": {\n        \"alias\": \"insert size standard deviation\",\n        \"function\": float,\n        \"pair_end_specific\": False\n    },\n    \"order\": [\"total reads/pairs\",\n            \"reads/pairs mapped\",\n            \"reads/pairs unmapped\",\n            \"reads average length\",\n            \"reads maximum length\",\n            \"reads average quality\",\n            \"insert size average\",\n            \"insert size standard deviation\"]\n}\n\n\nMACS2 = {\n    \"total fragments in treatment\": {\n        \"alias\": \"total reads/pairs in treatment\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"total tags in treatment\": {\n        \"alias\": \"total reads/pairs in treatment\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"fragments after filtering in treatment\": {\n        \"alias\": \"reads/pairs after filtering in treatment\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"tags after filtering in treatment\": {\n        \"alias\": \"reads/pairs after filtering in treatment\",\n        \"function\": int,\n        \"pair_end_specific\": False\n    },\n    \"Redundant rate in treatment\": {\n        \"alias\": \"redundant rate in treatment\",\n        \"function\": float,\n        \"pair_end_specific\": False\n    },\n    \"order\": [\"number of peaks called\",\n            \"mean peak size\",\n            \"total reads/pairs in treatment\",\n            \"reads/pairs after filtering in treatment\",\n            \"redundant rate in treatment\",\n            \"fraction of reads in peaks\"]\n}\n\n\ndef arg_parser():\n    general_parser = argparse.ArgumentParser()\n    general_parser.add_argument(\"--trim1\",           help=\"Path to Trimgalore report file for FASTQ 1. Optional\", required=False)\n    general_parser.add_argument(\"--trim2\",           help=\"Path to Trimgalore report file for FASTQ 2. Optional\", required=False)\n    general_parser.add_argument(\"--bowtie\",          help=\"Path to Bowtie report file\",                           required=True)\n    general_parser.add_argument(\"--bamstats\",        help=\"Path to bam statistics report file\",                   required=True)\n    general_parser.add_argument(\"--bamstatsfilter\",  help=\"Path to bam statistics report file after filtering\",   required=True)\n    general_parser.add_argument(\"--macs2\",           help=\"Path to MACS2 called peaks xls file\",                  required=True)\n    general_parser.add_argument(\"--preseq\",          help=\"Path to Preseq output file\",                           required=False)\n    general_parser.add_argument(\"--paired\",          help=\"Process as paired-end. Default: False\",                action=\"store_true\")\n    general_parser.add_argument(\"--output\",          help=\"Output filename prefix\",                               required=True)\n    return general_parser\n\n\ndef normalize_args(args, skip_list=[]):\n    normalized_args = {}\n    for key,value in args.__dict__.items():\n        if key not in skip_list:\n            normalized_args[key] = value if not value or os.path.isabs(value) else os.path.normpath(os.path.join(os.getcwd(), value))\n        else:\n            normalized_args[key]=value\n    return argparse.Namespace (**normalized_args)\n\n\ndef open_file(filename):\n    lines = []\n    with open(filename, 'r') as infile:\n        for line in infile:\n            if line.strip():\n                lines.append(line.strip())\n    return lines\n\n\ndef split_line(l, delimiter=\":\"):\n    return [i.strip() for i in l.split(delimiter)]\n\n\ndef get_correspondent_key(data, long_k):\n    for short_k, v in data.items():\n        if short_k in long_k:\n            return v[\"alias\"], v[\"function\"], v[\"pair_end_specific\"]\n    raise Exception\n\n\ndef process_trimgalore_report(filepath, collected_results, header=\"adapter trimming statistics\"):\n    if not collected_results.get(header, None):\n        collected_results[header] = {\"fastq\": []}\n    fastq = {}\n    for line in open_file(filepath):\n        try:\n            key, value = split_line(line)\n            if \"Total reads processed\" in line:\n                fastq[\"total reads processed\"] = int(value.strip().replace(\",\",\"\"))\n            elif \"Reads with adapters\" in line:\n                fastq[\"reads with adapters\"] = int(value.strip().replace(\",\",\"\").split()[0])\n            else:\n                res_key, res_function, pair_end_specific = get_correspondent_key(TRIMGALORE, key)\n                if not collected_results[header].get(res_key, None):\n                    collected_results[header][res_key] = res_function(value)\n        except Exception:\n            pass\n    if collected_results[header][\"trimming mode\"] == \"single-end\":\n        collected_results[header][\"number of reads/pairs analysed for length validation\"] = fastq[\"total reads processed\"]\n    collected_results[header][\"fastq\"].append(fastq)\n    if TRIMGALORE.get(\"order\", None):\n        collected_results[header] = {k: collected_results[header][k] for k in TRIMGALORE[\"order\"] if k in collected_results[header]}\n\n\ndef process_custom_report(filepath, collected_results, header, key_dict, pair_end=False):\n    if not collected_results.get(header, None):\n        collected_results[header] = {}\n    for line in open_file(filepath):\n        try:\n            key, value = split_line(line)\n            res_key, res_function, pair_end_specific = get_correspondent_key(key_dict, key)\n            if not collected_results[header].get(res_key, None):\n                if pair_end_specific and pair_end:\n                    collected_results[header][res_key] = res_function(res_function(value)/2)\n                else:\n                    collected_results[header][res_key] = res_function(value)\n        except Exception:\n            pass\n    if key_dict.get(\"order\", None):\n        collected_results[header] = {k: collected_results[header][k] for k in key_dict[\"order\"] if k in collected_results[header]}\n\n\ndef process_macs2_xls(filepath, collected_results, header):\n    if not collected_results.get(header, None):\n        collected_results[header] = {}\n    count, length, prev_start, prev_end, prev_chr = 0, 0, 0, 0, \"\"\n    for line in open_file(filepath):\n        if \"#\" in line or \"start\" in line:\n            continue\n        line_list = [l.strip() for l in line.strip().split()]\n        chr = line_list[0]\n        start = int(line_list[1])\n        end = int(line_list[2])\n        if start == prev_start and end == prev_end and chr == prev_chr:\n            continue\n        count = count + 1\n        length = length + end - start\n        prev_start, prev_end, prev_chr = start, end, chr\n    collected_results[header][\"number of peaks called\"] = count\n    collected_results[header][\"mean peak size\"] = round(float(length)/float(count), 2)\n    in_treatment = collected_results[header][\"reads/pairs after filtering in treatment\"]\n    mapped = collected_results[\"BAM statistics after filtering\"][\"reads/pairs mapped\"]\n    collected_results[header][\"fraction of reads in peaks\"] = round(float(in_treatment)/float(mapped),2)\n    if MACS2.get(\"order\", None):\n        collected_results[header] = {k: collected_results[header][k] for k in MACS2[\"order\"] if k in collected_results[header]}\n\n\ndef process_preseq_results(filepath, collected_results, header, threashold=0.001):\n    px, py = 0, 0\n    for line in open_file(filepath):\n        if \"TOTAL_READS\" in line:\n            continue\n        values = [float(l.strip()) for l in line.strip().split()]\n        dx, dy = values[0]-px, values[1]-py\n        px, py = values[0], values[1]\n        if dx != 0:\n            angle = math.degrees(math.atan2(dy, dx))\n            if angle <= threashold:\n                collected_results[header] = {\"maximum library diversity\": values[0]}\n                break\n\n\ndef collect_stats(args):\n    collected_results = {}\n    if args.trim1:\n        process_trimgalore_report(args.trim1, collected_results)\n    if args.trim2:\n        process_trimgalore_report(args.trim2, collected_results)\n    process_custom_report(args.bowtie, collected_results, \"alignment statistics\", BOWTIE)\n    process_custom_report(args.bamstats, collected_results, \"BAM statistics\", BAMSTATS, bool(args.paired))\n    process_custom_report(args.bamstatsfilter, collected_results, \"BAM statistics after filtering\", BAMSTATS, bool(args.paired))\n    process_custom_report(args.macs2, collected_results, \"peak calling statistics\", MACS2)\n    process_macs2_xls(args.macs2, collected_results, \"peak calling statistics\")\n    if args.preseq:\n        process_preseq_results(args.preseq, collected_results, \"library preparation\")\n    return (collected_results)\n\n\ndef export_results_yaml(collected_data, filepath):\n    with open(filepath+\".yaml\", 'w') as output_stream:\n        output_stream.write(yaml.dump(collected_data, width=1000, sort_keys=False))\n\n\ndef export_results_markdown(collected_data, filepath):\n    with open(filepath+\".md\", 'w') as output_stream:\n        for line in yaml.dump(collected_data, width=1000, sort_keys=False).split(\"\\n\"):\n            if not line.strip():\n                continue\n            if line.startswith(\"  - \"):\n                output_stream.write(line+\"\\n\")\n            elif line.startswith(\"    \"):\n                output_stream.write(\"<br>\"+line+\"\\n\")\n            elif line.startswith(\"  \"):\n                output_stream.write(\"- \"+line+\"\\n\")\n            else:\n                output_stream.write(\"### \"+line+\"\\n\")\n\n\ndef export_results_table(collected_data, filepath):\n    with open(filepath+\".tsv\", 'w') as output_stream:\n        total = collected_data[\"alignment statistics\"][\"total reads/pairs processed\"]\n        mapped = collected_data[\"BAM statistics after filtering\"][\"reads/pairs mapped\"]\n        multimapped = collected_data[\"alignment statistics\"].get(\"reads/pairs suppressed due to multimapping\", 0)\n        unmapped = collected_data[\"alignment statistics\"][\"reads/pairs unmapped\"]\n        filtered = collected_data[\"BAM statistics\"][\"reads/pairs mapped\"] - collected_data[\"BAM statistics after filtering\"][\"reads/pairs mapped\"]\n        header = [\n                    \"Tags total\",\n                    \"Mapped\",\n                    \"Multi-mapped\",\n                    \"Unmapped\",\n                    \"Filtered\",\n\n                    \"alignment statistics\",\n                    \"total reads/pairs processed\",\n                    \"reads/pairs with at least one reported alignment\",\n                    \"reads/pairs suppressed due to multimapping\",\n                    \"reads/pairs unmapped\",\n                    \n                    \"BAM statistics\",\n                    \"total reads/pairs\",\n                    \"reads/pairs mapped\",\n                    \"reads/pairs unmapped\",\n                    \"insert size average\",\n                    \"insert size standard deviation\",\n                    \"reads average length\",\n                    \"reads average quality\",\n                    \"reads maximum length\",\n\n                    \"BAM statistics after filtering\",\n                    \"total reads/pairs\",\n                    \"reads/pairs mapped\",\n                    \"reads/pairs unmapped\",\n                    \"insert size average\",\n                    \"insert size standard deviation\",\n                    \"reads average length\",\n                    \"reads average quality\",\n                    \"reads maximum length\",\n        \n                    \"peak calling statistics\",\n                    \"number of peaks called\",\n                    \"mean peak size\",\n                    \"total reads/pairs in treatment\",\n                    \"reads/pairs after filtering in treatment\",\n                    \"redundant rate in treatment\",\n                    \"fraction of reads in peaks\"]\n\n        if collected_data.get(\"adapter trimming statistics\", None):\n            header.extend([\"adapter trimming statistics\",\n                        \"trimming mode\",\n                        \"adapter sequence\",\n                        \"quality phred score cutoff\",\n                        \"minimum required adapter overlap\",\n                        \"maximum trimming error rate\",\n                        \"minimum required read length\",\n                        \"number of reads/pairs analysed for length validation\",\n                        \"reads/pairs removed because of length cutoff\",\n                        \"fastq1: total reads processed\",\n                        \"fastq1: reads with adapters\"])\n            if len(collected_data[\"adapter trimming statistics\"][\"fastq\"]) > 1:\n                header.extend([\"fastq2: total reads processed\", \"fastq2: reads with adapters\"])\n\n        output_stream.write(\"\\t\".join(header)+\"\\n\")\n\n        data = [\n                total,\n                mapped,\n                multimapped,\n                unmapped,\n                filtered,\n\n                \"\",\n                collected_data[\"alignment statistics\"][\"total reads/pairs processed\"],\n                collected_data[\"alignment statistics\"][\"reads/pairs with at least one reported alignment\"],\n                collected_data[\"alignment statistics\"].get(\"reads/pairs suppressed due to multimapping\", 0),\n                collected_data[\"alignment statistics\"][\"reads/pairs unmapped\"],\n                \n                \"\",\n                collected_data[\"BAM statistics\"][\"total reads/pairs\"],\n                collected_data[\"BAM statistics\"][\"reads/pairs mapped\"],\n                collected_data[\"BAM statistics\"][\"reads/pairs unmapped\"],\n                collected_data[\"BAM statistics\"][\"insert size average\"],\n                collected_data[\"BAM statistics\"][\"insert size standard deviation\"],\n                collected_data[\"BAM statistics\"][\"reads average length\"],\n                collected_data[\"BAM statistics\"][\"reads average quality\"],\n                collected_data[\"BAM statistics\"][\"reads maximum length\"],\n                \n                \"\",\n                collected_data[\"BAM statistics after filtering\"][\"total reads/pairs\"],\n                collected_data[\"BAM statistics after filtering\"][\"reads/pairs mapped\"],\n                collected_data[\"BAM statistics after filtering\"][\"reads/pairs unmapped\"],\n                collected_data[\"BAM statistics after filtering\"][\"insert size average\"],\n                collected_data[\"BAM statistics after filtering\"][\"insert size standard deviation\"],\n                collected_data[\"BAM statistics after filtering\"][\"reads average length\"],\n                collected_data[\"BAM statistics after filtering\"][\"reads average quality\"],\n                collected_data[\"BAM statistics after filtering\"][\"reads maximum length\"],\n                \n                \"\",\n                collected_data[\"peak calling statistics\"][\"number of peaks called\"],\n                collected_data[\"peak calling statistics\"][\"mean peak size\"],\n                collected_data[\"peak calling statistics\"][\"total reads/pairs in treatment\"],\n                collected_data[\"peak calling statistics\"][\"reads/pairs after filtering in treatment\"],\n                collected_data[\"peak calling statistics\"][\"redundant rate in treatment\"],\n                collected_data[\"peak calling statistics\"][\"fraction of reads in peaks\"]]\n\n        if collected_data.get(\"adapter trimming statistics\", None):\n            data.extend([\"\",\n                        collected_data[\"adapter trimming statistics\"][\"trimming mode\"],\n                        collected_data[\"adapter trimming statistics\"][\"adapter sequence\"],\n                        collected_data[\"adapter trimming statistics\"][\"quality phred score cutoff\"],\n                        collected_data[\"adapter trimming statistics\"][\"minimum required adapter overlap\"],\n                        collected_data[\"adapter trimming statistics\"][\"maximum trimming error rate\"],\n                        collected_data[\"adapter trimming statistics\"][\"minimum required read length\"],\n                        collected_data[\"adapter trimming statistics\"][\"number of reads/pairs analysed for length validation\"],\n                        collected_data[\"adapter trimming statistics\"][\"reads/pairs removed because of length cutoff\"],\n                        collected_data[\"adapter trimming statistics\"][\"fastq\"][0][\"total reads processed\"],\n                        collected_data[\"adapter trimming statistics\"][\"fastq\"][0][\"reads with adapters\"]])\n            if len(collected_data[\"adapter trimming statistics\"][\"fastq\"]) > 1:\n                data.extend([collected_data[\"adapter trimming statistics\"][\"fastq\"][1][\"total reads processed\"],\n                            collected_data[\"adapter trimming statistics\"][\"fastq\"][1][\"reads with adapters\"]])\n\n        output_stream.write(\"\\t\".join(str(l) for l in data))\n\n\ndef main(argsl=None):\n    if argsl is None:\n        argsl = sys.argv[1:]\n    args,_ = arg_parser().parse_known_args(argsl)\n    args = normalize_args(args, skip_list=[\"paired\"])\n    collected_data = collect_stats(args)\n    export_results_yaml(collected_data, args.output)\n    export_results_table(collected_data, args.output)\n    export_results_markdown(collected_data, args.output)\n\n\nif __name__ == \"__main__\":\n    sys.exit(main(sys.argv[1:]))\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "id": "#collect-statistics-chip-seq.cwl/script"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--trim1"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/trimgalore_report_fastq_1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--trim2"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/trimgalore_report_fastq_2"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(get_output_prefix()+\".md\")"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/collected_statistics_md"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(get_output_prefix()+\".tsv\")"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/collected_statistics_tsv"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(get_output_prefix()+\".yaml\")"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/collected_statistics_yaml"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "$(get_output_prefix()+\".tsv\")",
                        "outputEval": "$(parseInt(self[0].contents.split('\\n')[1].split('\\t')[1]))"
                    },
                    "id": "#collect-statistics-chip-seq.cwl/mapped_reads"
                }
            ],
            "baseCommand": [
                "python",
                "-c"
            ],
            "doc": "Tool processes and combines log files generated by Trimgalore, Bowtie, Samtools and MACS2.\n\n`get_output_prefix` function returns output file prefix equal to `output_prefix`+`_collected_statistics_report` (if this input is provided) or\ngenerated on the base of bowtie log basename with `_collected_statistics_report` extension.\n",
            "id": "#collect-statistics-chip-seq.cwl",
            "http://schema.org/name": "collect-statistics-chip-seq-trim",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/collect-statistics-chip-seq-trim.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Runs python code from the script input\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap:v0.0.3"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 6
                    },
                    "doc": "Compressed or uncompressed FASTQ file(s)\n",
                    "id": "#extract-fastq.cwl/compressed_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nshopt -s nocaseglob\nset -- \"$0\" \"$@\"\n\nfunction extract {\n  FILE=$1\n  COMBINED=$2\n  T=`file -b \"${FILE}\" | awk '{print $1}'`\n  case \"${T}\" in\n    \"bzip2\"|\"gzip\"|\"Zip\")\n      7z e -so \"${FILE}\" >> \"${COMBINED}\"\n      ;;\n    \"ASCII\")\n      cat \"${FILE}\" >> \"${COMBINED}\" || true\n      ;;\n    *)\n      echo \"Error: file type unknown\"\n      rm -f \"${COMBINED}\"\n      exit 1\n  esac\n} \n\nCOMBINED=\"\"\nfor FILE in \"$@\"; do\n    BASENAME=$(basename \"$FILE\")\n    ROOTNAME=\"${BASENAME%.*}\"\n    EXT_LIST=( \".fastq\" \".fq\" )\n    for ITEM in $EXT_LIST; do\n      if [[ $ROOTNAME == *$ITEM ]]; then\n        ROOTNAME=\"${ROOTNAME%.*}\"\n      fi\n    done\n    if [[ $COMBINED == \"\" ]]; then\n      COMBINED=\"${ROOTNAME}\"\n    else\n      COMBINED=\"${COMBINED}\"_\"${ROOTNAME}\"\n    fi\ndone\nCOMBINED=\"${COMBINED}\".fastq\n\nfor FILE in \"$@\"; do\n    echo \"Extracting:\" $FILE;\n    extract \"${FILE}\" \"${COMBINED}\"\ndone;\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Bash script to extract compressed FASTQ file\n",
                    "id": "#extract-fastq.cwl/script"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*"
                    },
                    "id": "#extract-fastq.cwl/fastq_file"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "Tool to decompress input FASTQ file(s).\nIf several FASTQ files are provided, they will be concatenated in the order that corresponds to files in input.\nBash script's logic:\n- disable case sensitive glob check\n- check if root name of input file already include '.fastq' or '.fq' extension. If yes, set DEFAULT_EXT to \"\",\n  otherwise use '.fastq'\n- check file type, decompress if needed\n- return 1, if file type is not recognized\nThis script also works of input file doesn't have any extension at all\n",
            "id": "#extract-fastq.cwl",
            "http://schema.org/name": "extract-fastq",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/extract-fastq.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Tool to decompress input FASTQ file"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { return inputs.input_file.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.') + \".fastxstat\" };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/fastx_toolkit:v0.0.14"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-i"
                    },
                    "doc": "FASTA/Q input file. If FASTA file is given, only nucleotides distribution is calculated (there's no quality info)\n",
                    "id": "#fastx-quality-stats.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-N"
                    },
                    "doc": "New output format (with more information per nucleotide/cycle).\ncycle (previously called 'column') = cycle number\nmax-count\nFor each nucleotide in the cycle (ALL/A/C/G/T/N):\n    count   = number of bases found in this column.\n    min     = Lowest quality score value found in this column.\n    max     = Highest quality score value found in this column.\n    sum     = Sum of quality score values for this column.\n    mean    = Mean quality score value for this column.\n    Q1\t= 1st quartile quality score.\n    med\t= Median quality score.\n    Q3\t= 3rd quartile quality score.\n    IQR\t= Inter-Quartile range (Q3-Q1).\n    lW\t= 'Left-Whisker' value (for boxplotting).\n    rW\t= 'Right-Whisker' value (for boxplotting).\n",
                    "id": "#fastx-quality-stats.cwl/new_output_format"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-o",
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Output file to store generated statistics. If not provided - return from default_output_filename function\n",
                    "id": "#fastx-quality-stats.cwl/output_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_filename == \"\"){\n      return default_output_filename();\n    } else {\n      return inputs.output_filename;\n    }\n}\n"
                    },
                    "doc": "Generated statistics file",
                    "id": "#fastx-quality-stats.cwl/statistics_file"
                }
            ],
            "baseCommand": [
                "fastx_quality_stats"
            ],
            "doc": "Tool calculates statistics on the base of FASTQ file quality scores.\nIf `output_filename` is not provided call function `default_output_filename` to return default output file name\ngenerated as `input_file` basename + `.fastxstat` extension.\n",
            "id": "#fastx-quality-stats.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#fastx-toolkit-metadata.yaml",
                "http://schema.org/name": "fastx-toolkit",
                "http://schema.org/about": "The FASTX-Toolkit is a collection of command line tools for Short-Reads FASTA/FASTQ files preprocessing. Available Tools:\n  FASTQ-to-FASTA converter (Convert FASTQ files to FASTA files)\n  FASTQ Information (Chart Quality Statistics and Nucleotide Distribution)\n  FASTQ/A Collapser (Collapsing identical sequences in a FASTQ/A file into a single sequence)\n  FASTQ/A Trimmer (Shortening reads in a FASTQ or FASTQ files, removing barcodes or noise)\n  FASTQ/A Renamer (Renames the sequence identifiers in FASTQ/A file)\n  FASTQ/A Clipper (Removing sequencing adapters/linkers)\n  FASTQ/A Reverse-Complement (Producing the Reverse-complement of each sequence in a FASTQ/FASTA file)\n  FASTQ/A Barcode splitter (Splitting a FASTQ/FASTA files containning multiple samples)\n  FASTA Formatter (changes the width of sequences line in a FASTA file)\n  FASTA Nucleotide Changer (Convets FASTA sequences from/to RNA/DNA)\n  FASTQ Quality Filter (Filters sequences based on quality)\n  FASTQ Quality Trimmer (Trims (cuts) sequences based on quality)\n  FASTQ Masker (Masks nucleotides with 'N' (or other character) based on quality)\n",
                "http://schema.org/url": "http://hannonlab.cshl.edu/fastx_toolkit/index.html",
                "http://schema.org/codeRepository": "https://github.com/agordon/fastx_toolkit",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "0.0.14",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C",
                "http://schema.org/discussionUrl": [
                    "https://github.com/agordon/fastx_toolkit/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "The Hannon Lab",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Assaf Gordon",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "fastx-quality-stats",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/fastx-quality-stats.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage: fastx_quality_stats [-h] [-N] [-i INFILE] [-o OUTFILE]\nPart of FASTX Toolkit 0.0.14 by A. Gordon (assafgordon@gmail.com)\n   [-h] = This helpful help screen.\n   [-i INFILE]  = FASTQ input file. default is STDIN.\n   [-o OUTFILE] = TEXT output file. default is STDOUT.\n   [-N]         = New output format (with more information per nucleotide/cycle).\nThe *OLD* output TEXT file will have the following fields (one row per column):\n\tcolumn\t= column number (1 to 36 for a 36-cycles read solexa file)\n\tcount   = number of bases found in this column.\n\tmin     = Lowest quality score value found in this column.\n\tmax     = Highest quality score value found in this column.\n\tsum     = Sum of quality score values for this column.\n\tmean    = Mean quality score value for this column.\n\tQ1\t= 1st quartile quality score.\n\tmed\t= Median quality score.\n\tQ3\t= 3rd quartile quality score.\n\tIQR\t= Inter-Quartile range (Q3-Q1).\n\tlW\t= 'Left-Whisker' value (for boxplotting).\n\trW\t= 'Right-Whisker' value (for boxplotting).\n\tA_Count\t= Count of 'A' nucleotides found in this column.\n\tC_Count\t= Count of 'C' nucleotides found in this column.\n\tG_Count\t= Count of 'G' nucleotides found in this column.\n\tT_Count\t= Count of 'T' nucleotides found in this column.\n\tN_Count = Count of 'N' nucleotides found in this column.\n\tmax-count = max. number of bases (in all cycles)\nThe *NEW* output format:\n\tcycle (previously called 'column') = cycle number\n\tmax-count\n  For each nucleotide in the cycle (ALL/A/C/G/T/N):\n\t\tcount   = number of bases found in this column.\n\t\tmin     = Lowest quality score value found in this column.\n\t\tmax     = Highest quality score value found in this column.\n\t\tsum     = Sum of quality score values for this column.\n\t\tmean    = Mean quality score value for this column.\n\t\tQ1\t= 1st quartile quality score.\n\t\tmed\t= Median quality score.\n\t\tQ3\t= 3rd quartile quality score.\n\t\tIQR\t= Inter-Quartile range (Q3-Q1).\n\t\tlW\t= 'Left-Whisker' value (for boxplotting).\n\t\trW\t= 'Right-Whisker' value (for boxplotting).\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function(ext) { var root = inputs.input_filename.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.input_filename.basename+ext:root+ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/iaintersect:v0.0.2"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--a=",
                        "separate": false
                    },
                    "doc": "Annotation file, tsv",
                    "id": "#iaintersect.cwl/annotation_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--sam_ignorechr=",
                        "separate": false
                    },
                    "doc": "The chromosome to be ignored, string\n",
                    "id": "#iaintersect.cwl/ignore_chr"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--in=",
                        "separate": false
                    },
                    "doc": "Input filename with MACS2 peak calling results, tsv",
                    "id": "#iaintersect.cwl/input_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 4,
                        "prefix": "--log=",
                        "separate": false,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename('_iaintersect.log');\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Log filename\n",
                    "id": "#iaintersect.cwl/log_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 3,
                        "prefix": "--out=",
                        "separate": false,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename('_iaintersect.tsv');\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Base output file name, tsv\n",
                    "id": "#iaintersect.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--promoter=",
                        "separate": false
                    },
                    "doc": "Promoter region around TSS, base pairs\n",
                    "id": "#iaintersect.cwl/promoter_bp"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--upstream=",
                        "separate": false
                    },
                    "doc": "Upstream region before promoter, base pairs\n",
                    "id": "#iaintersect.cwl/upstream_bp"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.log_filename == \"\"){\n    return default_output_filename('_iaintersect.log');\n  } else {\n    return inputs.log_filename;\n  }\n}\n"
                    },
                    "id": "#iaintersect.cwl/log_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_filename == \"\"){\n    return default_output_filename('_iaintersect.tsv');\n  } else {\n    return inputs.output_filename;\n  }\n}\n"
                    },
                    "id": "#iaintersect.cwl/result_file"
                }
            ],
            "baseCommand": [
                "iaintersect"
            ],
            "doc": "Tool assigns each peak obtained from MACS2 to a gene and region (upstream, promoter, exon, intron, intergenic)\n\n`default_output_filename` function returns output filename with sufix set as `ext` argument. Function is called when\neither `output_filename` or `log_filename` inputs are not provided.\n",
            "id": "#iaintersect.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#iaintersect-metadata.yaml",
                "http://schema.org/name": "iaintersect",
                "http://schema.org/about": "Based on BioWardrobe iaintersect (https://github.com/cincinnati-childrens-hospital/biowardrobe/tree/master/src/iaintersect) The software assigns each peak obtained from MACS2 to a gene and region (upstream, promoter, exon, intron, intergenic).\n",
                "http://schema.org/url": "https://github.com/Barski-lab/iaintersect/blob/master/README.md",
                "http://schema.org/codeRepository": "https://github.com/Barski-lab/iaintersect",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "0.0.1",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/discussionUrl": [
                    "https://github.com/Barski-lab/iaintersect/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Barski Research Lab",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Andrey Kartashov",
                                "http://schema.org/description": "Main author"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Michael Kotliar",
                                "http://schema.org/description": "Update to work with files instead of DB"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "iaintersect",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/iaintersect.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:\n  iaintersect [options] --in=pathToFile --a=pathtoFile --out=pathToFile\n  --a                        \tTab-separated annotation file\n  --in                       \tInput filename with MACS2 peak calling results, xls\n  --log                      \tLog filename (default is ./logfile_def.log)\n  --out                      \tBase output file name\n  --promoter                 \tPromoter region around TSS in bp\n  --sam_ignorechr            \tThe chromosome to be ignored\n  --upstream                 \tUpstream region before promoter in bp\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_output_filename = function() { if (inputs.output_filename) { return inputs.output_filename; } return inputs.unsorted_file.location.split('/').slice(-1)[0]; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap:v0.0.2"
                }
            ],
            "inputs": [
                {
                    "type": {
                        "type": "array",
                        "items": "string",
                        "inputBinding": {
                            "prefix": "-k"
                        }
                    },
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "-k, --key=POS1[,POS2]\nstart a key at POS1, end it at POS2 (origin 1)\n",
                    "id": "#linux-sort.cwl/key"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Name for generated output file\n",
                    "id": "#linux-sort.cwl/output_filename"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4
                    },
                    "id": "#linux-sort.cwl/unsorted_file"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "id": "#linux-sort.cwl/sorted_file",
                    "outputBinding": {
                        "glob": "$(get_output_filename())"
                    }
                }
            ],
            "stdout": "$(get_output_filename())",
            "baseCommand": [
                "sort"
            ],
            "doc": "Tool sorts data from `unsorted_file` by key\n\n`default_output_filename` function returns file name identical to `unsorted_file`, if `output_filename` is not provided.\n",
            "id": "#linux-sort.cwl",
            "http://schema.org/name": "linux-sort",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/linux-sort.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function(sufix) { sufix = sufix || \"_macs\"; if (Object.prototype.toString.call(inputs.treatment_file) === '[object Array]'){ var basename = inputs.treatment_file[0].basename; var root = basename.split('.').slice(0,-1).join('.'); return (root == \"\")?basename+sufix:root+sufix; } else { var basename = inputs.treatment_file.basename; var root = basename.split('.').slice(0,-1).join('.'); return (root == \"\")?basename+sufix:root+sufix; } }"
                    ]
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": [
                        {
                            "entryname": "run.py",
                            "entry": "#!/usr/bin/env python\nimport subprocess\nimport sys\nimport argparse\nimport re\n\ndef arg_parser():\n    \"\"\"Returns argument parser. used only to get -n and --extsize values\"\"\"\n    parser = argparse.ArgumentParser(description='MACS2 Biowardrobe parser')\n    parser.add_argument(\"-n\", help=\"MACS2 string name of the experiment\")\n    parser.add_argument(\"--extsize\", help=\"MACS2 arbitrary extension size in bp\")\n    return parser\n\ndef get_info (name):\n    fragments, islands = 0, 0\n    with open(name+'_peaks.xls', 'r') as infile:\n        for line in infile:\n            if re.match('^# d = ', line):\n                fragments = int(line.split('d = ')[1])\n                continue\n            if re.match('^#', line):\n                continue\n            if line.strip() != \"\":\n                islands = islands + 1\n    islands = islands - 1\n    return fragments, islands\n\nargs, _ = arg_parser().parse_known_args()\nargv=' '.join(sys.argv[1:])\n\nextsize=args.extsize\nname=args.n\n\nruntime_error=False\ncalculated_fragment_size=0\nexpected_fragment_size=0\nisland_count=0\n\ntry:\n    subprocess.check_output(argv, shell=True)\nexcept subprocess.CalledProcessError as e:\n    if '--nomodel' in argv:\n        print \"MACS2 critical error: \", str(e)\n        sys.exit(1) # stop the whole workflow\n    print \"MACS2 will be rerun to fix not critical error: \", str(e)\n    runtime_error=True\n    expected_fragment_size=extsize\nelse:\n    calculated_fragment_size,island_count=get_info(name)\n    expected_fragment_size=calculated_fragment_size\n\nif runtime_error or (calculated_fragment_size < 80 and ('--nomodel' not in argv)):\n    try:\n        subprocess.check_output(argv+' --nomodel', shell=True)\n    except subprocess.CalledProcessError as e:\n        print \"MACS2 forced error: \", str(e)\n        sys.exit(1) # stop the whole workflow\n    else:\n        calculated_fragment_size,island_count=get_info(name)\n\nwith open(name.replace('_macs','')+'_fragment_stat.tsv', 'w') as output_file:\n    output_file.write(' '.join([str(calculated_fragment_size),str(expected_fragment_size),str(island_count)]))\n"
                        }
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/macs2:v2.1.1"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 17,
                        "prefix": "--bdg"
                    },
                    "doc": "If this flag is on, MACS will store the fragment pileup, control lambda, -log10pvalue and -log10qvalue scores in bedGraph files.\nThe bedGraph files will be stored in current directory named NAME+\u2019_treat_pileup.bdg\u2019 for treatment data, NAME+\u2019_control_lambda.bdg\u2019\nfor local lambda values from control, NAME+\u2019_treat_pvalue.bdg\u2019 for Poisson pvalue scores (in -log10(pvalue) form),\nand NAME+\u2019_treat_qvalue.bdg\u2019 for q-value scores from\nBenjamini\u2013Hochberg\u2013Yekutieli procedure <http://en.wikipedia.org/wiki/False_discovery_rate#Dependent_tests>\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/bdg"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 36,
                        "prefix": "--broad"
                    },
                    "doc": "If set, MACS will try to call broad peaks by linking nearby highly enriched\nregions. The linking region is controlled by another cutoff through --linking-cutoff.\nThe maximum linking region length is 4 times of d from MACS.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/broad"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 37,
                        "prefix": "--broad-cutoff"
                    },
                    "doc": "Cutoff for broad region. This option is not available\nunless --broad is set. If -p is set, this is a pvalue\ncutoff, otherwise, it's a qvalue cutoff.\nDEFAULT: 0.1\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/broad_cutoff"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 16,
                        "prefix": "--buffer-size"
                    },
                    "doc": "Buffer size for incrementally increasing internal\narray size to store reads alignment information. In\nmost cases, you don't have to change this parameter.\nHowever, if there are large number of\nchromosomes/contigs/scaffolds in your alignment, it's\nrecommended to specify a smaller buffer size in order\nto decrease memory usage (but it will take longer time\nto read alignment files). Minimum memory requested for\nreading an alignment file is about # of CHROMOSOME *\nBUFFER_SIZE * 2 Bytes.\nDEFAULT: 100000\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/buffer_size"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 21,
                        "prefix": "--bw"
                    },
                    "doc": "Band width for picking regions to compute  fragment  size.  This\nvalue is only used while building the shifting model\nDEFAULT: 300\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/bw"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 39,
                        "prefix": "--call-summits"
                    },
                    "doc": "If set, MACS will use a more sophisticated signal processing approach to\nfind subpeak summits in each enriched peak region.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/call_summits"
                },
                {
                    "type": [
                        "null",
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-c"
                    },
                    "doc": "The control or mock data file. Please follow the same direction as for -t/\u2013treatment.\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/control_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 38,
                        "prefix": "--cutoff-analysis"
                    },
                    "doc": "While set, MACS2 will analyze number or total length of peaks that can be\ncalled by different p-value cutoff then output a summary table to help user\ndecide a better cutoff. The table will be saved in NAME_cutoff_analysis.txt\nfile. Note, minlen and maxgap may affect the results. WARNING: May take ~30\nfolds longer time to finish.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/cutoff_analysis"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 31,
                        "prefix": "--down-sample"
                    },
                    "doc": "When set, random sampling method will scale down the bigger sample. By default,\nMACS uses linear scaling. Warning: This option will make your result unstable\nand irreproducible since each time, random reads would be selected. Consider\nto use ''randsample'' script instead. <not implmented>If used together with\n--SPMR, 1 million unique reads will be randomly picked.</not implemented> Caution:\ndue to the implementation, the final number of selected reads may not be as\nyou expected!\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/down_sample"
                },
                {
                    "type": "int",
                    "inputBinding": {
                        "position": 26,
                        "prefix": "--extsize"
                    },
                    "doc": "The arbitrary extension size in bp. When nomodel is  true, MACS will use\nthis value as fragment size to  extend each read towards 3'' end, then pile\nthem up.  It''s exactly twice the number of obsolete SHIFTSIZE.  In previous\nlanguage, each read is moved 5''->3''  direction to middle of fragment by 1/2\nd, then  extended to both direction with 1/2 d. This is  equivalent to say each\nread is extended towards 5''->3''  into a d size fragment.EXTSIZE\nand  SHIFT can be combined when necessary. Check SHIFT  option.\nDEFAULT: 200\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/extsize"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 40,
                        "prefix": "--fe-cutoff"
                    },
                    "doc": "When set, the value will be used to filter out peaks\nwith low fold-enrichment. Note, MACS2 use 1.0 as\npseudocount while calculating fold-enrichment.\nDEFAULT: 1.0\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/fe_cutoff"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 23,
                        "prefix": "--fix-bimodal"
                    },
                    "doc": "Whether turn on the auto pair model process. If set, when MACS failed to\nbuild paired model, it will use the nomodel settings, the --exsize parameter\nto extend each tags towards 3'' direction. Not to use this automate fixation\nis a default behavior now.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/fix_bimodal"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "-f"
                    },
                    "doc": "{AUTO,BAM,SAM,BED,ELAND,ELANDMULTI,ELANDEXPORT,BOWTIE,BAMPE}, --format\n{AUTO,BAM,SAM,BED,ELAND,ELANDMULTI,ELANDEXPORT,BOWTIE,BAMPE} Format of tag file,\n\"AUTO\", \"BED\" or \"ELAND\" or \"ELANDMULTI\" or \"ELANDEXPORT\" or \"SAM\" or \"BAM\"\nor \"BOWTIE\" or \"BAMPE\". The default AUTO option will let MACS decide which format\nthe file is. Note that MACS can''t detect \"BAMPE\" or \"BEDPE\" format with \"AUTO\",\nand you have to implicitly specify the format for \"BAMPE\" and \"BEDPE\".\nDEFAULT: AUTO\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/format_mode"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "-g"
                    },
                    "doc": "It\u2019s the mappable genome size or effective genome size which is defined as the genome size which can be sequenced.\nBecause of the repetitive features on the chromsomes, the actual mappable genome size will be smaller than the\noriginal size, about 90% or 70% of the genome size. The default hs \u2013 2.7e9 is recommended for UCSC human hg18\nassembly. Here are all precompiled parameters for effective genome size:\n  hs:\t2.7e9\n  mm:\t1.87e9\n  ce:\t9e7\n  dm:\t1.2e8\nDEFAULT: hs\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/genome_size"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "--keep-dup"
                    },
                    "doc": "It controls the MACS behavior towards duplicate tags\nat the exact same location -- the same coordination\nand the same strand. The 'auto' option makes MACS\ncalculate the maximum tags at the exact same location\nbased on binomal distribution using 1e-5 as pvalue\ncutoff; and the 'all' option keeps every tags. If an\ninteger is given, at most this number of tags will be\nkept at the same location. The default is to keep one\ntag at the same location.\nDEFAULT: 1\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/keep_dup"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 35,
                        "prefix": "--llocal"
                    },
                    "doc": "The large nearby region in basepairs to calculate\ndynamic lambda. This is used to capture the surround\nbias. If you set this to 0, MACS will skip llocal\nlambda calculation. *Note* that MACS will always\nperform a d-size local lambda calculation. The final\nlocal bias should be the maximum of the lambda value\nfrom d, slocal, and llocal size windows.\nDEFAULT: 10000.\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/llocal"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 22,
                        "prefix": "-m"
                    },
                    "doc": "Select the regions within MFOLD range of high-\nconfidence enrichment ratio against background to\nbuild model. Fold-enrichment in regions must be lower\nthan upper limit, and higher than the lower limit. Use\nas \"-m 10 30\"\nDEFAULT: 5 50\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/mfold"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 33,
                        "prefix": "--nolambda"
                    },
                    "doc": "If True, MACS will use fixed background lambda as local lambda for every\npeak region. Normally, MACS calculates a dynamic local lambda to reflect the\nlocal bias due to potential chromatin structure.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/nolambda"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 24,
                        "prefix": "--nomodel"
                    },
                    "doc": "Whether or not to build the shifting model. If True,  MACS will not build\nmodel. by default it means  shifting size = 100, try to set extsize to change it.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/nomodel"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 999,
                        "prefix": "-n",
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "The name string of the experiment. MACS will use this string NAME to create output files like \u2018NAME_peaks.xls\u2019,\n\u2018NAME_negative_peaks.xls\u2019, \u2018NAME_peaks.bed\u2019 , \u2018NAME_summits.bed\u2019, \u2018NAME_model.r\u2019 and so on.\nSo please avoid any confliction between these filenames and your existing files.\nDEFAULT: generated on the base of the treatment input\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/output_prefix"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 28,
                        "prefix": "-p"
                    },
                    "doc": "Pvalue cutoff for peak detection. DEFAULT: not set.  -q, and -p are mutually\nexclusive. If pvalue cutoff is set, qvalue will not be calculated and reported\nas -1  in the final .xls file.\nDEFAULT: null\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/p_value"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 27,
                        "prefix": "-q"
                    },
                    "doc": "Minimum FDR (q-value) cutoff for peak detection. -q, and\n-p are mutually exclusive.\nDEFAULT: 0.05\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/q_value"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 30,
                        "prefix": "--ratio"
                    },
                    "doc": "When set, use a custom scaling ratio of ChIP/control\n(e.g. calculated using NCIS) for linear scaling.\nDEFAULT: null\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/ratio"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 32,
                        "prefix": "--seed"
                    },
                    "doc": "Set the random seed while down sampling data. Must be\na non-negative integer in order to be effective.\nDEFAULT: null\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/seed"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 25,
                        "prefix": "--shift"
                    },
                    "doc": "(NOT the legacy --shiftsize option!) The arbitrary shift in bp. Use discretion\nwhile setting it other than default value. When NOMODEL is set, MACS will use\nthis value to move cutting ends (5'') towards 5''->3'' direction then apply\nEXTSIZE to extend them to fragments. When this value is negative, ends will\nbe moved toward 3''->5'' direction. Recommended to keep it as default 0 for\nChIP-Seq datasets, or -1 * half of EXTSIZE together with EXTSIZE option for\ndetecting enriched cutting loci such as certain DNAseI-Seq datasets. Note, you\ncan''t set values other than 0 if format is BAMPE for paired-end data.\nDEFAULT: 0\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/shift"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 34,
                        "prefix": "--slocal"
                    },
                    "doc": "The small nearby region in basepairs to calculate\ndynamic lambda. This is used to capture the bias near\nthe peak summit region. Invalid if there is no control\ndata. If you set this to 0, MACS will skip slocal\nlambda calculation. *Note* that MACS will always\nperform a d-size local lambda calculation. The final\nlocal bias should be the maximum of the lambda value\nfrom d, slocal, and llocal size windows.\nDEFAULT: 1000\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/slocal"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 19,
                        "prefix": "--SPMR"
                    },
                    "doc": "If True, MACS will save signal per million reads for fragment pileup profiles.\nRequire --bdg to be set.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/spmr"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 29,
                        "prefix": "--to-large"
                    },
                    "doc": "When set, scale the small sample up to the bigger sample. By default, the\nbigger dataset will be scaled down towards the smaller dataset, which will lead\nto smaller p/qvalues and more specific results. Keep in mind that scaling down\nwill bring down background noise more.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/to_large"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 18,
                        "prefix": "--trackline"
                    },
                    "doc": "Tells MACS to include trackline with bedGraph files. To include this trackline\nwhile displaying bedGraph at UCSC genome browser, can show name and description\nof the file as well. Require -B to be set.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/trackline"
                },
                {
                    "type": [
                        "File",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-t"
                    },
                    "doc": "This is the only REQUIRED parameter for MACS. File can be in any supported format specified by \u2013format option.\nCheck \u2013format for detail. If you have more than one alignment files, you can specify them as `-t A B C`.\nMACS will pool up all these files together.\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/treatment_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 20,
                        "prefix": "--tsize"
                    },
                    "doc": "Tag size. This will overide the auto detected tag size.\nDEFAULT: False\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/tsize"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 41,
                        "prefix": "--verbose"
                    },
                    "doc": "Log level\n",
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/verbose"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${ if (inputs.output_prefix == \"\"){ return default_output_filename('_macs_peaks.broadPeak'); } else { return inputs.output_prefix + '_peaks.broadPeak'; } }"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/broad_peak_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${ if (inputs.output_prefix == \"\"){ return default_output_filename('_macs_control_lambda.bdg'); } else { return inputs.output_prefix + '_control_lambda.bdg'; } }"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/control_lambda_bdg_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${ if (inputs.output_prefix == \"\"){ return default_output_filename('_macs_peaks.gappedPeak'); } else { return inputs.output_prefix + '_peaks.gappedPeak'; } }"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/gapped_peak_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "${\n    if (inputs.output_prefix == \"\"){\n      return default_output_filename('_fragment_stat.tsv');\n    } else {\n      return inputs.output_prefix + '_fragment_stat.tsv';\n    }\n}\n",
                        "outputEval": "$(parseInt(self[0].contents.split(' ')[0]))"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/macs2_fragments_calculated"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "${\n    if (inputs.output_prefix == \"\"){\n      return default_output_filename('_fragment_stat.tsv');\n    } else {\n      return inputs.output_prefix + '_fragment_stat.tsv';\n    }\n}\n",
                        "outputEval": "$(parseInt(self[0].contents.split(' ')[1]))"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/macs2_fragments_expected"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "${\n    if (inputs.output_prefix == \"\"){\n      return default_output_filename('_fragment_stat.tsv');\n    } else {\n      return inputs.output_prefix + '_fragment_stat.tsv';\n    }\n}\n",
                        "outputEval": "$(parseInt(self[0].contents.split(' ')[2]))"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/macs2_islands"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_prefix == \"\"){\n      return default_output_filename('_fragment_stat.tsv');\n    } else {\n      return inputs.output_prefix + '_fragment_stat.tsv';\n    }\n}\n"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/macs2_stat_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_prefix == \"\"){\n      return default_output_filename('_macs.log');\n    } else {\n      return inputs.output_prefix + '.log';\n    }\n}\n"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/macs_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${ if (inputs.output_prefix == \"\"){ return default_output_filename('_macs_model.r'); } else { return inputs.output_prefix + '_model.r'; } }"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/moder_r_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${ if (inputs.output_prefix == \"\"){ return default_output_filename('_macs_peaks.narrowPeak'); } else { return inputs.output_prefix + '_peaks.narrowPeak'; } }"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/narrow_peak_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${ if (inputs.output_prefix == \"\"){ return default_output_filename('_macs_summits.bed'); } else { return inputs.output_prefix + '_summits.bed'; } }"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/peak_summits_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${ if (inputs.output_prefix == \"\"){ return default_output_filename('_macs_peaks.xls'); } else { return inputs.output_prefix + '_peaks.xls'; } }"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/peak_xls_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${ if (inputs.output_prefix == \"\"){ return default_output_filename('_macs_treat_pileup.bdg'); } else { return inputs.output_prefix + '_treat_pileup.bdg'; } }"
                    },
                    "id": "#macs2-callpeak-biowardrobe-only.cwl/treat_pileup_bdg_file"
                }
            ],
            "baseCommand": [
                "python",
                "run.py",
                "macs2 callpeak"
            ],
            "arguments": [
                {
                    "valueFrom": "${ if (inputs.output_prefix == \"\"){ return ' 2> ' + default_output_filename('_macs.log'); } else { return ' 2> ' + inputs.output_prefix + '.log'; } }",
                    "position": 100000,
                    "shellQuote": true
                }
            ],
            "doc": "Tool runs peak calling using MACS2 within the custom script `run.py`\n\n`default_output_filename` function returns default output prefix for all generated output files. Function is called\nwhen `output_prefix` input is not set. Optional argument `sufix` is equal to `_macs2` by default.\n`sufix` is hardcoded and should not be changed (if necessary, update `_macs2` in all tool outputs and\n`run.py` script too). `sufix` is included in the filenames of all output files except those, who ends\nwith `_fragment_stat.tsv`. For the rest of the output files `_macs2` is already included as a part of the\n`sufix` argument when `default_output_filename` function is called. This is mostly done because of the\nnaming conventions for output files in supplemental project `https://github.com/Barski-lab/biowardrobe-analysis`\n(if `_fragment_stat.tsv` is updated to `_macs2_fragment_stat.tsv` in `biowardrobe-analysis`, the way we use\n`default_output_filename` function can be simplified). The basename for output filename is taken from\n`treatment_file` input (if array - the first item is used).\n\n`baseCommand` runs Python script `run.py` staged into the output directory. Within this script MACS2 is called one\nortwo times as a subprocess with all provided input parameters.\n\n`extsize` is mandatory parameter. It will be ignored by MASC2, when `--nomodel` is not set.\n\nThe aim of `run.py` script is to return `calculated_fragment_size`, `expected_fragment_size` and `island_count` as\na text file.\n\nScenarios:\n1. Run MACS2 with all input parameters.\n   If MACS2 first run failed and `--nomodel` wasn't set (it means that `--extsize` wasn't used), set\n   `expected_fragment_size` to `--extsize` value and try to rerun MACS2 with `--nomodel` parameter. If MACS2 second\n   run failed too - exit workflow, if it didn't fail - get values for `calculated_fragment_size` and `island_count`\n   from generated output file.\n   Returned data:\n    `calculated_fragment_size` - the value of fragment size from MACS2 second run\n    `expected_fragment_size` - the value of fragment size equal to `--extsize` (set when MACS2 first run failed)\n    `island_count` - the number of islands from MACS2 second run\n\n2. Run MACS2 with all input parameters.\n   If MACS2 first run failed and `--nomodel` was already set, exit workflow, because there is no need to rerun\n   MACS2, it will fail too.\n\n3. Run MACS2 with all input parameters.\n   If MACS2 first run didn't fail, we get `calculated_fragment_size` and `island_count` from generated output file,\n   and set `expected_fragment_size` to be equal to `calculated_fragment_size`. If `calculated_fragment_size` is\n   bigger or equal to 80, there is no need to rerun MACS2.\n   Returned data:\n    `calculated_fragment_size` - the value of fragment size from MACS2 first run\n    `expected_fragment_size` - the value of fragment size equal to `calculated_fragment_size`\n    `island_count` - the number of islands from MACS2 first run\n\n4. Run MACS2 with all input parameters.\n   If MACS2 first run didn't fail, we get `calculated_fragment_size` and `island_count` from generated output file,\n   and set `expected_fragment_size` to be equal to `calculated_fragment_size`. If `calculated_fragment_size` is less\n   then 80 and MACS2 was run without '--nomodel' (it means we didn't force it to use the fixed fragment size\n   `--extsize`), we rerun MACS2 with `--nomodel` argument. If MACS2 second run failed too - exit workflow, if not -\n   get values for `calculated_fragment_size` and `island_count` from generated output file.\n   Returned data:\n    `calculated_fragment_size` - the value of fragment size from MACS2 second run\n    `expected_fragment_size` - the value of fragment size from MACS2 first run\n    `island_count` - the number of islands from MACS2 second run\n\n5. Run MACS2 with all input parameters.\n   If MACS2 first run didn't fail and was already run with '--nomodel' parameter (forced to use fixed fragment size\n   `--extsize`), we get `calculated_fragment_size` and `island_count` from generated output file, and set\n   `expected_fragment_size` to be equal to `calculated_fragment_size`. No mater which value we got for\n   `calculated_fragment_size`, we don't rerun MACS2.\n   Returned data:\n    `calculated_fragment_size` - the value of fragment size got from MACS2 first run\n    `expected_fragment_size` - the value of fragment size equal to `calculated_fragment_size`\n    `island_count` - the number of islands got from MACS2 first run of MACS2\n",
            "id": "#macs2-callpeak-biowardrobe-only.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#macs2-metadata.yaml",
                "http://schema.org/name": "MACS2",
                "http://schema.org/about": "With the improvement of sequencing techniques, chromatin immunoprecipitation followed by high throughput\nsequencing (ChIP-Seq) is getting popular to study genome-wide protein-DNA interactions. To address the\nlack of powerful ChIP-Seq analysis method, we present a novel algorithm, named Model-based Analysis\nof ChIP-Seq (MACS), for identifying transcript factor binding sites. MACS captures the influence of\ngenome complexity to evaluate the significance of enriched ChIP regions, and MACS improves the spatial\nresolution of binding sites through combining the information of both sequencing tag position and\norientation. MACS can be easily used for ChIP-Seq data alone, or with control sample with the increase\nof specificity.\n",
                "http://schema.org/url": "http://liulab.dfci.harvard.edu/MACS/",
                "http://schema.org/codeRepository": "https://github.com/taoliu/MACS",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/BSD-3-Clause"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "2.1.1.20160309",
                    "http://schema.org/applicationCategory": "command line tool"
                },
                "http://schema.org/programmingLanguage": "Python",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#macs2-metadata.yaml/10.1186/gb-2008-9-9-r137",
                        "datePublished": "17 September 2008",
                        "http://schema.org/name": "Model-based analysis of ChIP-Seq (MACS)",
                        "http://schema.org/author": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Zhang Y"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Liu T"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Meyer CA"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Eeckhoute J"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Johnson DS"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Bernstein BE"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Nusbaum C"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Myers RM"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Brown M"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Li W"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Liu XS"
                            }
                        ],
                        "http://schema.org/url": "https://genomebiology.biomedcentral.com/articles/10.1186/gb-2008-9-9-r137"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://groups.google.com/forum/#!forum/macs-announcement",
                    "https://github.com/taoliu/MACS/issues"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Xiaole Shirley Liu's Lab",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Yong Zhang",
                                "http://schema.org/description": "Main author"
                            },
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Tao Liu",
                                "http://schema.org/description": "Main author"
                            }
                        ]
                    }
                ]
            },
            "http://schema.org/name": "macs2-callpeak",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/macs2-callpeak.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage: macs2 callpeak [-h] -t TFILE [TFILE ...] [-c [CFILE [CFILE ...]]]\n                      [-f {AUTO,BAM,SAM,BED,ELAND,ELANDMULTI,ELANDEXPORT,BOWTIE,BAMPE,BEDPE}]\n                      [-g GSIZE] [--keep-dup KEEPDUPLICATES]\n                      [--buffer-size BUFFER_SIZE] [--outdir OUTDIR] [-n NAME]\n                      [-B] [--verbose VERBOSE] [--trackline] [--SPMR]\n                      [-s TSIZE] [--bw BW] [-m MFOLD MFOLD] [--fix-bimodal]\n                      [--nomodel] [--shift SHIFT] [--extsize EXTSIZE]\n                      [-q QVALUE | -p PVALUE] [--to-large] [--ratio RATIO]\n                      [--down-sample] [--seed SEED] [--tempdir TEMPDIR]\n                      [--nolambda] [--slocal SMALLLOCAL] [--llocal LARGELOCAL]\n                      [--broad] [--broad-cutoff BROADCUTOFF]\n                      [--cutoff-analysis] [--call-summits]\n                      [--fe-cutoff FECUTOFF]\n\noptional arguments:\n  -h, --help            show this help message and exit\n\nInput files arguments:\n  -t TFILE [TFILE ...], --treatment TFILE [TFILE ...]\n                        ChIP-seq treatment file. If multiple files are given\n                        as '-t A B C', then they will all be read and pooled\n                        together. REQUIRED.\n  -c [CFILE [CFILE ...]], --control [CFILE [CFILE ...]]\n                        Control file. If multiple files are given as '-c A B\n                        C', they will be pooled to estimate ChIP-seq\n                        background noise.\n  -f {AUTO,BAM,SAM,BED,ELAND,ELANDMULTI,ELANDEXPORT,BOWTIE,BAMPE,BEDPE}, --format {AUTO,BAM,SAM,BED,ELAND,ELANDMULTI,ELANDEXPORT,BOWTIE,BAMPE,BEDPE}\n                        Format of tag file, \"AUTO\", \"BED\" or \"ELAND\" or\n                        \"ELANDMULTI\" or \"ELANDEXPORT\" or \"SAM\" or \"BAM\" or\n                        \"BOWTIE\" or \"BAMPE\" or \"BEDPE\". The default AUTO\n                        option will let MACS decide which format (except for\n                        BAMPE and BEDPE which should be implicitly set) the\n                        file is. Please check the definition in README. Please\n                        note that if the format is set as BAMPE or BEDPE,\n                        MACS2 will call its special Paired-end mode to call\n                        peaks by piling up the actual ChIPed fragments defined\n                        by both aligned ends, instead of predicting the\n                        fragment size first and extending reads. Also please\n                        note that the BEDPE only contains three columns, and\n                        is NOT the same BEDPE format used by BEDTOOLS.\n                        DEFAULT: \"AUTO\"\n  -g GSIZE, --gsize GSIZE\n                        Effective genome size. It can be 1.0e+9 or 1000000000,\n                        or shortcuts:'hs' for human (2.7e9), 'mm' for mouse\n                        (1.87e9), 'ce' for C. elegans (9e7) and 'dm' for\n                        fruitfly (1.2e8), Default:hs\n  --keep-dup KEEPDUPLICATES\n                        It controls the MACS behavior towards duplicate tags\n                        at the exact same location -- the same coordination\n                        and the same strand. The 'auto' option makes MACS\n                        calculate the maximum tags at the exact same location\n                        based on binomal distribution using 1e-5 as pvalue\n                        cutoff; and the 'all' option keeps every tags. If an\n                        integer is given, at most this number of tags will be\n                        kept at the same location. Note, if you've used\n                        samtools or picard to flag reads as 'PCR/Optical\n                        duplicate' in bit 1024, MACS2 will still read them\n                        although the reads may be decided by MACS2 as\n                        duplicate later. The default is to keep one tag at the\n                        same location. Default: 1\n  --buffer-size BUFFER_SIZE\n                        Buffer size for incrementally increasing internal\n                        array size to store reads alignment information. In\n                        most cases, you don't have to change this parameter.\n                        However, if there are large number of\n                        chromosomes/contigs/scaffolds in your alignment, it's\n                        recommended to specify a smaller buffer size in order\n                        to decrease memory usage (but it will take longer time\n                        to read alignment files). Minimum memory requested for\n                        reading an alignment file is about # of CHROMOSOME *\n                        BUFFER_SIZE * 2 Bytes. DEFAULT: 100000\n\nOutput arguments:\n  --outdir OUTDIR       If specified all output files will be written to that\n                        directory. Default: the current working directory\n  -n NAME, --name NAME  Experiment name, which will be used to generate output\n                        file names. DEFAULT: \"NA\"\n  -B, --bdg             Whether or not to save extended fragment pileup, and\n                        local lambda tracks (two files) at every bp into a\n                        bedGraph file. DEFAULT: False\n  --verbose VERBOSE     Set verbose level of runtime message. 0: only show\n                        critical message, 1: show additional warning message,\n                        2: show process information, 3: show debug messages.\n                        DEFAULT:2\n  --trackline           Tells MACS to include trackline with bedGraph files.\n                        To include this trackline while displaying bedGraph at\n                        UCSC genome browser, can show name and description of\n                        the file as well. However my suggestion is to convert\n                        bedGraph to bigWig, then show the smaller and faster\n                        binary bigWig file at UCSC genome browser, as well as\n                        downstream analysis. Require -B to be set. Default:\n                        Not include trackline.\n  --SPMR                If True, MACS will save signal per million reads for\n                        fragment pileup profiles. Require -B to be set.\n                        Default: False\n\nShifting model arguments:\n  -s TSIZE, --tsize TSIZE\n                        Tag size. This will override the auto detected tag\n                        size. DEFAULT: Not set\n  --bw BW               Band width for picking regions to compute fragment\n                        size. This value is only used while building the\n                        shifting model. DEFAULT: 300\n  -m MFOLD MFOLD, --mfold MFOLD MFOLD\n                        Select the regions within MFOLD range of high-\n                        confidence enrichment ratio against background to\n                        build model. Fold-enrichment in regions must be lower\n                        than upper limit, and higher than the lower limit. Use\n                        as \"-m 10 30\". DEFAULT:5 50\n  --fix-bimodal         Whether turn on the auto pair model process. If set,\n                        when MACS failed to build paired model, it will use\n                        the nomodel settings, the --exsize parameter to extend\n                        each tags towards 3' direction. Not to use this\n                        automate fixation is a default behavior now. DEFAULT:\n                        False\n  --nomodel             Whether or not to build the shifting model. If True,\n                        MACS will not build model. by default it means\n                        shifting size = 100, try to set extsize to change it.\n                        DEFAULT: False\n  --shift SHIFT         (NOT the legacy --shiftsize option!) The arbitrary\n                        shift in bp. Use discretion while setting it other\n                        than default value. When NOMODEL is set, MACS will use\n                        this value to move cutting ends (5') towards 5'->3'\n                        direction then apply EXTSIZE to extend them to\n                        fragments. When this value is negative, ends will be\n                        moved toward 3'->5' direction. Recommended to keep it\n                        as default 0 for ChIP-Seq datasets, or -1 * half of\n                        EXTSIZE together with EXTSIZE option for detecting\n                        enriched cutting loci such as certain DNAseI-Seq\n                        datasets. Note, you can't set values other than 0 if\n                        format is BAMPE or BEDPE for paired-end data. DEFAULT:\n                        0.\n  --extsize EXTSIZE     The arbitrary extension size in bp. When nomodel is\n                        true, MACS will use this value as fragment size to\n                        extend each read towards 3' end, then pile them up.\n                        It's exactly twice the number of obsolete SHIFTSIZE.\n                        In previous language, each read is moved 5'->3'\n                        direction to middle of fragment by 1/2 d, then\n                        extended to both direction with 1/2 d. This is\n                        equivalent to say each read is extended towards 5'->3'\n                        into a d size fragment. DEFAULT: 200. EXTSIZE and\n                        SHIFT can be combined when necessary. Check SHIFT\n                        option.\n\nPeak calling arguments:\n  -q QVALUE, --qvalue QVALUE\n                        Minimum FDR (q-value) cutoff for peak detection.\n                        DEFAULT: 0.05. -q, and -p are mutually exclusive.\n  -p PVALUE, --pvalue PVALUE\n                        Pvalue cutoff for peak detection. DEFAULT: not set.\n                        -q, and -p are mutually exclusive. If pvalue cutoff is\n                        set, qvalue will not be calculated and reported as -1\n                        in the final .xls file.\n  --to-large            When set, scale the small sample up to the bigger\n                        sample. By default, the bigger dataset will be scaled\n                        down towards the smaller dataset, which will lead to\n                        smaller p/qvalues and more specific results. Keep in\n                        mind that scaling down will bring down background\n                        noise more. DEFAULT: False\n  --ratio RATIO         When set, use a custom scaling ratio of ChIP/control\n                        (e.g. calculated using NCIS) for linear scaling.\n                        DEFAULT: ingore\n  --down-sample         When set, random sampling method will scale down the\n                        bigger sample. By default, MACS uses linear scaling.\n                        Warning: This option will make your result unstable\n                        and irreproducible since each time, random reads would\n                        be selected. Consider to use 'randsample' script\n                        instead. <not implmented>If used together with --SPMR,\n                        1 million unique reads will be randomly picked.</not\n                        implemented> Caution: due to the implementation, the\n                        final number of selected reads may not be as you\n                        expected! DEFAULT: False\n  --seed SEED           Set the random seed while down sampling data. Must be\n                        a non-negative integer in order to be effective.\n                        DEFAULT: not set\n  --tempdir TEMPDIR     Optional directory to store temp files. DEFAULT: /tmp\n  --nolambda            If True, MACS will use fixed background lambda as\n                        local lambda for every peak region. Normally, MACS\n                        calculates a dynamic local lambda to reflect the local\n                        bias due to potential chromatin structure.\n  --slocal SMALLLOCAL   The small nearby region in basepairs to calculate\n                        dynamic lambda. This is used to capture the bias near\n                        the peak summit region. Invalid if there is no control\n                        data. If you set this to 0, MACS will skip slocal\n                        lambda calculation. *Note* that MACS will always\n                        perform a d-size local lambda calculation. The final\n                        local bias should be the maximum of the lambda value\n                        from d, slocal, and llocal size windows. DEFAULT: 1000\n  --llocal LARGELOCAL   The large nearby region in basepairs to calculate\n                        dynamic lambda. This is used to capture the surround\n                        bias. If you set this to 0, MACS will skip llocal\n                        lambda calculation. *Note* that MACS will always\n                        perform a d-size local lambda calculation. The final\n                        local bias should be the maximum of the lambda value\n                        from d, slocal, and llocal size windows. DEFAULT:\n                        10000.\n  --broad               If set, MACS will try to call broad peaks by linking\n                        nearby highly enriched regions. The linking region is\n                        controlled by another cutoff through --linking-cutoff.\n                        The maximum linking region length is 4 times of d from\n                        MACS. DEFAULT: False\n  --broad-cutoff BROADCUTOFF\n                        Cutoff for broad region. This option is not available\n                        unless --broad is set. If -p is set, this is a pvalue\n                        cutoff, otherwise, it's a qvalue cutoff. DEFAULT: 0.1\n  --cutoff-analysis     While set, MACS2 will analyze number or total length\n                        of peaks that can be called by different p-value\n                        cutoff then output a summary table to help user decide\n                        a better cutoff. The table will be saved in\n                        NAME_cutoff_analysis.txt file. Note, minlen and maxgap\n                        may affect the results. WARNING: May take ~30 folds\n                        longer time to finish. DEFAULT: False\n\nPost-processing options:\n  --call-summits        If set, MACS will use a more sophisticated signal\n                        processing approach to find subpeak summits in each\n                        enriched peak region. DEFAULT: False\n  --fe-cutoff FECUTOFF  When set, the value will be used to filter out peaks\n                        with low fold-enrichment. Note, MACS2 use 1.0 as\n                        pseudocount while calculating fold-enrichment.\n                        DEFAULT: 1.0\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_output_filename = function(input_file) { if (inputs.estimates_filename == \"\") { var ext = \"_preseq_estimates.tsv\"; var root = input_file.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.input_file.basename+ext:root+ext; } else { return inputs.estimates_filename; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "stevetsa/preseq:2.0"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 16
                    },
                    "doc": "Coordinate sorted BAM file",
                    "id": "#preseq-lc-extrap.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 8,
                        "prefix": "-bootstraps"
                    },
                    "doc": "Number of bootstraps, default: 100",
                    "id": "#preseq-lc-extrap.cwl/bootstraps"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-cval"
                    },
                    "doc": "Level for confidence intervals, default: 0.95",
                    "id": "#preseq-lc-extrap.cwl/confidence_level"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-defects"
                    },
                    "doc": "Defects mode to extrapolate without testing for defects",
                    "id": "#preseq-lc-extrap.cwl/defects_mode"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "-output",
                        "valueFrom": "$(get_output_filename(inputs.bam_file))"
                    },
                    "default": "",
                    "doc": "Output filename",
                    "id": "#preseq-lc-extrap.cwl/estimates_filename"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "-extrap"
                    },
                    "doc": "Maximum extrapolation, default: 1e+10",
                    "id": "#preseq-lc-extrap.cwl/extrapolation"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "prefix": "-step"
                    },
                    "doc": "Step size in extrapolations, default: 1e+06",
                    "id": "#preseq-lc-extrap.cwl/extrapolations_step"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-seg_len"
                    },
                    "doc": "Maximum segment length when merging paired end bam reads, default: 5000",
                    "id": "#preseq-lc-extrap.cwl/max_fragment_size"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "-pe"
                    },
                    "doc": "Input is paired end read file",
                    "id": "#preseq-lc-extrap.cwl/pe_mode"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-quick"
                    },
                    "doc": "Quick mode, estimate yield without bootstrapping for confidence intervals",
                    "id": "#preseq-lc-extrap.cwl/quick mode"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-terms"
                    },
                    "doc": "Maximum number of terms",
                    "id": "#preseq-lc-extrap.cwl/terms"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "-verbose"
                    },
                    "doc": "Verbose mode",
                    "id": "#preseq-lc-extrap.cwl/verbose_mode"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(get_output_filename(inputs.bam_file))"
                    },
                    "id": "#preseq-lc-extrap.cwl/estimates_file"
                }
            ],
            "baseCommand": [
                "preseq",
                "lc_extrap",
                "-bam"
            ],
            "successCodes": [
                1
            ],
            "doc": "Tool runs preseq lc_extrap. Only BAM input file is supported (-B option is used by default)\nsuccessCodes: [1] - is used to pass this tool as a step in a workflow in case the BAM file was not correct for Preseq\nDiscarded arguments:\n  -V, -vals        input is a text file containing only the observed counts\n  -H, -hist        input is a text file containing the observed histogram\n",
            "id": "#preseq-lc-extrap.cwl",
            "http://schema.org/mainEntity": {
                "$import": "#atdp-metadata.yaml"
            },
            "http://schema.org/name": "preseq-lc-extrap",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/preseq-lc-extrap.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: lc_extrap [OPTIONS] <sorted-bed-file>\n\nOptions:\n  -o, -output      yield output file (default: stdout)\n  -e, -extrap      maximum extrapolation (default: 1e+10)\n  -s, -step        step size in extrapolations (default: 1e+06)\n  -n, -bootstraps  number of bootstraps (default: 100),\n  -c, -cval        level for confidence intervals (default: 0.95)\n  -x, -terms       maximum number of terms\n  -v, -verbose     print more information\n  -B, -bam         input is in BAM format\n  -l, -seg_len     maximum segment length when merging paired end bam reads\n                   (default: 5000)\n  -P, -pe          input is paired end read file\n  -V, -vals        input is a text file containing only the observed counts\n  -H, -hist        input is a text file containing the observed histogram\n  -Q, -quick       quick mode, estimate yield without bootstrapping for\n                   confidence intervals\n  -D, -defects     defects mode to extrapolate without testing for defects\n\nHelp options:\n  -?, -help        print this help message\n      -about       print about message\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_target_name = function() { return inputs.target_filename.split('/').slice(-1)[0]; }"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/scidap:v0.0.3"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\ncp $0 $1\nif [ -f $0.bai ]; then\n  cp $0.bai $1.bai\nfi\n",
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#rename.cwl/script"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5
                    },
                    "id": "#rename.cwl/source_file"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 6,
                        "valueFrom": "$(get_target_name())"
                    },
                    "id": "#rename.cwl/target_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(get_target_name())"
                    },
                    "secondaryFiles": "${\n    if (inputs.source_file.secondaryFiles && inputs.source_file.secondaryFiles.length > 0){\n      return inputs.target_filename+\".bai\";\n    } else {\n      return \"null\";\n    }\n  }\n",
                    "id": "#rename.cwl/target_file"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "Tool renames `source_file` to `target_filename`.\nInput `target_filename` should be set as string. If it's a full path, only basename will be used.\nIf BAI file is present, it will be renamed too\n",
            "id": "#rename.cwl",
            "http://schema.org/name": "rename",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/rename.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "cp source target"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/samtools:v1.4"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 6
                    },
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "doc": "Indexed BAM+BAI files",
                    "id": "#samtools-filter.cwl/bam_bai_pair"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 7
                    },
                    "doc": "Space separated list of the chromosemes to exclude",
                    "id": "#samtools-filter.cwl/exclude_chromosome"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "valueFrom": "${\n  return (self == \"\")?inputs.bam_bai_pair.basename:self;\n}\n"
                    },
                    "default": "",
                    "doc": "Output filename for the filtered BAM file",
                    "id": "#samtools-filter.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 8
                    },
                    "default": 0,
                    "doc": "Skip alignments with MAPQ smaller than INT. Default 0",
                    "id": "#samtools-filter.cwl/quality"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\necho \"Filtering BAM file\"\necho \"samtools idxstats $0 | cut -f 1 | grep -v -E \\\"`echo $1 | sed -e 's/ /$|/g'`$|\\*\\\" | xargs samtools view -q $2 -o $3 $0\"\nsamtools idxstats $0 | cut -f 1 | grep -v -E \"`echo $1 | sed -e 's/ /$|/g'`$|\\*\" | xargs samtools view -q $2 -o $3 $0\necho \"Sorting BAM file\"\necho \"samtools sort $3 -o $3\"\nsamtools sort $3 -o $3\necho \"Indexing BAM file\"\necho \"samtools index $3\"\nsamtools index $3\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Script to exclude chromosomes from the BAM file and filter reads by quality",
                    "id": "#samtools-filter.cwl/script"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "*.bam"
                    },
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "doc": "Filtered BAM+BAI files",
                    "id": "#samtools-filter.cwl/filtered_bam_bai_pair"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "Excludes chromosomes from the input BAM file. Filters reads by quality.\nIf there is only one chromosome present, you cannot exclude it\n",
            "id": "#samtools-filter.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#samtools-metadata.yaml",
                "http://schema.org/name": "samtools",
                "http://schema.org/about": "A suite of programs for interacting with high-throughput sequencing data. It consists of three separate repositories: Samtools (Reading/writing/editing/indexing/viewing SAM/BAM/CRAM format), BCFtools (Reading/writing BCF2/VCF/gVCF files and calling/filtering/summarising SNP and short indel sequence variants) and HTSlib (A C library for reading/writing high-throughput sequencing data).\n",
                "http://schema.org/url": "http://www.htslib.org/",
                "http://schema.org/codeRepository": "https://github.com/samtools/samtools.git",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/MIT",
                    "https://opensource.org/licenses/BSD-3-Clause"
                ],
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "1.4",
                    "http://schema.org/applicationCategory": "commandline tool"
                },
                "http://schema.org/programmingLanguage": "C, Perl",
                "http://schema.org/publication": [
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btr509",
                        "http://schema.org/name": "(Li, 2011) A statistical framework for SNP calling, mutation discovery, association mapping and population genetical parameter estimation from sequencing data. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/21903627"
                    },
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btr076",
                        "http://schema.org/name": "(Li, 2011) Improving SNP discovery by base alignment quality. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/21320865"
                    },
                    {
                        "class": "http://schema.org/ScholarlyArticle",
                        "id": "#btp352",
                        "http://schema.org/name": "(Li et al., 2009) The Sequence Alignment/Map format and SAMtools. Bioinformatics.",
                        "http://schema.org/url": "http://www.ncbi.nlm.nih.gov/pubmed/19505943"
                    }
                ],
                "http://schema.org/discussionUrl": [
                    "https://lists.sourceforge.net/lists/listinfo/samtools-help",
                    "https://lists.sourceforge.net/lists/listinfo/samtools-devel"
                ],
                "http://schema.org/creator": [
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Sanger Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Heng Li",
                                "http://schema.org/description": "wrote most of the initial source codes of SAMtools and various converters."
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Broad Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Bob Handsaker",
                                "http://schema.org/description": "A major contributor to the\nSAM/BAM specification. He designed and implemented the BGZF format, the\nunderlying indexable compression format for the BAM format. BGZF does\nnot support arithmetic between file offsets.\n"
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Organization",
                        "http://schema.org/name": "Beijing Genome Institute",
                        "http://schema.org/member": [
                            {
                                "class": "http://schema.org/Person",
                                "http://schema.org/name": "Jue Ruan",
                                "http://schema.org/description": "Designed and implemented the\nRAZF format, an alternative indexable compression format. RAZF is no longer\nused by or provided with SAMtools. Source code remains available in older\nSAMtools 0.1.x releases and from the standalone branch in the repository.\n"
                            }
                        ]
                    },
                    {
                        "class": "http://schema.org/Person",
                        "http://schema.org/name": "Colin Hercus",
                        "http://schema.org/description": "updated novo2sam.pl to support gapped alignment by novoalign."
                    },
                    {
                        "class": "http://schema.org/Person",
                        "http://schema.org/name": "Petr Danecek",
                        "http://schema.org/description": "contributed the header parsing library sam_header.c and sam2vcf.pl script."
                    }
                ]
            },
            "http://schema.org/name": "samtools-filter",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/samtools-filter.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Excludes chromosomes from the input BAM file"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { return inputs.bam_file.location.split('/').slice(-1)[0]; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/samtools:v1.4"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10
                    },
                    "doc": "Input sorted bam file (index file is optional)\n",
                    "id": "#samtools-rmdup.cwl/bam_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  samtools rmdup \"${@:1}\"\nelse\n  echo \"Skip samtools rmdup \" ${@:1}\n  SOURCE=${@:(-2):1}\n  TARGET=${@:(-1):1}\n  cp $SOURCE $TARGET\n  if [ -f $SOURCE.bai ]; then\n    cp $SOURCE.bai $TARGET.bai\n  fi\nfi\n",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Bash function to run samtools rmdup with all input parameters or skip it if trigger is false\n",
                    "id": "#samtools-rmdup.cwl/bash_script"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 4,
                        "prefix": "-S"
                    },
                    "doc": "treat PE reads as SE in rmdup (force -s)\n",
                    "id": "#samtools-rmdup.cwl/force_single_end"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "valueFrom": "${\n    if (self == \"\" || inputs.trigger == false){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Writes the output bam file to output_filename if set,\notherwise generates output_filename on the base of bam_file\n",
                    "id": "#samtools-rmdup.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-s"
                    },
                    "doc": "rmdup for SE reads\n",
                    "id": "#samtools-rmdup.cwl/single_end"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": true,
                    "inputBinding": {
                        "position": 2,
                        "valueFrom": "${ return self ? \"true\" : \"false\" }\n"
                    },
                    "doc": "If true - run samtools rmdup, if false - return input bam_file and optional index file\nUse valueFrom to return string instead of boolean, because if return boolean False, argument is not printed\n",
                    "id": "#samtools-rmdup.cwl/trigger"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n  if (inputs.output_filename == \"\" || inputs.trigger == false){\n    return default_output_filename().split('.').slice(0,-1).join('.') + '.rmdup';\n  } else {\n    return inputs.output_filename.split('.').slice(0,-1).join('.') + '.rmdup';\n  }\n}\n"
                    },
                    "id": "#samtools-rmdup.cwl/rmdup_log"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_filename == \"\" || inputs.trigger == false){\n      return default_output_filename();\n    } else {\n      return inputs.output_filename;\n    }\n}\n"
                    },
                    "secondaryFiles": "${\n    if (inputs.bam_file.secondaryFiles && inputs.trigger == false){\n      return inputs.bam_file.secondaryFiles;\n    } else {\n      return \"null\";\n    }\n  }\n",
                    "doc": "File with removed duplicates or bam_file with optional secondaryFiles",
                    "id": "#samtools-rmdup.cwl/rmdup_output"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "arguments": [
                {
                    "valueFrom": "${\n  if (inputs.output_filename == \"\" || inputs.trigger == false){\n    return \" > \" + default_output_filename().split('.').slice(0,-1).join('.') + \".rmdup 2>&1\";\n  } else {\n    return \" > \" + inputs.output_filename.split('.').slice(0,-1).join('.') + \".rmdup 2>&1\";\n  }\n}\n",
                    "position": 100000,
                    "shellQuote": false
                }
            ],
            "doc": "Tool to remove duplicates from coordinate sorted BAM file set as input `bam_file`.\nIf input `trigger` is set to `true` or isn't set at all (`true` is used by default), run `samtools rmdup`, return\nnewly generated BAM file without duplicates and log as outputs `rmdup_output` and `rmdup_log`.\nIf input `trigger` is set to `false`, return unchanged BAM and index files (if provided in secondaryFiles).\n\nTrigger logic is implemented in bash script set by default in input `bash_script`. If first argment $0 (which is `trigger` input)\nis true, run `samtools rmdup` with the rest of the arguments. If $0 is not true, skip samtools running and copy\ninto output directory BAM file. Input BAM file and the filename which we are waiting to be the\noutput of tool are always set as two last arguments of the script.\n\nInput `trigger` is Boolean, but returns String, because of `valueFrom` field. The `valueFrom` is used, because if `trigger`\nis false, cwl-runner doesn't append this argument at all to the the `baseCommand` - new feature of CWL v1.0.2. Alternatively,\n`prefix` field could be used, but it causes changing logic in bash script saved in `bash_script` input.\n\n`default_output_filename` function is used for generating output filename if input `output_filename` is not set or in\ncase when `trigger` is false and we need to return original BAM and index files.\n\nOutput `rmdup_output` returns `secondaryFiles` only in case when `trigger` was set to false (we need to rerun index).\n  \n",
            "id": "#samtools-rmdup.cwl",
            "http://schema.org/mainEntity": {
                "$import": "#samtools-metadata.yaml"
            },
            "http://schema.org/name": "samtools-rmdup",
            "http://schema.org/downloadUrl": "https://github.com/Barski-lab/workflows/blob/master/tools/samtools-rmdup.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:michael.kotliar@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage:  samtools rmdup [-sS] <input.srt.bam> <output.bam>\n\nOption: -s    rmdup for SE reads\n        -S    treat PE reads as SE in rmdup (force -s)\n      --input-fmt-option OPT[=VAL]\n               Specify a single input file format option in the form\n               of OPTION or OPTION=VALUE\n      --output-fmt FORMAT[,OPT[=VAL]]...\n               Specify output format (SAM, BAM, CRAM)\n      --output-fmt-option OPT[=VAL]\n               Specify a single output file format option in the form\n               of OPTION or OPTION=VALUE\n      --reference FILE\n               Reference sequence FASTA FILE [null]\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "ShellCommandRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var ext = function() { if (inputs.csi && !inputs.bai){ return '.csi'; } else { return '.bai'; } };",
                        "var default_bam = function() { if (inputs.trigger == true){ return inputs.sort_input.location.split('/').slice(-1)[0].split('.').slice(0,-1).join('.')+\".bam\"; } else { return inputs.sort_input.location.split('/').slice(-1)[0]; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/samtools:v1.4"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Generate BAI-format index for BAM files [default]. If input isn't cram.\n",
                    "id": "#samtools-sort-index.cwl/bai"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: samtools index \" ${@:1}\n  samtools index \"${@:1}\"\nelse\n  echo \"Skip samtools index \" ${@:1}\nfi\n",
                    "inputBinding": {
                        "position": 20
                    },
                    "doc": "Bash function to run samtools index with all input parameters or skip it if trigger is false\n",
                    "id": "#samtools-sort-index.cwl/bash_script_index"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: samtools sort \" ${@:1}\n  samtools sort \"${@:1}\"\nelse\n  echo \"Skip samtools sort \" ${@:1}\n  SOURCE=${@:(-1):1}\n  TARGET=$(basename \"$SOURCE\")\n  cp $SOURCE $TARGET\nfi\n",
                    "inputBinding": {
                        "position": 5
                    },
                    "doc": "Bash function to run samtools sort with all input parameters or skip it if trigger is false\n",
                    "id": "#samtools-sort-index.cwl/bash_script_sort"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "doc": "Generate CSI-format index for BAM files. If input isn't cram.\n",
                    "id": "#samtools-sort-index.cwl/csi"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 24,
                        "prefix": "-m"
                    },
                    "doc": "Set minimum interval size for CSI indices to 2^INT [14]\n",
                    "id": "#samtools-sort-index.cwl/csi_interval"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-l"
                    },
                    "doc": "SORT: desired compression level for the final output file, ranging from 0 (uncompressed)\nor 1 (fastest but minimal compression) to 9 (best compression but slowest to write),\nsimilarly to gzip(1)'s compression level setting.\nIf -l is not used, the default compression level will apply.\n",
                    "id": "#samtools-sort-index.cwl/sort_compression_level"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 16
                    },
                    "doc": "Input only in.sam|in.bam|in.cram. Optionally could be supplemented with index file in secondaryFiles\n",
                    "id": "#samtools-sort-index.cwl/sort_input"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "-o",
                        "valueFrom": "${\n    if (self == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "Write the final sorted output to FILE. Only out.bam|out.cram.\nIf output file extension is set to SAM, tool will fail on the index step\n",
                    "id": "#samtools-sort-index.cwl/sort_output_filename"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "doc": "Set number of sorting and compression threads [1] (Only for sorting)\n",
                    "id": "#samtools-sort-index.cwl/threads"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": true,
                    "doc": "If true - run samtools, if false - return sort_input and optional index file in secondaryFiles.\n",
                    "id": "#samtools-sort-index.cwl/trigger"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return inputs.sort_output_filename;\n    }\n}\n"
                    },
                    "secondaryFiles": "${ if (inputs.trigger == true){ return self.basename + ext(); } else { return inputs.sort_input.secondaryFiles?inputs.sort_input.secondaryFiles:\"null\"; } }",
                    "id": "#samtools-sort-index.cwl/bam_bai_pair"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "arguments": [
                {
                    "valueFrom": "${ return inputs.trigger ? \"true\" : \"false\" }\n",
                    "position": 6
                },
                {
                    "valueFrom": "bam",
                    "position": 13,
                    "prefix": "-O"
                },
                {
                    "valueFrom": "$(inputs.threads?inputs.threads:1)",
                    "position": 15,
                    "prefix": "-@"
                },
                {
                    "valueFrom": ";",
                    "position": 17,
                    "shellQuote": false
                },
                {
                    "valueFrom": "bash",
                    "position": 18
                },
                {
                    "valueFrom": "-c",
                    "position": 19
                },
                {
                    "valueFrom": "${ return inputs.trigger ? \"true\" : \"false\" }\n",
                    "position": 21
                },
                {
                    "valueFrom": "$(inputs.bai?'-b':inputs.csi?'-c':[])",
                    "position": 23
                },
                {
                    "valueFrom": "$(inputs.threads?inputs.threads:1)",
                    "position": 25,
                    "prefix": "-@"
                },
                {
                    "valueFrom": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam();\n    } else {\n      return inputs.sort_output_filename;\n    }\n}\n",
                    "position": 26
                },
                {
                    "valueFrom": "${\n    if (inputs.sort_output_filename == \"\" || inputs.trigger == false){\n      return default_bam() + ext();\n    } else {\n      return inputs.sort_output_filename + ext();\n    }\n}\n",
                    "position": 27
                }
            ],
            "doc": "Tool to sort and index input BAM/SAM/CRAM.\nIf input `trigger` is set to `true` or isn't set at all (`true` is used by default), run `samtools sort` and\n`samtools index`, return sorted BAM and BAI/CSI index file.\nIf input `trigger` is set to `false`, return unchanged `sort_input` (BAM/SAM/CRAM) and index (BAI/CSI, if provided in\n`secondaryFiles`) files.\n\nTrigger logic is implemented in two bash scripts set by default as `bash_script_sort` and `bash_script_index` inputs.\nFor both of then, if the first argument $0 (which is `trigger` input) is true, run `samtools sort/index` with the rest\nof the arguments. If $0 is not true, skip `samtools sort/index` and return `sort_input` and `secondaryFiles`\n(if provided).\n\nInput `trigger` is Boolean, but returns String, because of `valueFrom` field. The `valueFrom` is used, because if `trigger`\nis false, cwl-runner doesn't append this argument at all to the the `baseCommand` - new feature of CWL v1.0.2. Alternatively,\n`prefix` field could be used, but it causes changing in script logic.\n\nIf using `sort_output_filename`, the output file extension should be `*.bam`, because `samtools sort` defines the output\nfile format on the base of the file extension. If `*.sam` is sed as output filename, it cannot be usefully indexed\nby `samtools index`.\n\n`default_bam` function is used to generate output filename for `samtools sort` if input `sort_output_filename` is not\nset or when `trigger` is false and we need to return `sort_input` and `secondaryFiles` (if provided) files. Output\nfilename is generated on the base of `sort_input` basename with `.bam` extension by default.\n\n`ext` function is used to return the index file extension (BAI/CSI) based on `csi` and `bai` inputs according to the\nfollowing logic\n  `csi` &&  `bai`  => BAI\n !`csi` && !`bai ` => BAI\n  `csi` && !`bai ` => CSI\n",
            "id": "#samtools-sort-index.cwl",
            "http://schema.org/mainEntity": {
                "$import": "#samtools-metadata.yaml"
            },
            "http://schema.org/name": "samtools-sort-index",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/samtools-sort-index.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Usage: samtools sort [options...] [in.bam]\nOptions:\n  -l INT     Set compression level, from 0 (uncompressed) to 9 (best)\n  -m INT     Set maximum memory per thread; suffix K/M/G recognized [768M]\n  -n         Sort by read name\n  -o FILE    Write final output to FILE rather than standard output\n  -T PREFIX  Write temporary files to PREFIX.nnnn.bam\n      --input-fmt-option OPT[=VAL]\n               Specify a single input file format option in the form\n               of OPTION or OPTION=VALUE\n  -O, --output-fmt FORMAT[,OPT[=VAL]]...\n               Specify output format (SAM, BAM, CRAM)\n      --output-fmt-option OPT[=VAL]\n               Specify a single output file format option in the form\n               of OPTION or OPTION=VALUE\n      --reference FILE\n               Reference sequence FASTA FILE [null]\n  -@, --threads INT\n               Number of additional threads to use [0]\n\nUsage: samtools index [-bc] [-m INT] <in.bam> [out.index]\nOptions:\n  -b       Generate BAI-format index for BAM files [default]\n  -c       Generate CSI-format index for BAM files\n  -m INT   Set minimum interval size for CSI indices to 2^INT [14]\n  -@ INT   Sets the number of threads [none]\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { if (inputs.output_filename == \"\"){ var root = inputs.bambai_pair.basename.split('.').slice(0,-1).join('.'); return (root == \"\")?inputs.bambai_pair.basename+\".log\":root+\".log\"; } else { return inputs.output_filename; } };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/samtools:v1.4"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 5
                    },
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "doc": "Coordinate sorted BAM alignment and index BAI files",
                    "id": "#samtools-stats.cwl/bambai_pair"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "",
                    "doc": "Output file name",
                    "id": "#samtools-stats.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nsamtools stats $0 > all_statistics.log\ncat all_statistics.log | grep SN | cut -f 2- > sn_section.log\necho -e \"insert size\\tpairs total\\tinward oriented pairs\\toutward oriented pairs\\tother pairs\" > is_section.tsv\ncat all_statistics.log | grep ^IS | cut -f 2- | head -n 1501 >> is_section.tsv\ncat all_statistics.log\n",
                    "inputBinding": {
                        "position": 4
                    },
                    "doc": "samtools stats with optional filtering criteria passes as $1",
                    "id": "#samtools-stats.cwl/script"
                }
            ],
            "outputs": [
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "sn_section.log",
                        "outputEval": "${\n  var s = self[0].contents.substring(self[0].contents.indexOf(\"average length\"));\n  return parseInt(s.substring(s.indexOf(\"average length\")+16, s.indexOf(\"\\n\")));\n}\n"
                    },
                    "doc": "Reads Average Length",
                    "id": "#samtools-stats.cwl/average_length"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "is_section.tsv"
                    },
                    "doc": "BAM file statistics (IS section)",
                    "id": "#samtools-stats.cwl/ext_is_section"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(default_output_filename())"
                    },
                    "doc": "BAM file statistics",
                    "id": "#samtools-stats.cwl/log_file"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "sn_section.log",
                        "outputEval": "${\n  var s = self[0].contents.substring(self[0].contents.indexOf(\"raw total sequences\"));\n  return parseInt(s.substring(s.indexOf(\"raw total sequences\")+21, s.indexOf(\"\\n\")));\n}\n"
                    },
                    "doc": "Raw Total Sequences",
                    "id": "#samtools-stats.cwl/raw_total_sequences"
                },
                {
                    "type": "int",
                    "outputBinding": {
                        "loadContents": true,
                        "glob": "sn_section.log",
                        "outputEval": "${\n  var s = self[0].contents.substring(self[0].contents.indexOf(\"reads mapped\"));\n  return parseInt(s.substring(s.indexOf(\"reads mapped\")+14, s.indexOf(\"\\n\")));\n}\n"
                    },
                    "doc": "Reads Mapped",
                    "id": "#samtools-stats.cwl/reads_mapped"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "stdout": "$(default_output_filename())",
            "doc": "Generates statistics for the input BAM file.\n",
            "id": "#samtools-stats.cwl",
            "http://schema.org/mainEntity": {
                "$import": "#samtools-metadata.yaml"
            },
            "http://schema.org/name": "samtools-stats",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/samtools-stats.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "Generates statistics for the input BAM file.\n"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_log_name = function() { var lognames = {}; lognames[\"pair\"] = (inputs.paired && inputs.input_file_pair) ? inputs.input_file_pair.basename+'_trimming_report.txt':null; lognames[\"single\"] = inputs.input_file.basename+'_trimming_report.txt'; return lognames; }"
                    ]
                },
                {
                    "class": "InitialWorkDirRequirement",
                    "listing": "${\n  var listing = [inputs.input_file]\n  if (inputs.input_file_pair){\n    listing.push(inputs.input_file_pair);\n  }\n  return listing;\n}\n"
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "scidap/trimgalore:v0.6.6"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-a"
                    },
                    "doc": "Adapter sequence to be trimmed. If not specified explicitly, Trim Galore will\ntry to auto-detect whether the Illumina universal, Nextera transposase or Illumina\nsmall RNA adapter sequence was used. Also see '--illumina', '--nextera' and\n'--small_rna'. If no adapter can be detected within the first 1 million sequences\nof the first file specified Trim Galore defaults to '--illumina'.\n",
                    "id": "#trimgalore.cwl/adapter"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-a2"
                    },
                    "doc": "Optional adapter sequence to be trimmed off read 2 of paired-end files. This\noption requires '--paired' to be specified as well. If the libraries to be trimmed\nare smallRNA then a2 will be set to the Illumina small RNA 5' adapter automatically\n(GATCGTCGGACT).\n",
                    "id": "#trimgalore.cwl/adapter_pair"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "#!/bin/bash\nif [ \"$0\" = \"true\" ]\nthen\n  echo \"Run: trimgalore \" ${@:1}\n  trim_galore \"${@:1}\"\nelse\n  echo \"Skip run trimgalore\"\nfi\n",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Bash function to run trimgalore with all input parameters or skip it if trigger is false\n",
                    "id": "#trimgalore.cwl/bash_script"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 25,
                        "prefix": "--clip_R1"
                    },
                    "doc": "Instructs Trim Galore to remove <int> bp from the 5' end of read 1 (or single-end\nreads). This may be useful if the qualities were very poor, or if there is some\nsort of unwanted bias at the 5' end.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/clip_R1"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 26,
                        "prefix": "--clip_R2"
                    },
                    "doc": "Instructs Trim Galore to remove <int> bp from the 5' end of read 2 (paired-end reads\nonly). This may be useful if the qualities were very poor, or if there is some sort\nof unwanted bias at the 5' end. For paired-end BS-Seq, it is recommended to remove\nthe first few bp because the end-repair reaction may introduce a bias towards low\nmethylation. Please refer to the M-bias plot section in the Bismark User Guide for\nsome examples.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/clip_R2"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 19,
                        "prefix": "--dont_gzip"
                    },
                    "doc": "Output files won't be compressed with GZIP. This option overrides --gzip.\n",
                    "id": "#trimgalore.cwl/dont_gzip"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 17,
                        "prefix": "-e"
                    },
                    "doc": "Maximum allowed error rate (no. of errors divided by the length of the matching region)\nDefault: 0.1\n",
                    "id": "#trimgalore.cwl/error_rate"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 18,
                        "prefix": "--gzip"
                    },
                    "doc": "Compress the output file with GZIP. If the input files are GZIP-compressed\nthe output files will automatically be GZIP compressed as well. As of v0.2.8 the\ncompression will take place on the fly.\n",
                    "id": "#trimgalore.cwl/gzip"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "prefix": "--illumina"
                    },
                    "doc": "Adapter sequence to be trimmed is the first 13bp of the Illumina universal adapter\n'AGATCGGAAGAGC' instead of the default auto-detection of adapter sequence.\n",
                    "id": "#trimgalore.cwl/illumina"
                },
                {
                    "type": [
                        "File"
                    ],
                    "inputBinding": {
                        "position": 100
                    },
                    "doc": "Input FASTQ file\n",
                    "id": "#trimgalore.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 101,
                        "valueFrom": "${\n    if (!inputs.paired){\n      return null;\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "doc": "Input FASTQ file, if paired-end data is analysed\n",
                    "id": "#trimgalore.cwl/input_file_pair"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 20,
                        "prefix": "--length"
                    },
                    "doc": "Discard reads that became shorter than length INT because of either\nquality or adapter trimming. A value of '0' effectively disables\nthis behaviour.\nDefault: 20 bp.\n",
                    "id": "#trimgalore.cwl/length"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 35,
                        "prefix": "-r1"
                    },
                    "doc": "Unpaired single-end read length cutoff needed for read 1 to be written to\n'.unpaired_1.fq' output file. These reads may be mapped in single-end mode.\nDefault: 35 bp.\n",
                    "id": "#trimgalore.cwl/length_1"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 36,
                        "prefix": "-r2"
                    },
                    "doc": "Unpaired single-end read length cutoff needed for read 2 to be written to\n'.unpaired_2.fq' output file. These reads may be mapped in single-end mode.\nDefault: 35 bp.\n",
                    "id": "#trimgalore.cwl/length_2"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 15,
                        "prefix": "--max_length"
                    },
                    "doc": "Discard reads that are longer than <INT> bp after trimming. This is only advised for\nsmallRNA sequencing to remove non-small RNA sequences.\n",
                    "id": "#trimgalore.cwl/max_length"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 21,
                        "prefix": "--max_n"
                    },
                    "doc": "The total number of Ns (as integer) a read may contain before it will be removed altogether.\nIn a paired-end setting, either read exceeding this limit will result in the entire\npair being removed from the trimmed output files.\n",
                    "id": "#trimgalore.cwl/max_n"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 13,
                        "prefix": "--nextera"
                    },
                    "doc": "Adapter sequence to be trimmed is the first 12bp of the Nextera adapter\n'CTGTCTCTTATA' instead of the default auto-detection of adapter sequence.\n",
                    "id": "#trimgalore.cwl/nextera"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 23,
                        "prefix": "--no_report_file"
                    },
                    "doc": "If specified no report file will be generated.\n",
                    "id": "#trimgalore.cwl/no_report_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 30,
                        "prefix": "--non_directional"
                    },
                    "doc": "Selecting this option for non-directional RRBS libraries will screen\nquality-trimmed sequences for 'CAA' or 'CGA' at the start of the read\nand, if found, removes the first two basepairs. Like with the option\n'--rrbs' this avoids using cytosine positions that were filled-in\nduring the end-repair step. '--non_directional' requires '--rrbs' to\nbe specified as well. Note that this option does not set '--clip_r2 2' in\npaired-end mode.\n",
                    "id": "#trimgalore.cwl/non_directional"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 32,
                        "prefix": "--paired",
                        "valueFrom": "${\n    if (!inputs.input_file_pair){\n      return null;\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "doc": "This option performs length trimming of quality/adapter/RRBS trimmed reads for\npaired-end files. To pass the validation test, both sequences of a sequence pair\nare required to have a certain minimum length which is governed by the option\n--length (see above). If only one read passes this length threshold the\nother read can be rescued (see option --retain_unpaired). Using this option lets\nyou discard too short read pairs without disturbing the sequence-by-sequence order\nof FastQ files which is required by many aligners.\n",
                    "id": "#trimgalore.cwl/paired"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--phred33"
                    },
                    "doc": "Instructs Cutadapt to use ASCII+33 quality scores as Phred scores\n(Sanger/Illumina 1.9+ encoding) for quality trimming.\nDefault: ON.\n",
                    "id": "#trimgalore.cwl/phred33"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--phred64"
                    },
                    "doc": "Instructs Cutadapt to use ASCII+64 quality scores as Phred scores\n(Illumina 1.5 encoding) for quality trimming.\n",
                    "id": "#trimgalore.cwl/phred64"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-q"
                    },
                    "doc": "Trim low-quality ends from reads in addition to adapter removal.\nDefault Phred score: 20.\n",
                    "id": "#trimgalore.cwl/quality"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 34,
                        "prefix": "--retain_unpaired"
                    },
                    "doc": "If only one of the two paired-end reads became too short, the longer\nread will be written to either '.unpaired_1.fq' or '.unpaired_2.fq'\noutput files. The length cutoff for unpaired single-end reads is\ngoverned by the parameters -r1/--length_1 and -r2/--length_2.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/retain_unpaired"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 29,
                        "prefix": "--rrbs"
                    },
                    "doc": "Specifies that the input file was an MspI digested RRBS sample (recognition\nsite: CCGG). Single-end or Read 1 sequences (paired-end) which were adapter-trimmed\nwill have a further 2 bp removed from their 3' end. Sequences which were merely\ntrimmed because of poor quality will not be shortened further. Read 2 of paired-end\nlibraries will in addition have the first 2 bp removed from the 5' end (by setting\n'--clip_r2 2'). This is to avoid using artificial methylation calls from the filled-in\ncytosine positions close to the 3' MspI site in sequenced fragments.\nThis option is not recommended for users of the NuGEN ovation RRBS System 1-16\nkit (see below).\n",
                    "id": "#trimgalore.cwl/rrbs"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 14,
                        "prefix": "--small_rna"
                    },
                    "doc": "Adapter sequence to be trimmed is the first 12bp of the Illumina Small RNA 3' Adapter\n'TGGAATTCTCGG' instead of the default auto-detection of adapter sequence. Selecting\nto trim smallRNA adapters will also lower the --length value to 18bp. If the smallRNA\nlibraries are paired-end then a2 will be set to the Illumina small RNA 5' adapter\nautomatically (GATCGTCGGACT) unless -a 2 had been defined explicitly.\n",
                    "id": "#trimgalore.cwl/small_rna"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 16,
                        "prefix": "--stringency"
                    },
                    "doc": "Overlap with adapter sequence required to trim a sequence. Defaults to a\nvery stringent setting of 1, i.e. even a single bp of overlapping sequence\nwill be trimmed off from the 3' end of any read.\n",
                    "id": "#trimgalore.cwl/stringency"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 24,
                        "prefix": "--suppress_warn"
                    },
                    "doc": "If specified any output to STDOUT or STDERR will be suppressed.\n",
                    "id": "#trimgalore.cwl/suppress_warn"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 27,
                        "prefix": "--three_prime_clip_R1"
                    },
                    "doc": "Instructs Trim Galore to remove <int> bp from the 3' end of read 1 (or single-end\nreads) AFTER adapter/quality trimming has been performed. This may remove some unwanted\nbias from the 3' end that is not directly related to adapter sequence or basecall quality.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/three_prime_clip_R1"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 28,
                        "prefix": "--three_prime_clip_R2"
                    },
                    "doc": "Instructs Trim Galore to remove <int> bp from the 3' end of read 2 AFTER\nadapter/quality trimming has been performed. This may remove some unwanted bias from\nthe 3' end that is not directly related to adapter sequence or basecall quality.\nDefault: OFF.\n",
                    "id": "#trimgalore.cwl/three_prime_clip_R2"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": true,
                    "inputBinding": {
                        "position": 2,
                        "valueFrom": "$(self?\"true\":\"false\")"
                    },
                    "doc": "If true - run trimgalore, if false - return input fastq files, previously staged into output directory.\nUse valueFrom to return string instead of boolean to make sure that value is printed to command line in both cases\n",
                    "id": "#trimgalore.cwl/trigger"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 33,
                        "prefix": "--trim1"
                    },
                    "doc": "Trims 1 bp off every read from its 3' end. This may be needed for FastQ files that\nare to be aligned as paired-end data with Bowtie.\n",
                    "id": "#trimgalore.cwl/trim1"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 22,
                        "prefix": "--trim-n"
                    },
                    "doc": "Removes Ns from either side of the read. This option does currently not work in RRBS mode.\n",
                    "id": "#trimgalore.cwl/trim_n"
                }
            ],
            "outputs": [
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(default_log_name()['single'])"
                    },
                    "id": "#trimgalore.cwl/report_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "$(default_log_name()['pair'])"
                    },
                    "id": "#trimgalore.cwl/report_file_pair"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.trigger == false){\n      return inputs.input_file.basename;\n    } else if (inputs.paired && inputs.input_file_pair){\n      return \"*_val_1.fq*\";\n    } else {\n      return \"*_trimmed.fq*\";\n    }\n}\n"
                    },
                    "id": "#trimgalore.cwl/trimmed_file"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "${\n    if (inputs.paired && inputs.input_file_pair && inputs.trigger == false){\n      return inputs.input_file_pair.basename;\n    } else {\n      return \"*_val_2.fq*\";\n    }\n}\n"
                    },
                    "id": "#trimgalore.cwl/trimmed_file_pair"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*_unpaired_1.fq*"
                    },
                    "id": "#trimgalore.cwl/unpaired_file_1"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "outputBinding": {
                        "glob": "*_unpaired_2.fq*"
                    },
                    "id": "#trimgalore.cwl/unpaired_file_2"
                }
            ],
            "baseCommand": [
                "bash",
                "-c"
            ],
            "doc": "Tool runs Trimgalore - the wrapper around Cutadapt and FastQC to consistently apply adapter and quality trimming\nto FastQ files.\n\n`default_log_name` function returns names for generated log files (for both paired-end and single-end cases).\n`trim_galore` itself doesn't support setting custom names for output files.\n\nFor paired-end data processing both `input_file_pair` and `paired` should be set. If either of them is not set,\nthe other one becomes unset automatically.\n\nIf input trigger was set to false, skip running trimaglore and return unchanged input files\n",
            "id": "#trimgalore.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#trimgalore-metadata.yaml",
                "http://schema.org/name": "TrimGalore",
                "http://schema.org/about": "A wrapper around Cutadapt and FastQC to consistently apply adapter and quality trimming to FastQ files,\nwith extra functionality for RRBS data\n",
                "http://schema.org/url": "https://www.bioinformatics.babraham.ac.uk/projects/trim_galore/",
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "v0.4.4",
                    "http://schema.org/applicationCategory": "commandline tool",
                    "http://schema.org/downloadURL": "https://github.com/FelixKrueger/TrimGalore"
                },
                "http://schema.org/programmingLanguage": "Perl",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ]
            },
            "http://schema.org/name": "trimgalore",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/trimgalore.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "trim_galore [options] <filename(s)>\n\nCurrently NOT supported as cwl tool options:\n\n-h/--help               Print this help message and exits.\n\n-v/--version            Print the version information and exits.\n\n--fastqc                Run FastQC in the default mode on the FastQ file once trimming is complete.\n\n--fastqc_args \"<ARGS>\"  Passes extra arguments to FastQC. If more than one argument is to be passed\n                        to FastQC they must be in the form \"arg1 arg2 etc.\". An example would be:\n                        --fastqc_args \"--nogroup --outdir /home/\". Passing extra arguments will\n                        automatically invoke FastQC, so --fastqc does not have to be specified\n                        separately.\n\n--path_to_cutadapt </path/to/cutadapt>     You may use this option to specify a path to the Cutadapt executable,\n                        e.g. /my/home/cutadapt-1.7.1/bin/cutadapt. Else it is assumed that Cutadapt is in\n                        the PATH.\n\n-o/--output_dir <DIR>   If specified all output will be written to this directory instead of the current\n                        directory.\n\n--keep                  Keep the quality trimmed intermediate file. Default: off, which means\n                        the temporary file is being deleted after adapter trimming. Only has\n                        an effect for RRBS samples since other FastQ files are not trimmed\n                        for poor qualities separately.\n\n\nGeneral options:\n\n-q/--quality <INT>      Trim low-quality ends from reads in addition to adapter removal. For\n                        RRBS samples, quality trimming will be performed first, and adapter\n                        trimming is carried in a second round. Other files are quality and adapter\n                        trimmed in a single pass. The algorithm is the same as the one used by BWA\n                        (Subtract INT from all qualities; compute partial sums from all indices\n                        to the end of the sequence; cut sequence at the index at which the sum is\n                        minimal). Default Phred score: 20.\n\n--phred33               Instructs Cutadapt to use ASCII+33 quality scores as Phred scores\n                        (Sanger/Illumina 1.9+ encoding) for quality trimming. Default: ON.\n\n--phred64               Instructs Cutadapt to use ASCII+64 quality scores as Phred scores\n                        (Illumina 1.5 encoding) for quality trimming.\n\n-a/--adapter <STRING>   Adapter sequence to be trimmed. If not specified explicitly, Trim Galore will\n                        try to auto-detect whether the Illumina universal, Nextera transposase or Illumina\n                        small RNA adapter sequence was used. Also see '--illumina', '--nextera' and\n                        '--small_rna'. If no adapter can be detected within the first 1 million sequences\n                        of the first file specified Trim Galore defaults to '--illumina'.\n\n-a2/--adapter2 <STRING> Optional adapter sequence to be trimmed off read 2 of paired-end files. This\n                        option requires '--paired' to be specified as well. If the libraries to be trimmed\n                        are smallRNA then a2 will be set to the Illumina small RNA 5' adapter automatically\n                        (GATCGTCGGACT).\n\n--illumina              Adapter sequence to be trimmed is the first 13bp of the Illumina universal adapter\n                        'AGATCGGAAGAGC' instead of the default auto-detection of adapter sequence.\n\n--nextera               Adapter sequence to be trimmed is the first 12bp of the Nextera adapter\n                        'CTGTCTCTTATA' instead of the default auto-detection of adapter sequence.\n\n--small_rna             Adapter sequence to be trimmed is the first 12bp of the Illumina Small RNA 3' Adapter\n                        'TGGAATTCTCGG' instead of the default auto-detection of adapter sequence. Selecting\n                        to trim smallRNA adapters will also lower the --length value to 18bp. If the smallRNA\n                        libraries are paired-end then a2 will be set to the Illumina small RNA 5' adapter\n                        automatically (GATCGTCGGACT) unless -a 2 had been defined explicitly.\n\n--max_length <INT>      Discard reads that are longer than <INT> bp after trimming. This is only advised for\n                        smallRNA sequencing to remove non-small RNA sequences.\n\n\n--stringency <INT>      Overlap with adapter sequence required to trim a sequence. Defaults to a\n                        very stringent setting of 1, i.e. even a single bp of overlapping sequence\n                        will be trimmed off from the 3' end of any read.\n\n-e <ERROR RATE>         Maximum allowed error rate (no. of errors divided by the length of the matching\n                        region) (default: 0.1)\n\n--gzip                  Compress the output file with GZIP. If the input files are GZIP-compressed\n                        the output files will automatically be GZIP compressed as well. As of v0.2.8 the\n                        compression will take place on the fly.\n\n--dont_gzip             Output files won't be compressed with GZIP. This option overrides --gzip.\n\n--length <INT>          Discard reads that became shorter than length INT because of either\n                        quality or adapter trimming. A value of '0' effectively disables\n                        this behaviour. Default: 20 bp.\n\n                        For paired-end files, both reads of a read-pair need to be longer than\n                        <INT> bp to be printed out to validated paired-end files (see option --paired).\n                        If only one read became too short there is the possibility of keeping such\n                        unpaired single-end reads (see --retain_unpaired). Default pair-cutoff: 20 bp.\n\n--max_n COUNT           The total number of Ns (as integer) a read may contain before it will be removed altogether.\n                        In a paired-end setting, either read exceeding this limit will result in the entire\n                        pair being removed from the trimmed output files.\n\n--trim-n                Removes Ns from either side of the read. This option does currently not work in RRBS mode.\n\n--no_report_file        If specified no report file will be generated.\n\n--suppress_warn         If specified any output to STDOUT or STDERR will be suppressed.\n\n--clip_R1 <int>         Instructs Trim Galore to remove <int> bp from the 5' end of read 1 (or single-end\n                        reads). This may be useful if the qualities were very poor, or if there is some\n                        sort of unwanted bias at the 5' end. Default: OFF.\n\n--clip_R2 <int>         Instructs Trim Galore to remove <int> bp from the 5' end of read 2 (paired-end reads\n                        only). This may be useful if the qualities were very poor, or if there is some sort\n                        of unwanted bias at the 5' end. For paired-end BS-Seq, it is recommended to remove\n                        the first few bp because the end-repair reaction may introduce a bias towards low\n                        methylation. Please refer to the M-bias plot section in the Bismark User Guide for\n                        some examples. Default: OFF.\n\n--three_prime_clip_R1 <int>     Instructs Trim Galore to remove <int> bp from the 3' end of read 1 (or single-end\n                        reads) AFTER adapter/quality trimming has been performed. This may remove some unwanted\n                        bias from the 3' end that is not directly related to adapter sequence or basecall quality.\n                        Default: OFF.\n\n--three_prime_clip_R2 <int>     Instructs Trim Galore to remove <int> bp from the 3' end of read 2 AFTER\n                        adapter/quality trimming has been performed. This may remove some unwanted bias from\n                        the 3' end that is not directly related to adapter sequence or basecall quality.\n                        Default: OFF.\n\n--path_to_cutadapt </path/to/cutadapt>     You may use this option to specify a path to the Cutadapt executable,\n                        e.g. /my/home/cutadapt-1.7.1/bin/cutadapt. Else it is assumed that Cutadapt is in\n                        the PATH.\n\n\nRRBS-specific options (MspI digested material):\n\n--rrbs                  Specifies that the input file was an MspI digested RRBS sample (recognition\n                        site: CCGG). Single-end or Read 1 sequences (paired-end) which were adapter-trimmed\n                        will have a further 2 bp removed from their 3' end. Sequences which were merely\n                        trimmed because of poor quality will not be shortened further. Read 2 of paired-end\n                        libraries will in addition have the first 2 bp removed from the 5' end (by setting\n                        '--clip_r2 2'). This is to avoid using artificial methylation calls from the filled-in\n                        cytosine positions close to the 3' MspI site in sequenced fragments.\n                        This option is not recommended for users of the NuGEN ovation RRBS System 1-16\n                        kit (see below).\n\n--non_directional       Selecting this option for non-directional RRBS libraries will screen\n                        quality-trimmed sequences for 'CAA' or 'CGA' at the start of the read\n                        and, if found, removes the first two basepairs. Like with the option\n                        '--rrbs' this avoids using cytosine positions that were filled-in\n                        during the end-repair step. '--non_directional' requires '--rrbs' to\n                        be specified as well. Note that this option does not set '--clip_r2 2' in\n                        paired-end mode.\n\n\nNote for RRBS using the NuGEN Ovation RRBS System 1-16 kit:\n\nOwing to the fact that the NuGEN Ovation kit attaches a varying number of nucleotides (0-3) after each MspI\nsite Trim Galore should be run WITHOUT the option --rrbs. This trimming is accomplished in a subsequent\ndiversity trimming step afterwards (see their manual).\n\n\nNote for RRBS using MseI:\n\nIf your DNA material was digested with MseI (recognition motif: TTAA) instead of MspI it is NOT necessary\nto specify --rrbs or --non_directional since virtually all reads should start with the sequence\n'TAA', and this holds true for both directional and non-directional libraries. As the end-repair of 'TAA'\nrestricted sites does not involve any cytosines it does not need to be treated especially. Instead, simply\nrun Trim Galore! in the standard (i.e. non-RRBS) mode.\n\n\nPaired-end specific options:\n\n--paired                This option performs length trimming of quality/adapter/RRBS trimmed reads for\n                        paired-end files. To pass the validation test, both sequences of a sequence pair\n                        are required to have a certain minimum length which is governed by the option\n                        --length (see above). If only one read passes this length threshold the\n                        other read can be rescued (see option --retain_unpaired). Using this option lets\n                        you discard too short read pairs without disturbing the sequence-by-sequence order\n                        of FastQ files which is required by many aligners.\n\n                        Trim Galore! expects paired-end files to be supplied in a pairwise fashion, e.g.\n                        file1_1.fq file1_2.fq SRR2_1.fq.gz SRR2_2.fq.gz ... .\n\n-t/--trim1              Trims 1 bp off every read from its 3' end. This may be needed for FastQ files that\n                        are to be aligned as paired-end data with Bowtie. This is because Bowtie (1) regards\n                        alignments like this:\n\n                          R1 --------------------------->     or this:    ----------------------->  R1\n                          R2 <---------------------------                       <-----------------  R2\n\n                        as invalid (whenever a start/end coordinate is contained within the other read).\n                        NOTE: If you are planning to use Bowtie2, BWA etc. you don't need to specify this option.\n\n--retain_unpaired       If only one of the two paired-end reads became too short, the longer\n                        read will be written to either '.unpaired_1.fq' or '.unpaired_2.fq'\n                        output files. The length cutoff for unpaired single-end reads is\n                        governed by the parameters -r1/--length_1 and -r2/--length_2. Default: OFF.\n\n-r1/--length_1 <INT>    Unpaired single-end read length cutoff needed for read 1 to be written to\n                        '.unpaired_1.fq' output file. These reads may be mapped in single-end mode.\n                        Default: 35 bp.\n\n-r2/--length_2 <INT>    Unpaired single-end read length cutoff needed for read 2 to be written to\n                        '.unpaired_2.fq' output file. These reads may be mapped in single-end mode.\n                        Default: 35 bp."
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var default_output_filename = function() { var basename = inputs.bedgraph_file.location.split('/').slice(-1)[0]; var root = basename.split('.').slice(0,-1).join('.'); var ext = \".bigWig\"; return (root == \"\")?basename+ext:root+ext; };"
                    ]
                }
            ],
            "hints": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "biowardrobe2/ucscuserapps:v358"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10
                    },
                    "doc": "Four column bedGraph file: <chrom> <start> <end> <value>\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/bedgraph_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "separate": false,
                        "position": 7,
                        "prefix": "-blockSize="
                    },
                    "doc": "Number of items to bundle in r-tree.  Default 256\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/block_size"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 11
                    },
                    "doc": "Two-column chromosome length file: <chromosome name> <size in bases>\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/chrom_length_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "separate": false,
                        "position": 6,
                        "prefix": "-itemsPerSlot="
                    },
                    "doc": "Number of data points bundled at lowest level. Default 1024\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/items_per_slot"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 12,
                        "valueFrom": "${\n    if (self == \"\"){\n      return default_output_filename();\n    } else {\n      return self;\n    }\n}\n"
                    },
                    "default": "",
                    "doc": "If set, writes the output bigWig file to output_filename,\notherwise generates filename from default_output_filename()\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/output_filename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "-unc"
                    },
                    "doc": "Disable compression\n",
                    "id": "#ucsc-bedgraphtobigwig.cwl/unc"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "${\n    if (inputs.output_filename == \"\"){\n      return default_output_filename();\n    } else {\n      return inputs.output_filename;\n    }\n}\n"
                    },
                    "id": "#ucsc-bedgraphtobigwig.cwl/bigwig_file"
                }
            ],
            "baseCommand": [
                "bedGraphToBigWig"
            ],
            "doc": "Tool converts bedGraph to bigWig file.\n\n`default_output_filename` function returns filename for generated bigWig if `output_filename` is not provided.\nDefault filename is generated on the base of `bedgraph_file` basename with the updated to `*.bigWig` extension.\n",
            "id": "#ucsc-bedgraphtobigwig.cwl",
            "http://schema.org/mainEntity": {
                "class": "http://schema.org/SoftwareSourceCode",
                "id": "#ucsc-metadata.yaml",
                "http://schema.org/name": "UCSC userApps",
                "http://schema.org/about": "UCSC genome browser bioinformatic utilities\n",
                "http://schema.org/url": "https://genome.ucsc.edu/util.html",
                "http://schema.org/targetProduct": {
                    "class": "http://schema.org/SoftwareApplication",
                    "http://schema.org/softwareVersion": "v358",
                    "http://schema.org/applicationCategory": "commandline tool",
                    "http://schema.org/downloadURL": "http://hgdownload.cse.ucsc.edu/admin/exe/userApps.v358.src.tgz"
                },
                "http://schema.org/programmingLanguage": "C++",
                "http://schema.org/license": [
                    "https://opensource.org/licenses/GPL-3.0"
                ]
            },
            "http://schema.org/name": "ucsc-bedgraphtobigwig",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/Barski-lab/workflows/master/tools/ucsc-bedgraphtobigwig.cwl",
            "http://schema.org/codeRepository": "https://github.com/Barski-lab/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Andrey Kartashov",
                                            "http://schema.org/email": "mailto:Andrey.Kartashov@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0001-9102-5681"
                                                }
                                            ]
                                        },
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:misha.kotliar@gmail.com",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ],
            "http://schema.org/about": "usage:\n   bedGraphToBigWig in.bedGraph chrom.sizes out.bw\nwhere in.bedGraph is a four column file in the format:\n      <chrom> <start> <end> <value>\nand chrom.sizes is a two-column file/URL: <chromosome name> <size in bases>\nand out.bw is the output indexed big wig file.\nIf the assembly <db> is hosted by UCSC, chrom.sizes can be a URL like\n  http://hgdownload.cse.ucsc.edu/goldenPath/<db>/bigZips/<db>.chrom.sizes\nor you may use the script fetchChromSizes to download the chrom.sizes file.\nIf not hosted by UCSC, a chrom.sizes file can be generated by running\ntwoBitInfo on the assembly .2bit file.\nThe input bedGraph file must be sorted, use the unix sort command:\n  sort -k1,1 -k2,2n unsorted.bedGraph > sorted.bedGraph\noptions:\n   -blockSize=N - Number of items to bundle in r-tree.  Default 256\n   -itemsPerSlot=N - Number of data points bundled at lowest level. Default 1024\n   -unc - If set, do not use compression.\n"
        },
        {
            "class": "Workflow",
            "requirements": [
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "var get_root = function(basename) { return basename.split('.').slice(0,1).join('.'); };"
                    ]
                }
            ],
            "sd:metadata": [
                "../metadata/chipseq-header.cwl"
            ],
            "sd:upstream": {
                "genome_indices": "genome-indices.cwl",
                "control_file": "trim-atacseq-se.cwl"
            },
            "inputs": [
                {
                    "type": "File",
                    "sd:upstreamSource": "genome_indices/annotation",
                    "label": "Annotation file",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Tab-separated annotation file",
                    "id": "#main/annotation_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": false,
                    "label": "Callpeak broad",
                    "doc": "Set to call broad peak for MACS2",
                    "id": "#main/broad_peak"
                },
                {
                    "type": "File",
                    "sd:upstreamSource": "genome_indices/chrom_length",
                    "label": "Chromosomes length file",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "Chromosomes length file",
                    "id": "#main/chrom_length"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 0,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Clip from 3p end",
                    "doc": "Number of bases to clip from the 3p end",
                    "id": "#main/clip_3p_end"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 0,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Clip from 5p end",
                    "doc": "Number of bases to clip from the 5p end",
                    "id": "#main/clip_5p_end"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "default": null,
                    "sd:upstreamSource": "control_file/bambai_pair",
                    "sd:localLabel": true,
                    "label": "Use experiment as a control",
                    "format": "http://edamontology.org/format_2572",
                    "doc": "Use experiment as a control for MACS2 peak calling",
                    "id": "#main/control_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "default": "chrM",
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Exclude chromosomes",
                    "doc": "Space separated list of chromosomes to be excluded",
                    "id": "#main/exclude_chromosome"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 150,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Expected fragment size",
                    "doc": "Expected fragment size for MACS2",
                    "id": "#main/exp_fragment_size"
                },
                {
                    "type": "File",
                    "label": "FASTQ input file",
                    "format": "http://edamontology.org/format_1930",
                    "doc": "Reads data in a FASTQ format, received after single end sequencing",
                    "id": "#main/fastq_file"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": false,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Force fragment size",
                    "doc": "Force MACS2 to use exp_fragment_size",
                    "id": "#main/force_fragment_size"
                },
                {
                    "type": "string",
                    "sd:upstreamSource": "genome_indices/genome_size",
                    "label": "Effective genome size",
                    "doc": "MACS2 effective genome size: hs, mm, ce, dm or number, for example 2.7e9",
                    "id": "#main/genome_size"
                },
                {
                    "type": "Directory",
                    "sd:upstreamSource": "genome_indices/bowtie_indices",
                    "label": "Indexed genome folder (bowtie)",
                    "doc": "Path to indexed genome folder by **bowtie**",
                    "id": "#main/indices_folder"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 1000,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Max distance from gene TSS (in both direction) overlapping which the peak will be assigned to the promoter region",
                    "doc": "Max distance from gene TSS (in both direction) overlapping which the peak will be assigned to the promoter region",
                    "id": "#main/promoter_dist"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "default": false,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Remove duplicates",
                    "doc": "Calls samtools rmdup to remove duplicates from sortesd BAM file",
                    "id": "#main/remove_duplicates"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 2,
                    "sd:layout": {
                        "advanced": true
                    },
                    "doc": "Number of threads for those steps that support multithreading",
                    "label": "Number of threads",
                    "id": "#main/threads"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 20000,
                    "sd:layout": {
                        "advanced": true
                    },
                    "label": "Max distance from the promoter (only in upstream direction) overlapping which the peak will be assigned to the upstream region",
                    "doc": "Max distance from the promoter (only in upstream direction) overlapping which the peak will be assigned to the upstream region",
                    "id": "#main/upstream_dist"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "label": "ATDP log",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Average Tag Density generated log",
                    "outputSource": "#main/average_tag_density/log_file",
                    "id": "#main/atdp_log"
                },
                {
                    "type": "File",
                    "label": "ATDP results",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Average Tag Density generated results",
                    "outputSource": "#main/average_tag_density/result_file",
                    "sd:visualPlugins": [
                        {
                            "scatter": {
                                "tab": "QC Plots",
                                "Title": "Average Tag Density",
                                "xAxisTitle": "Distance From TSS (bases)",
                                "yAxisTitle": "Average Tag Density (per bp)",
                                "colors": [
                                    "#b3de69"
                                ],
                                "height": 500,
                                "data": [
                                    "$1",
                                    "$2"
                                ],
                                "comparable": "atdp"
                            }
                        }
                    ],
                    "id": "#main/atdp_result"
                },
                {
                    "type": "File",
                    "label": "BAM statistics report (original)",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "BAM statistics report (right after alignment and sorting)",
                    "outputSource": "#main/get_bam_statistics/log_file",
                    "id": "#main/bam_statistics_report"
                },
                {
                    "type": "File",
                    "label": "BAM statistics report (after filtering)",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "BAM statistics report (after all filters applied)",
                    "outputSource": "#main/get_bam_statistics_after_filtering/log_file",
                    "id": "#main/bam_statistics_report_after_filtering"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_2572",
                    "label": "Coordinate sorted BAM alignment file (+index BAI)",
                    "doc": "Coordinate sorted BAM file and BAI index file",
                    "outputSource": "#main/samtools_sort_index_after_rmdup/bam_bai_pair",
                    "sd:visualPlugins": [
                        {
                            "igvbrowser": {
                                "tab": "IGV Genome Browser",
                                "id": "#main/bambai_pair/igvbrowser",
                                "optional": true,
                                "type": "alignment",
                                "format": "#main/bambai_pair/bam",
                                "name": "#main/bambai_pair/BAM Track",
                                "displayMode": "SQUISHED"
                            }
                        }
                    ],
                    "id": "#main/bambai_pair"
                },
                {
                    "type": "File",
                    "format": "http://edamontology.org/format_3006",
                    "label": "BigWig file",
                    "doc": "Generated BigWig file",
                    "outputSource": "#main/bam_to_bigwig/bigwig_file",
                    "sd:visualPlugins": [
                        {
                            "igvbrowser": {
                                "tab": "IGV Genome Browser",
                                "id": "#main/bigwig/igvbrowser",
                                "type": "wig",
                                "name": "#main/bigwig/BigWig Track",
                                "height": 120
                            }
                        }
                    ],
                    "id": "#main/bigwig"
                },
                {
                    "type": "File",
                    "label": "BOWTIE alignment log",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "BOWTIE generated alignment log",
                    "outputSource": "#main/bowtie_aligner/log_file",
                    "id": "#main/bowtie_log"
                },
                {
                    "type": "int",
                    "label": "Estimated fragment size",
                    "doc": "Estimated fragment size for downstream analyses",
                    "outputSource": "#main/macs2_callpeak/macs2_fragments_calculated",
                    "id": "#main/estimated_fragment_size"
                },
                {
                    "type": "File",
                    "label": "FASTQ statistics",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "fastx_quality_stats generated FASTQ file quality statistics file",
                    "outputSource": "#main/fastx_quality_stats/statistics_file",
                    "sd:visualPlugins": [
                        {
                            "line": {
                                "tab": "QC Plots",
                                "Title": "Base frequency plot",
                                "xAxisTitle": "Nucleotide position",
                                "yAxisTitle": "Frequency",
                                "colors": [
                                    "#b3de69",
                                    "#888888",
                                    "#fb8072",
                                    "#fdc381",
                                    "#99c0db"
                                ],
                                "data": [
                                    "$13",
                                    "$14",
                                    "$15",
                                    "$16",
                                    "$17"
                                ]
                            }
                        },
                        {
                            "boxplot": {
                                "tab": "QC Plots",
                                "Title": "Quality Control",
                                "xAxisTitle": "Nucleotide position",
                                "yAxisTitle": "Quality score",
                                "colors": [
                                    "#b3de69",
                                    "#888888",
                                    "#fb8072",
                                    "#fdc381",
                                    "#99c0db"
                                ],
                                "data": [
                                    "$11",
                                    "$7",
                                    "$8",
                                    "$9",
                                    "$12"
                                ]
                            }
                        }
                    ],
                    "id": "#main/fastx_statistics"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Bowtie & Samtools Rmdup combined formatted log",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Processed and combined Bowtie aligner and Samtools rmdup formatted log",
                    "outputSource": "#main/get_stat/collected_statistics_tsv",
                    "sd:visualPlugins": [
                        {
                            "tableView": {
                                "vertical": true,
                                "tab": "Overview"
                            }
                        }
                    ],
                    "sd:preview": {
                        "sd:visualPlugins": [
                            {
                                "pie": {
                                    "colors": [
                                        "#b3de69",
                                        "#99c0db",
                                        "#fb8072",
                                        "#fdc381"
                                    ],
                                    "data": [
                                        "$2",
                                        "$3",
                                        "$4",
                                        "$5"
                                    ]
                                }
                            }
                        ]
                    },
                    "id": "#main/get_stat_formatted_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "YAML formatted combined log",
                    "format": "http://edamontology.org/format_3750",
                    "doc": "YAML formatted combined log",
                    "outputSource": "#main/get_stat/collected_statistics_yaml",
                    "id": "#main/get_stat_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Markdown formatted combined log",
                    "format": "http://edamontology.org/format_3835",
                    "doc": "Markdown formatted combined log",
                    "outputSource": "#main/get_stat/collected_statistics_md",
                    "sd:visualPlugins": [
                        {
                            "markdownView": {
                                "tab": "Overview"
                            }
                        }
                    ],
                    "id": "#main/get_stat_markdown"
                },
                {
                    "type": "File",
                    "label": "Island intersect log",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Iaintersect generated log",
                    "outputSource": "#main/island_intersect/log_file",
                    "id": "#main/iaintersect_log"
                },
                {
                    "type": "File",
                    "label": "Island intersect results",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Iaintersect generated results",
                    "outputSource": "#main/island_intersect/result_file",
                    "sd:visualPlugins": [
                        {
                            "syncfusiongrid": {
                                "tab": "Peak Calling",
                                "Title": "Islands list"
                            }
                        }
                    ],
                    "id": "#main/iaintersect_result"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Broad peaks",
                    "format": "http://edamontology.org/format_3614",
                    "doc": "Contains the peak locations together with peak summit, pvalue and qvalue",
                    "outputSource": "#main/macs2_callpeak/broad_peak_file",
                    "sd:visualPlugins": [
                        {
                            "igvbrowser": {
                                "tab": "IGV Genome Browser",
                                "id": "#main/macs2_broad_peaks/igvbrowser",
                                "type": "annotation",
                                "name": "#main/macs2_broad_peaks/Broad peaks",
                                "displayMode": "COLLAPSE",
                                "height": 40
                            }
                        }
                    ],
                    "id": "#main/macs2_broad_peaks"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Called peaks",
                    "format": "http://edamontology.org/format_3468",
                    "doc": "XLS file to include information about called peaks",
                    "outputSource": "#main/macs2_callpeak/peak_xls_file",
                    "id": "#main/macs2_called_peaks"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "FRAGMENT, FRAGMENTE, ISLANDS",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "fragment, calculated fragment, islands count from MACS2 results",
                    "outputSource": "#main/macs2_callpeak/macs2_stat_file",
                    "id": "#main/macs2_fragment_stat"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Gapped peaks",
                    "format": "http://edamontology.org/format_3586",
                    "doc": "Contains both the broad region and narrow peaks",
                    "outputSource": "#main/macs2_callpeak/gapped_peak_file",
                    "sd:visualPlugins": [
                        {
                            "igvbrowser": {
                                "tab": "IGV Genome Browser",
                                "id": "#main/macs2_gapped_peak/igvbrowser",
                                "type": "annotation",
                                "name": "#main/macs2_gapped_peak/Gapped peaks",
                                "displayMode": "COLLAPSE",
                                "height": 40
                            }
                        }
                    ],
                    "id": "#main/macs2_gapped_peak"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "MACS2 log",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "MACS2 output log",
                    "outputSource": "#main/macs2_callpeak/macs_log",
                    "id": "#main/macs2_log"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "MACS2 generated R script",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "R script to produce a PDF image about the model based on your data",
                    "outputSource": "#main/macs2_callpeak/moder_r_file",
                    "id": "#main/macs2_moder_r"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Narrow peaks",
                    "format": "http://edamontology.org/format_3613",
                    "doc": "Contains the peak locations together with peak summit, pvalue and qvalue",
                    "outputSource": "#main/macs2_callpeak/narrow_peak_file",
                    "sd:visualPlugins": [
                        {
                            "igvbrowser": {
                                "tab": "IGV Genome Browser",
                                "id": "#main/macs2_narrow_peaks/igvbrowser",
                                "type": "annotation",
                                "name": "#main/macs2_narrow_peaks/Narrow peaks",
                                "displayMode": "COLLAPSE",
                                "height": 40
                            }
                        }
                    ],
                    "id": "#main/macs2_narrow_peaks"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Peak summits",
                    "format": "http://edamontology.org/format_3003",
                    "doc": "Contains the peak summits locations for every peaks",
                    "outputSource": "#main/macs2_callpeak/peak_summits_file",
                    "id": "#main/macs2_peak_summits"
                },
                {
                    "type": "int",
                    "label": "Mapped reads number",
                    "doc": "Mapped reads number for downstream analyses",
                    "outputSource": "#main/get_stat/mapped_reads",
                    "id": "#main/mapped_reads_number"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "format": "http://edamontology.org/format_1930",
                    "label": "Multimapped FASTQ file(s)",
                    "doc": "Multimapped FASTQ file(s)",
                    "outputSource": "#main/bowtie_aligner/multimapped_fastq",
                    "id": "#main/multimapped_fastq"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "label": "Preseq estimates",
                    "format": "http://edamontology.org/format_3475",
                    "doc": "Preseq estimated results",
                    "outputSource": "#main/preseq/estimates_file",
                    "sd:visualPlugins": [
                        {
                            "scatter": {
                                "tab": "QC Plots",
                                "Title": "Preseq Estimates",
                                "xAxisTitle": "Total reads count",
                                "yAxisTitle": "Expected distinct reads count",
                                "colors": [
                                    "#4b78a3"
                                ],
                                "height": 500,
                                "data": [
                                    "$1",
                                    "$2"
                                ],
                                "comparable": "preseq"
                            }
                        }
                    ],
                    "id": "#main/preseq_estimates"
                },
                {
                    "type": "File",
                    "label": "Remove duplicates log",
                    "format": "http://edamontology.org/format_2330",
                    "doc": "Samtools rmdup generated log",
                    "outputSource": "#main/samtools_rmdup/rmdup_log",
                    "id": "#main/samtools_rmdup_log"
                },
                {
                    "type": "File",
                    "label": "TrimGalore report",
                    "doc": "TrimGalore generated log",
                    "outputSource": "#main/trim_fastq/report_file",
                    "id": "#main/trim_report"
                },
                {
                    "type": [
                        "null",
                        {
                            "type": "array",
                            "items": "File"
                        }
                    ],
                    "format": "http://edamontology.org/format_1930",
                    "label": "Unaligned FASTQ file(s)",
                    "doc": "Unaligned FASTQ file(s)",
                    "outputSource": "#main/bowtie_aligner/unaligned_fastq",
                    "id": "#main/unaligned_fastq"
                }
            ],
            "steps": [
                {
                    "label": "Read enrichment around genes TSS",
                    "doc": "Generates average tag density plot around genes TSS as a lot of cis-regulatory\nelements are close to the TSS of their targets.\n",
                    "run": "#atdp.cwl",
                    "in": [
                        {
                            "source": "#main/annotation_file",
                            "id": "#main/average_tag_density/annotation_filename"
                        },
                        {
                            "default": 200,
                            "id": "#main/average_tag_density/avd_heat_window_bp"
                        },
                        {
                            "default": 50,
                            "id": "#main/average_tag_density/avd_smooth_bp"
                        },
                        {
                            "default": 5000,
                            "id": "#main/average_tag_density/avd_window_bp"
                        },
                        {
                            "default": "chrX chrY",
                            "id": "#main/average_tag_density/double_chr"
                        },
                        {
                            "source": "#main/macs2_callpeak/macs2_fragments_calculated",
                            "id": "#main/average_tag_density/fragmentsize_bp"
                        },
                        {
                            "default": "chrM",
                            "id": "#main/average_tag_density/ignore_chr"
                        },
                        {
                            "source": "#main/samtools_sort_index_after_rmdup/bam_bai_pair",
                            "id": "#main/average_tag_density/input_file"
                        },
                        {
                            "source": "#main/get_stat/mapped_reads",
                            "id": "#main/average_tag_density/mapped_reads"
                        }
                    ],
                    "out": [
                        "#main/average_tag_density/result_file",
                        "#main/average_tag_density/log_file"
                    ],
                    "id": "#main/average_tag_density"
                },
                {
                    "run": "#bam-bedgraph-bigwig.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index_after_rmdup/bam_bai_pair",
                            "id": "#main/bam_to_bigwig/bam_file"
                        },
                        {
                            "source": "#main/chrom_length",
                            "id": "#main/bam_to_bigwig/chrom_length_file"
                        },
                        {
                            "default": 9,
                            "id": "#main/bam_to_bigwig/fragment_size"
                        },
                        {
                            "source": "#main/get_stat/mapped_reads",
                            "id": "#main/bam_to_bigwig/mapped_reads_number"
                        }
                    ],
                    "out": [
                        "#main/bam_to_bigwig/bigwig_file"
                    ],
                    "id": "#main/bam_to_bigwig"
                },
                {
                    "label": "Alignment to reference genome",
                    "doc": "Aligns reads to the reference genome keeping only uniquely mapped reads with\nless than 3 mismatches.  \n",
                    "run": "#bowtie-alignreads.cwl",
                    "in": [
                        {
                            "default": 500,
                            "id": "#main/bowtie_aligner/X"
                        },
                        {
                            "default": true,
                            "id": "#main/bowtie_aligner/best"
                        },
                        {
                            "source": "#main/clip_3p_end",
                            "id": "#main/bowtie_aligner/clip_3p_end"
                        },
                        {
                            "source": "#main/clip_5p_end",
                            "id": "#main/bowtie_aligner/clip_5p_end"
                        },
                        {
                            "source": "#main/indices_folder",
                            "id": "#main/bowtie_aligner/indices_folder"
                        },
                        {
                            "default": 1,
                            "id": "#main/bowtie_aligner/m"
                        },
                        {
                            "default": "multimapped_reads",
                            "id": "#main/bowtie_aligner/multimapped_prefix"
                        },
                        {
                            "default": true,
                            "id": "#main/bowtie_aligner/q"
                        },
                        {
                            "default": true,
                            "id": "#main/bowtie_aligner/sam"
                        },
                        {
                            "default": true,
                            "id": "#main/bowtie_aligner/strata"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/bowtie_aligner/threads"
                        },
                        {
                            "default": "unaligned_reads",
                            "id": "#main/bowtie_aligner/unaligned_prefix"
                        },
                        {
                            "source": "#main/rename/target_file",
                            "id": "#main/bowtie_aligner/upstream_filelist"
                        },
                        {
                            "default": 3,
                            "id": "#main/bowtie_aligner/v"
                        }
                    ],
                    "out": [
                        "#main/bowtie_aligner/sam_file",
                        "#main/bowtie_aligner/log_file",
                        "#main/bowtie_aligner/unaligned_fastq",
                        "#main/bowtie_aligner/multimapped_fastq"
                    ],
                    "id": "#main/bowtie_aligner"
                },
                {
                    "run": "#bypass-trimgalore-se.cwl",
                    "in": [
                        {
                            "default": 100,
                            "id": "#main/bypass_trim/min_reads_count"
                        },
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "id": "#main/bypass_trim/original_fastq_file"
                        },
                        {
                            "source": "#main/trim_fastq/trimmed_file",
                            "id": "#main/bypass_trim/trimmed_fastq_file"
                        },
                        {
                            "source": "#main/trim_fastq/report_file",
                            "id": "#main/bypass_trim/trimming_report_file"
                        }
                    ],
                    "out": [
                        "#main/bypass_trim/selected_fastq_file",
                        "#main/bypass_trim/selected_report_file"
                    ],
                    "id": "#main/bypass_trim"
                },
                {
                    "label": "Loading unmapped sequence data",
                    "doc": "Most DNA cores and commercial NGS companies return unmapped sequence data in FASTQ format.\nThe data can be uploaded from users computer, downloaded directly from an ftp server of\nthe core facility by providing a URL or from GEO by providing SRA accession number.  \n",
                    "run": "#extract-fastq.cwl",
                    "in": [
                        {
                            "source": "#main/fastq_file",
                            "id": "#main/extract_fastq/compressed_file"
                        }
                    ],
                    "out": [
                        "#main/extract_fastq/fastq_file"
                    ],
                    "id": "#main/extract_fastq"
                },
                {
                    "label": "Quality control of unmapped sequence data",
                    "doc": "Evaluates the quality of your sequence data. Provides per base quality scores as well as\nbase frequencies along the reads. These metrics can be used to identify whether your data\nhas any problems that should be taken into account in the subsequent analysis steps.  \n",
                    "run": "#fastx-quality-stats.cwl",
                    "in": [
                        {
                            "source": "#main/rename/target_file",
                            "id": "#main/fastx_quality_stats/input_file"
                        }
                    ],
                    "out": [
                        "#main/fastx_quality_stats/statistics_file"
                    ],
                    "id": "#main/fastx_quality_stats"
                },
                {
                    "run": "#samtools-filter.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index/bam_bai_pair",
                            "id": "#main/filter_bam/bam_bai_pair"
                        },
                        {
                            "source": "#main/exclude_chromosome",
                            "id": "#main/filter_bam/exclude_chromosome"
                        }
                    ],
                    "out": [
                        "#main/filter_bam/filtered_bam_bai_pair"
                    ],
                    "id": "#main/filter_bam"
                },
                {
                    "label": "Quality control of aligned sequence data",
                    "doc": "Calculates alignment statistics, such as reads mapped/unmapped, average\nread length and quality score, etc.  \n",
                    "run": "#samtools-stats.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index/bam_bai_pair",
                            "id": "#main/get_bam_statistics/bambai_pair"
                        },
                        {
                            "source": "#main/samtools_sort_index/bam_bai_pair",
                            "valueFrom": "$(get_root(self.basename)+\"_bam_statistics_report.txt\")",
                            "id": "#main/get_bam_statistics/output_filename"
                        }
                    ],
                    "out": [
                        "#main/get_bam_statistics/log_file"
                    ],
                    "id": "#main/get_bam_statistics"
                },
                {
                    "run": "#samtools-stats.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_sort_index_after_rmdup/bam_bai_pair",
                            "id": "#main/get_bam_statistics_after_filtering/bambai_pair"
                        },
                        {
                            "source": "#main/samtools_sort_index_after_rmdup/bam_bai_pair",
                            "valueFrom": "$(get_root(self.basename)+\"_bam_statistics_report_after_filtering.txt\")",
                            "id": "#main/get_bam_statistics_after_filtering/output_filename"
                        }
                    ],
                    "out": [
                        "#main/get_bam_statistics_after_filtering/log_file"
                    ],
                    "id": "#main/get_bam_statistics_after_filtering"
                },
                {
                    "run": "#collect-statistics-chip-seq.cwl",
                    "in": [
                        {
                            "source": "#main/get_bam_statistics_after_filtering/log_file",
                            "id": "#main/get_stat/bam_statistics_after_filtering_report"
                        },
                        {
                            "source": "#main/get_bam_statistics/log_file",
                            "id": "#main/get_stat/bam_statistics_report"
                        },
                        {
                            "source": "#main/bowtie_aligner/log_file",
                            "id": "#main/get_stat/bowtie_alignment_report"
                        },
                        {
                            "source": "#main/macs2_callpeak/peak_xls_file",
                            "id": "#main/get_stat/macs2_called_peaks"
                        },
                        {
                            "source": "#main/preseq/estimates_file",
                            "id": "#main/get_stat/preseq_results"
                        },
                        {
                            "source": "#main/bypass_trim/selected_report_file",
                            "id": "#main/get_stat/trimgalore_report_fastq_1"
                        }
                    ],
                    "out": [
                        "#main/get_stat/collected_statistics_yaml",
                        "#main/get_stat/collected_statistics_tsv",
                        "#main/get_stat/mapped_reads",
                        "#main/get_stat/collected_statistics_md"
                    ],
                    "id": "#main/get_stat"
                },
                {
                    "label": "Peak annotation",
                    "doc": "Assigns nearest genes to peaks to explore the biological implication of the open\nchromatin binding sites.\n",
                    "run": "#iaintersect.cwl",
                    "in": [
                        {
                            "source": "#main/annotation_file",
                            "id": "#main/island_intersect/annotation_filename"
                        },
                        {
                            "source": "#main/macs2_callpeak/peak_xls_file",
                            "id": "#main/island_intersect/input_filename"
                        },
                        {
                            "source": "#main/promoter_dist",
                            "id": "#main/island_intersect/promoter_bp"
                        },
                        {
                            "source": "#main/upstream_dist",
                            "id": "#main/island_intersect/upstream_bp"
                        }
                    ],
                    "out": [
                        "#main/island_intersect/result_file",
                        "#main/island_intersect/log_file"
                    ],
                    "id": "#main/island_intersect"
                },
                {
                    "label": "Peak detection",
                    "doc": "Identifies enriched with aligned reads genome areas. Those areas correspond to the\ntranscription factor binding sites.  \n",
                    "run": "#macs2-callpeak-biowardrobe-only.cwl",
                    "in": [
                        {
                            "source": "#main/broad_peak",
                            "id": "#main/macs2_callpeak/broad"
                        },
                        {
                            "default": 10000,
                            "id": "#main/macs2_callpeak/buffer_size"
                        },
                        {
                            "source": "#main/exp_fragment_size",
                            "id": "#main/macs2_callpeak/bw"
                        },
                        {
                            "source": "#main/broad_peak",
                            "valueFrom": "$(!self)",
                            "id": "#main/macs2_callpeak/call_summits"
                        },
                        {
                            "source": "#main/control_file",
                            "id": "#main/macs2_callpeak/control_file"
                        },
                        {
                            "source": "#main/exp_fragment_size",
                            "id": "#main/macs2_callpeak/extsize"
                        },
                        {
                            "default": "BAM",
                            "id": "#main/macs2_callpeak/format_mode"
                        },
                        {
                            "source": "#main/genome_size",
                            "id": "#main/macs2_callpeak/genome_size"
                        },
                        {
                            "default": "auto",
                            "id": "#main/macs2_callpeak/keep_dup"
                        },
                        {
                            "default": "4 40",
                            "id": "#main/macs2_callpeak/mfold"
                        },
                        {
                            "source": "#main/control_file",
                            "valueFrom": "$(!self)",
                            "id": "#main/macs2_callpeak/nolambda"
                        },
                        {
                            "source": "#main/force_fragment_size",
                            "id": "#main/macs2_callpeak/nomodel"
                        },
                        {
                            "default": 0.05,
                            "id": "#main/macs2_callpeak/q_value"
                        },
                        {
                            "source": "#main/samtools_sort_index_after_rmdup/bam_bai_pair",
                            "id": "#main/macs2_callpeak/treatment_file"
                        },
                        {
                            "default": 3,
                            "id": "#main/macs2_callpeak/verbose"
                        }
                    ],
                    "out": [
                        "#main/macs2_callpeak/peak_xls_file",
                        "#main/macs2_callpeak/narrow_peak_file",
                        "#main/macs2_callpeak/peak_summits_file",
                        "#main/macs2_callpeak/broad_peak_file",
                        "#main/macs2_callpeak/moder_r_file",
                        "#main/macs2_callpeak/gapped_peak_file",
                        "#main/macs2_callpeak/treat_pileup_bdg_file",
                        "#main/macs2_callpeak/control_lambda_bdg_file",
                        "#main/macs2_callpeak/macs_log",
                        "#main/macs2_callpeak/macs2_stat_file",
                        "#main/macs2_callpeak/macs2_fragments_calculated"
                    ],
                    "id": "#main/macs2_callpeak"
                },
                {
                    "label": "Sequencing depth estimation",
                    "doc": "Estimates the complexity of the sequencing library, evaluates how many reads can\nbe expected from the additional sequencing of the same experiment.  \n",
                    "run": "#preseq-lc-extrap.cwl",
                    "in": [
                        {
                            "source": "#main/filter_bam/filtered_bam_bai_pair",
                            "id": "#main/preseq/bam_file"
                        },
                        {
                            "default": 1000000000,
                            "id": "#main/preseq/extrapolation"
                        }
                    ],
                    "out": [
                        "#main/preseq/estimates_file"
                    ],
                    "id": "#main/preseq"
                },
                {
                    "run": "#rename.cwl",
                    "in": [
                        {
                            "source": "#main/bypass_trim/selected_fastq_file",
                            "id": "#main/rename/source_file"
                        },
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "valueFrom": "$(self.basename)",
                            "id": "#main/rename/target_filename"
                        }
                    ],
                    "out": [
                        "#main/rename/target_file"
                    ],
                    "id": "#main/rename"
                },
                {
                    "label": "PCR duplicates removal",
                    "doc": "Removes potential PCR duplicates. This step is used to remove reads overamplified\nin PCR. Unfortunately, it may also remove \"good\" reads. We do not recommend to\nremove duplicates unless the library is heavily duplicated.  \n",
                    "run": "#samtools-rmdup.cwl",
                    "in": [
                        {
                            "source": "#main/filter_bam/filtered_bam_bai_pair",
                            "id": "#main/samtools_rmdup/bam_file"
                        },
                        {
                            "default": true,
                            "id": "#main/samtools_rmdup/single_end"
                        },
                        {
                            "source": "#main/remove_duplicates",
                            "id": "#main/samtools_rmdup/trigger"
                        }
                    ],
                    "out": [
                        "#main/samtools_rmdup/rmdup_output",
                        "#main/samtools_rmdup/rmdup_log"
                    ],
                    "id": "#main/samtools_rmdup"
                },
                {
                    "run": "#samtools-sort-index.cwl",
                    "in": [
                        {
                            "source": "#main/bowtie_aligner/sam_file",
                            "id": "#main/samtools_sort_index/sort_input"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/samtools_sort_index/threads"
                        }
                    ],
                    "out": [
                        "#main/samtools_sort_index/bam_bai_pair"
                    ],
                    "id": "#main/samtools_sort_index"
                },
                {
                    "run": "#samtools-sort-index.cwl",
                    "in": [
                        {
                            "source": "#main/samtools_rmdup/rmdup_output",
                            "id": "#main/samtools_sort_index_after_rmdup/sort_input"
                        },
                        {
                            "source": "#main/threads",
                            "id": "#main/samtools_sort_index_after_rmdup/threads"
                        },
                        {
                            "source": "#main/remove_duplicates",
                            "id": "#main/samtools_sort_index_after_rmdup/trigger"
                        }
                    ],
                    "out": [
                        "#main/samtools_sort_index_after_rmdup/bam_bai_pair"
                    ],
                    "id": "#main/samtools_sort_index_after_rmdup"
                },
                {
                    "label": "Adapter trimming",
                    "doc": "For libraries sequenced on the Illumina platform it\u2019s recommended to remove adapter sequences\nfrom the reads. If adapters are not trimmed there is a high risk of reads being unmapped to a\nreference genome. This becomes particularly important when the reads are long and the fragments\nare short - resulting in sequencing adapters at the end of read. If adapter trimming will cause\nall the reads become too short (<30bp), this step will be skipped.  \n",
                    "run": "#trimgalore.cwl",
                    "in": [
                        {
                            "default": true,
                            "id": "#main/trim_fastq/dont_gzip"
                        },
                        {
                            "source": "#main/extract_fastq/fastq_file",
                            "id": "#main/trim_fastq/input_file"
                        },
                        {
                            "default": 30,
                            "id": "#main/trim_fastq/length"
                        }
                    ],
                    "out": [
                        "#main/trim_fastq/trimmed_file",
                        "#main/trim_fastq/report_file"
                    ],
                    "id": "#main/trim_fastq"
                }
            ],
            "label": "Trim Galore ATAC-Seq pipeline single-read",
            "doc": "This ATAC pipeline is based on original [BioWardrobe's](https://biowardrobe.com) [PubMed ID:26248465](https://www.ncbi.nlm.nih.gov/pubmed/26248465)\n**ChIP-Seq** basic analysis workflow for a **single-read** experiment with Trim Galore. The pipeline was adapted for\nATAC-Seq single-read data analysis by updating genome coverage step.\n### Data Analysis Steps\nSciDAP starts from the .fastq files which most DNA cores and commercial NGS companies return. Starting from raw data allows us to ensure that all experiments have been processed in the same way and simplifies the deposition of data to GEO upon publication. The data can be uploaded from users computer, downloaded directly from an ftp server of the core facility by providing a URL or from GEO by providing SRA accession number.\nOur current pipelines include the following steps:\n1. Trimming the adapters with TrimGalore. This step is particularly important when the reads are long and the fragments are short as in ATAC -resulting in sequencing adapters at the end of read. If adapter is not removed the read will not map. TrimGalore can recognize standard adapters, such as Nexterra/Tn5 adapters.\n2. QC\n3. (Optional) trimming adapters on 5' or 3' end by the specified number of bases.\n4. Mapping reads with BowTie. Only uniquely mapped reads with less than 3 mismatches are used in the downstream analysis. Results are saved as a .bam file.\n5. Reads mapping to chromosome M are removed. Since there are many copies of chromosome M in the cell and it is not protected by histones, some ATAC libraries have up to 50% of reads mapping to chrM. We recommend using OMNI-ATAC protocol that reduces chrM reads and provides better specificity. \n6.  (Optional) Removal of duplicates (reads/pairs of reads mapping to exactly same location). This step is used to remove reads overamplified in PCR. Unfortunately, it may also remove \"good\" reads. We usually do not remove duplicates unless the library is heavily duplicated. Please note that MACS2 will remove 'excessive' duplicates during peak calling ina smart way (those not supported by other nearby reads).\n7.  Peakcalling by MACS2. (Optionally), it is possible to specify read extension length for MACS2 to use if the length determined automatically is wrong. \n8.  Generation of BigWig coverage files for display on the browser. Since the cuts by the Tn5 transposome are 9bp apart, we show coverage by 9bp reads rather than fragments as in ChIP-Seq. The coverage shows the number of fragments at each base in the genome normalized to the number of millions of mapped reads. This way the peak of coverage will be located at the most accessible site. \n\n### Details\n\n_Trim Galore_ is a wrapper around [Cutadapt](https://github.com/marcelm/cutadapt)\nand [FastQC](http://www.bioinformatics.babraham.ac.uk/projects/fastqc/) to consistently\napply adapter and quality trimming to FastQ files, with extra functionality for RRBS data.\n\nIn outputs it returns coordinate sorted BAM file alongside with index BAI file, quality\nstatistics of the input FASTQ file, reads coverage in a form of BigWig file, peaks calling\ndata in a form of narrowPeak or broadPeak files, islands with the assigned nearest genes and\nregion type, data for average tag density plot (on the base of BAM file).\n\nWorkflow starts with step *fastx\\_quality\\_stats* from FASTX-Toolkit\nto calculate quality statistics for input FASTQ file.\n\nAt the same time `bowtie` is used to align\nreads from input FASTQ file to reference genome *bowtie\\_aligner*. The output of this step\nis unsorted SAM file which is being sorted and indexed by `samtools sort` and `samtools index`\n*samtools\\_sort\\_index*.\n\nBased on workflow\u2019s input parameters indexed and sorted BAM file\ncan be processed by `samtools rmdup` *samtools\\_rmdup* to get rid of duplicated reads.\nIf removing duplicates is not required the original input BAM and BAI\nfiles return. Otherwise step *samtools\\_sort\\_index\\_after\\_rmdup* repeat `samtools sort` and `samtools index` with BAM and BAI files.\n\nRight after that `macs2 callpeak` performs peak calling *macs2\\_callpeak*. On the base of returned outputs the next step\n*macs2\\_island\\_count* calculates the number of islands and estimated fragment size. If the last\none is less that 80bp (hardcoded in the workflow) `macs2 callpeak` is rerun again with forced fixed\nfragment size value (*macs2\\_callpeak\\_forced*). If at the very beginning it was set in workflow\ninput parameters to force run peak calling with fixed fragment size, this step is skipped and the\noriginal peak calling results are saved.\n\nIn the next step workflow again calculates the number of islands and estimates fragment size (*macs2\\_island\\_count\\_forced*)\nfor the data obtained from *macs2\\_callpeak\\_forced* step. If the last one was skipped the results from *macs2\\_island\\_count\\_forced* step\nare equal to the ones obtained from *macs2\\_island\\_count* step.\n\nNext step (*macs2\\_stat*) is used to define which of the islands and estimated fragment size should be used\nin workflow output: either from *macs2\\_island\\_count* step or from *macs2\\_island\\_count\\_forced* step. If input\ntrigger of this step is set to True it means that *macs2\\_callpeak\\_forced* step was run and it returned different\nfrom *macs2\\_callpeak* step results, so *macs2\\_stat* step should return [fragments\\_new, fragments\\_old, islands\\_new],\nif trigger is False the step returns [fragments\\_old, fragments\\_old, islands\\_old], where sufix \"old\" defines\nresults obtained from *macs2\\_island\\_count* step and sufix \"new\" - from *macs2\\_island\\_count\\_forced* step.\n\nThe following two steps (*bamtools\\_stats* and *bam\\_to\\_bigwig*) are used to calculate coverage on the base\nof input BAM file and save it in BigWig format. For that purpose bamtools stats returns the number of\nmapped reads number which is then used as scaling factor by bedtools genomecov when it performs coverage\ncalculation and saves it in BED format. The last one is then being sorted and converted to BigWig format by\nbedGraphToBigWig tool from UCSC utilities. To adapt the pipeline for ATAC-Seq data analysis we calculate genome\ncoverage using only the first 9 bp from every read.\n\nStep *get\\_stat* is used to return a text file with statistics\nin a form of [TOTAL, ALIGNED, SUPRESSED, USED] reads count.\n\nStep *island\\_intersect* assigns genes and regions to the islands obtained from *macs2\\_callpeak\\_forced*.\nStep *average\\_tag\\_density* is used to calculate data for average tag density plot on the base of BAM file.",
            "id": "#main",
            "http://schema.org/name": "Trim Galore ATAC-Seq pipeline single-read",
            "http://schema.org/alternateName": "ATAC-Seq basic analysis workflow for a single-read experiment with Trim Galore",
            "http://schema.org/downloadUrl": "https://raw.githubusercontent.com/datirium/workflows/master/workflows/trim-atacseq-se.cwl",
            "http://schema.org/codeRepository": "https://github.com/datirium/workflows",
            "http://schema.org/license": "http://www.apache.org/licenses/LICENSE-2.0",
            "http://schema.org/isPartOf": {
                "class": "http://schema.org/CreativeWork",
                "http://schema.org/name": "Common Workflow Language",
                "http://schema.org/url": "http://commonwl.org/"
            },
            "http://schema.org/creator": [
                {
                    "class": "http://schema.org/Organization",
                    "http://schema.org/legalName": "Cincinnati Children's Hospital Medical Center",
                    "http://schema.org/location": [
                        {
                            "class": "http://schema.org/PostalAddress",
                            "http://schema.org/addressCountry": "USA",
                            "http://schema.org/addressLocality": "Cincinnati",
                            "http://schema.org/addressRegion": "OH",
                            "http://schema.org/postalCode": "45229",
                            "http://schema.org/streetAddress": "3333 Burnet Ave",
                            "http://schema.org/telephone": "+1(513)636-4200"
                        }
                    ],
                    "http://schema.org/logo": "https://www.cincinnatichildrens.org/-/media/cincinnati%20childrens/global%20shared/childrens-logo-new.png",
                    "http://schema.org/department": [
                        {
                            "class": "http://schema.org/Organization",
                            "http://schema.org/legalName": "Allergy and Immunology",
                            "http://schema.org/department": [
                                {
                                    "class": "http://schema.org/Organization",
                                    "http://schema.org/legalName": "Barski Research Lab",
                                    "http://schema.org/member": [
                                        {
                                            "class": "http://schema.org/Person",
                                            "http://schema.org/name": "Michael Kotliar",
                                            "http://schema.org/email": "mailto:michael.kotliar@cchmc.org",
                                            "http://schema.org/sameAs": [
                                                {
                                                    "$import": "#0000-0002-6486-3898"
                                                }
                                            ]
                                        }
                                    ]
                                }
                            ]
                        }
                    ]
                }
            ]
        }
    ],
    "cwlVersion": "v1.0",
    "$schemas": [
        "https://github.com/schemaorg/schemaorg/raw/main/data/releases/11.01/schemaorg-current-http.rdf"
    ],
    "$namespaces": {
        "s": "http://schema.org/"
    }
}
